

class CDFOutput
{
public:
	CDFOutput() 
	: 	in_string(false), in_italic(false), in_super(false), in_bold(false),
		depth(0), indent(0), compact(0), no_space(false), 
	  	fout(0) {}
	bool is_open() { return fout != 0; }
	void open(FILE *f)
	{	if (fout != 0)
			close();
		fout = f;
		depth = 0;
		indent = 0;
		compact = 0;
		no_space = false;
	}
	void close()
	{	end_string();
		fclose(fout);
		fout = 0;
	}

	void write_raw_string(const char *str, bool italic = false, bool super = false, bool bold = false)
	{	// Trust our caller that it does not contain '}' character
		if (fout == 0) return;
		start_string(italic, super, bold);
		fprintf(fout, "%s", str);
		no_space = true;
	}
	void write_string(const char *str, bool italic = false, bool super = false, bool bold = false)
	{
		if (fout == 0) return;
		start_string(italic, super, bold);
		for (; *str != '\0'; str++)
		{	unsigned char ch = *str;
			if (' ' <= ch && ch < 127 && ch != '{' && ch != '#')
				fprintf(fout, "%c", ch);
			else
				fprintf(fout, "#%02X", (unsigned char)ch);
		}
		no_space = true;
	}
	void write_string_len(const char *str, int len, bool italic = false, bool super = false, bool bold = false)
	{
		if (fout == 0) return;
		start_string(italic, super, bold);
		for (int i = 0; i < len; i++)
		{	unsigned char ch = str[i];
			if (' ' <= ch && ch < 127 && ch != '{' && ch != '#')
				fprintf(fout, "%c", ch);
			else
				fprintf(fout, "#%02X", (unsigned char)ch);
		}
		no_space = true;
	}
	void write_char(char ch, bool italic = false, bool super = false, bool bold = false)
	{
		if (fout == 0) return;
		start_string(italic, super, bold);
		if (' ' <= ch && ch < 127 && ch != '{' && ch != '#')
			fprintf(fout, "%c", ch);
		else
			fprintf(fout, "#%02X", (unsigned char)ch);
	}
	void write_ident(const char *str)
	{
		if (fout == 0) return;
		new_element();
		fprintf(fout, "%s", str);
		no_space = true;
	}
	void write_bool(bool b)
	{
		if (fout == 0) return;
		new_element();
		fprintf(fout, "%s", b ? "true" : "false");
		no_space = true;
	}
	void write_number(long l)
	{
		if (fout == 0) return;
		new_element();
		fprintf(fout, "%ld", l);
		no_space = true;
	}
		
	void open_list(const char *name = 0)
	{	
		if (fout == 0) return;
		new_element();
		push();
		fprintf(fout, "%s[", name ? name : "");
		indent += 4;
		no_space = false;
	}
	void open_compact_list(const char *name = 0)
	{
		if (fout == 0) return;
		end_string();
		open_list(name);
		set_compact();
	}
	void close_list()
	{
		if (fout == 0) return;
		end_string();
		pop();
		fprintf(fout, "]");
		indent -= 4;
		no_space = true;
		
	}
	void open_record(const char *name = 0)
	{	
		if (fout == 0) return;
		new_element();
		push();
		fprintf(fout, "%s(", name ? name : "");
		indent += 2;
		no_space = false;
	}
	void open_compact_record(const char *name = 0)
	{	
		if (fout == 0) return;
		end_string();
		open_record(name);
		set_compact();
	}
	void close_record()
	{	
		if (fout == 0) return;
		end_string();
		pop();
		fprintf(fout, ")");
		indent -= 2;
		no_space = true;
	}
	void record_element(const char *name = 0)
	{	
		if (fout == 0) return;
		new_element();
		if (name != 0)
			fprintf(fout, "%s=", name);
		after_name = true;
		no_space = false;
	}
	
	void set_compact()
	{	
		if (fout == 0) return;
		if (depth > 0 && !states[depth-1].compact)
		{	states[depth-1].compact = true;
			compact++;
		}
	}

private:
	void push()
	{	states[depth].compact = 0;
		depth++;
	}
	void pop()
	{
		if (depth > 0)
		{	depth--;
			if (states[depth].compact)
				compact--;
		}
	}
	void new_element()
	{	
		end_string();
		if (depth == 0 || compact > 0 || after_name)
		{	
			if (no_space)
			{	fprintf(fout, " ");
				no_space = false;
			}
			after_name = false;
		}
		else
		{
			fprintf(fout, "\n");
			int i;
			for (i = indent; i >= 4; i-=4)
				fprintf(fout, "\t");
			for (; i > 0; i--)
				fprintf(fout, " ");
		}
	}
	void start_string(bool italic, bool super, bool bold)
	{
		if (bold && super)
			bold = false;
		if (!in_string)
			new_element();
		if (italic != in_italic || super != in_super || bold != in_bold)
		{
			if (in_string) { fprintf(fout, "}"); in_string = false; }
			if (in_bold) { fprintf(fout, ")"); in_bold = false; }
			if (in_super) { fprintf(fout, ")"); in_super = false; }
			if (in_italic && !italic) { fprintf(fout, ")"); in_italic = false; }
			if (italic && !in_italic) {	fprintf(fout, "I("); in_italic = true; }
			if (super && !in_super) { fprintf(fout, "P("); in_super = true; }
			if (bold && !in_bold) { fprintf(fout, "B("); in_bold = true; }
		}
		if (!in_string) { fprintf(fout, "{"); in_string = true; }
	}
	void end_string()
	{	
		if (in_string) { fprintf(fout, "}"); in_string = false; }
		if (in_bold)  { fprintf(fout, ")"); in_bold = false; }
		if (in_super)  { fprintf(fout, ")"); in_super = false; }
		if (in_italic) { fprintf(fout, ")"); in_italic = false; }
	}
	bool in_string;
	bool in_italic;
	bool in_super;
	bool in_bold;
	struct 
	{	bool compact;
	} states[100];
	int depth;
	int indent;
	int compact;
	bool no_space;
	bool after_name;
	FILE *fout;
};

