#ifdef TEST
#include <stdio.h>
#include <malloc.h>
#include <ctype.h>
#include <string.h>
#include "stddef.c"
#include "greek.cpp"
#endif
#include "textacc.cpp"
#include "CDFparser.cpp"

const byte gt_empty = 0;
const byte gt_header = 1;
const byte gt_col1 = 2;
const byte gt_col2 = 3;
const byte gt_col3 = 4;
const byte gt_col4 = 5;
const byte gt_col1_vn = 6;
const byte gt_col2_vn = 7;
const byte gt_col3_vn = 8;
const byte gt_col4_vn = 9;
const byte gt_peri_title = 10;
const byte gt_normal = 11;

#define MODE_B    (1<<0)
#define MODE_I    (1<<1)
#define MODE_P	  (1<<5) // !!
#define MODE_CHAR (MODE_B|MODE_I|MODE_P)
#define MODE_S	  (1<<2)
#define MODE_S1   (1<<3)
#define MODE_S2	  (1<<4)
#define MODE_G    (1<<6)
#define MODE_H    (1<<7)
#define MODE_X    (1<<8)

class CFrameAccessor
{
	friend class CTextOnFramesAccessor;
public:
	void Open(const char *bname)
	{	more = false;
		book_name = bname;
		char longbname[100];
		sprintf(longbname, "sbr\\%s.sbr", bname);
		
		if (!file_text_acc.open(longbname))
		{	printf("Could not open %s\n", longbname);
			return;
		}
		if (!file_text_acc.more)
		{	printf("File %s is empty\n", longbname);
			return;
		}
		parser.open(&file_text_acc);
		if (parser.state != CDFparser::st_list_open && *parser.name != '\0')
		{	more = false;
			return;
		}
		frame_depth = parser.current_depth();
		parser.next();
		Next();
	}
	bool More()
	{	return more;
	}
	void Next()
	{	
		more = false;
		if (parser.state >= CDFparser::st_end) 
			return;
		if (   parser.state != CDFparser::st_list_open 
			|| parser.current_depth() > frame_depth+1)
			parser.skip_till_depth(frame_depth);

		if (   parser.state != CDFparser::st_list_open
		    || parser.current_depth() != frame_depth+1)
			return;

		more = true;
		if (streq("col1", parser.name))
			cur_geom_type = gt_col1;
		else if (streq("col2", parser.name))
			cur_geom_type = gt_col2;
		else if (streq("col3", parser.name))
			cur_geom_type = gt_col3;
		else if (streq("col4", parser.name))
			cur_geom_type = gt_col4;
		else if (streq("vn1", parser.name))
			cur_geom_type = gt_col1_vn;
		else if (streq("vn2", parser.name))
			cur_geom_type = gt_col2_vn;
		else if (streq("vn3", parser.name))
			cur_geom_type = gt_col3_vn;
		else if (streq("vn4", parser.name))
			cur_geom_type = gt_col4_vn;
		else if (streq("peri", parser.name))
			cur_geom_type = gt_peri_title;
		else if (streq("norm", parser.name))
			cur_geom_type = gt_normal;
		else
		{	printf("unrecognized frame type '%s'\n", parser.name);
			cur_geom_type = gt_normal;
		}
		parser.next();
		read_paragraph();
	}
	inline byte geom_type()
	{	return cur_geom_type;
	}
	bool TextIsNumber()
	{	char *s = par_text; 
		for (; *s == ' '; s++);
		for (; isdigit(*s); s++);
		for (; *s == ' '; s++);
		return *s == '\0';
	}
	int NumberValueText()
	{	int v = 0;
		for (char *s = par_text; *s != '\0'; s++)
			if (isdigit(*s))
				v = 10*v + *s - '0';
		return v;
	}
	void PrintContext(char *context)
	{	printf("\n%s,%ld:%ld %s: ", book_name, parser.linenr, parser.columnnr, context);
		
	}
	void ReportError()
	{
		if (parser.state == CDFparser::st_error)
		{	printf("\n%s,%ld:%ld: Fatal(I): syntax error in input file: %s",
				   book_name, parser.linenr, parser.columnnr, parser.error_text);
		}
	}
	const char* book_name;
	
private:
	bool more;
	FileTextAccessor file_text_acc;
	CDFparser parser;
	int frame_depth;
	byte cur_geom_type;
	bool more_par;
	char par_text[10000];
	char par_mode[10000];
	char par_name[100];

	void read_paragraph()
	{
		par_text[0] = '\0';
		
		if (parser.state != CDFparser::st_list_open)
		{	more_par = false;
			return;
		}
		more_par = true;
		strcpy(par_name, parser.name);
		
		int i = 0;
		int mode = 0;
	
		parser.next();
		while (   parser.state < CDFparser::st_end 
		       && parser.state != CDFparser::st_list_close)	
			if (!read_par_text(i, mode))
			{	PrintContext("");
				printf("Error: %d", parser.state);
				break;
			}
		par_text[i] = '\0';
		if (parser.state == CDFparser::st_list_close)
			parser.next();
	}
	bool read_par_text(int &i, int mode)
	{
		if (parser.state == CDFparser::st_in_string)
		{
			while (parser.state == CDFparser::st_in_string)
			{	if (i < 9999)
				{	par_text[i] = parser.string_char;
					par_mode[i] = mode;
					i++;
				}
				parser.next();
			}
			if (parser.state != CDFparser::st_string_end)
			{	// some error
				return false;
			}
			else
				parser.next();
		}
		else if (parser.state == CDFparser::st_list_open)
		{	if (streq(parser.name, "G"))
			{
				mode |= MODE_G;
				parser.next();
				
				char greek[100];
				int j = 0;
				while (parser.state == CDFparser::st_in_string)
				{	if (parser.as_hex)
					{	if (j < 97)
						{	sprintf(&greek[j], "#%02X", (byte)parser.string_char);
							j += 3;
						}
					}
					else
					{	if (j < 99)
							greek[j++] = parser.string_char;
					}
					parser.next();
				}
				greek[j] = '\0';
				//printf("\nGreek |%s|", greek);
				byte as_hex[1000];
				char *s = text_to_greek(greek, as_hex);
				for (; *s != '\0'; s++)	
				{	if (i < 9999)
					{	par_text[i] = *s;
						//printf(" %02X", (byte)par_text[i]);
						par_mode[i] = mode | (as_hex[i] ? MODE_X : 0);
						i++;
					}
				}
				//printf("|");
				if (parser.state == CDFparser::st_string_end)
					parser.next();
			}
			else
			{
				if (streq(parser.name, "B"))
					mode |= MODE_B;
				else if (streq(parser.name, "I"))
					mode |= MODE_I;
				else if (streq(parser.name, "P"))
					mode |= MODE_P;
				else if (streq(parser.name, "H"))
					mode |= MODE_H;
					
				parser.next();
				while (   parser.state < CDFparser::st_end 
					   && parser.state != CDFparser::st_list_close)	
					read_par_text(i, mode);
			}
			if (parser.state == CDFparser::st_list_close)
				parser.next();
		}
		else if (parser.state == CDFparser::st_record_open)
		{ 
			if (streq(parser.name, "t"))
			{	if (i < 9999)
				{	par_text[i] = 9;
					par_mode[i] = mode;
					i++;
				}
				parser.next();
			}
			else
			{
				if (streq(parser.name, "S"))
					mode |= MODE_S;
				else if (streq(parser.name, "S1"))
					mode |= MODE_S1;
				else if (streq(parser.name, "S2"))
					mode |= MODE_S2;
				
				parser.next();
				if (parser.state == CDFparser::st_int)
				{	if (i < 9999)
					{	par_text[i] = parser.int_val;
						par_mode[i] = mode;
						i++;
					}
					parser.next();
				}	
			}				
			if (parser.state == CDFparser::st_record_close)
				parser.next();
		}
		else
			return false;
		return true;
	}
};

class CTextOnFramesAccessor
{
  public:
	CTextOnFramesAccessor(CFrameAccessor &new_frame_acc) 
	{	frame_acc = &new_frame_acc;
		par_text = frame_acc->par_text;
		par_mode = frame_acc->par_mode;
		pos = 0;
		in_new_frame = false;
	}	
	void Init()
	{	more = frame_acc->more_par;
		if (!more)
			return;
		pos = 0;
		is_newline = false;
		ch = par_text[0];
		if (ch == '\0')
		{	ch = '\n';
			is_newline = true;
		}
		else if ((par_mode[0] & MODE_G) != MODE_G && (ch == 0xCA || ch == 255))
		    ch = ' ';
		is_tab = ch == 9;
		in_new_par_style = true;
		set_par_style();
		in_new_mode = true;
		mode = par_mode[0] & MODE_CHAR;
		set_font();
	}
	inline bool More()
	{	return more;
	}
	void Next()
	{	in_new_par_style = false;
		if (is_newline)
		{	is_newline = false;
			frame_acc->read_paragraph();
			more = frame_acc->more_par;
			if (!more)
				return;
			pos = 0;
			ch = par_text[pos];
			in_new_par_style = false;
			set_par_style();
			in_new_mode = true;
			mode = par_mode[0] & MODE_CHAR;
		}
		else
		{	pos++;
			ch = par_text[pos];
			word new_mode = par_mode[pos] & MODE_CHAR;
			in_new_mode = new_mode != mode;
			mode = new_mode;
		}
		set_font();
		if (ch == '\0')
		{	ch = '\n';
			is_newline = true;
		}
		else if (font_family != 'G' && (ch == 0xCA || ch == 255))
		    ch = ' ';
		is_tab = ch == 9;
	}
	void SkipSpaces()
	{	for (; More(); Next())
			if (ch != ' ')
				break;
	}
	void SkipSpacesAndControl()
	{	for (; More(); Next())
			if (ch > ' ')
				break;
	}
	bool ParseNumber(word &nr)
	{
		bool found_digit = FALSE;
		nr = 0;
	
		for (; More(); Next())
		{	if (ch < '0' || ch > '9')
				break;
			found_digit = TRUE;
			nr = nr * 10 + ch - '0';
		}
	
		return found_digit;
	}
	bool ParseChar(const char this_ch)
	{
		if (More() && ch == this_ch)
		{	Next();
			return TRUE;
		}
		else
			return FALSE;
	}
	bool ParseLiteral(const char *s)
	{
		if (!StartsWith(s))
			return FALSE;
			
		for (; *s != '\0' &&  More(); Next(), s++);
		
		return TRUE;
	}
	bool StartsWith(const char *s)
	{
		// Look ahead in text buffer
		// WARNING: this can go wrong if the text runs over in the next frame!!!
		for (lword i = pos; *s != '\0' && i < end; i++, s++)
			if (par_text[i] != (byte)*s)
				return FALSE;
		return *s == '\0';
	}
	void Dump(int nr)
	{
		printf("Dump|");
		for (lword i = pos; i < end && i < pos+nr && par_text[i] != '\0'; i++)
		{	char ch = par_text[i];
			printf(' ' <= ch && ch < 127 ? "%c" : "%02X", ch);
		}
		printf("|");
	}	
	void SkipTillNewline()
	{	for (; More(); Next())
			if (is_newline)
				break;
	}
	void SkipLine()
	{	SkipTillNewline();
		if (More() && is_newline)
			Next();
	}
	void SkipEmptyLines()
	{	while(More() && is_newline)
			Next();
	}
	bool SingleCharTab()
	{	return    More() && ch >= 'a' && ch <= 'z' 
			   && pos+1 < end && par_text[pos+1] == '\t';
	}
	
	byte ch;
	bool is_tab;
	bool is_newline;
	bool in_new_frame;
	const char *par_style;
	bool in_new_par_style;
	char font_family;
	bool in_new_mode;
	word size;
	word mode;

	// Use with care:
  	char *par_text;
  	char *par_mode;

  private:
	CFrameAccessor *frame_acc;
  	bool more;
  	lword start;
  	lword end;
  	lword pos;

	void set_par_style()
	{	char *new_par_style = "normal";
		if (streq(frame_acc->par_name, "number"))
			new_par_style = "Number";
		else if (streq(frame_acc->par_name, "grieks"))
			new_par_style = "Grieks";
		else if (streq(frame_acc->par_name, "trans"))
			new_par_style = "Trans";
		else if (streq(frame_acc->par_name, "vertaling"))
			new_par_style = "Vertaling";
		else if (streq(frame_acc->par_name, "a1"))
			new_par_style = "alinea1";
		else if (streq(frame_acc->par_name, "a2"))
			new_par_style = "alinea2";
		else if (streq(frame_acc->par_name, "a3"))
			new_par_style = "alinea3";
		else if (streq(frame_acc->par_name, "kopwoord"))
			new_par_style = "kopwoord";
		if (par_style != new_par_style)
			in_new_par_style = true;
		par_style = new_par_style;
	}
		
	void set_font()
	{	if (par_mode[pos] & MODE_S)
			font_family = 'S';
		else if (par_mode[pos] & MODE_S1)
			font_family = '1';
		else if (par_mode[pos] & MODE_S2)
			font_family = '2';
		else if (par_mode[pos] & MODE_G)
			font_family = 'G';
		else if (par_mode[pos] & MODE_H)
			font_family = 'H';
		else
			font_family = 'T';
	}
};





#ifdef TEST ///////////////////////////////////////////////////////////////////////

int depth;
void newline(FILE *fout)
{
	fprintf(fout, "\n");
	int d = depth;
	for (; d >= 4; d -= 4)
		fprintf(fout, "\t");
	for (; d > 0; d--)
		fprintf(fout, " ");	
}

#define IN_ITALIC  (1<<0)
#define IN_BOLD    (1<<1)
#define IN_SUP     (1<<2)
#define IN_GREEK   (1<<3)
#define IN_HELV    (1<<4)

bool equal_greek(const char *a, const char *b)
{
	for (;; a++, b++)
	{
		if (*a == '\0')
			return *b == '\0';
		if (*b == '\0')
			return false;
		if (   *b == (char)0xD8 
		    && (a[0] == 0x5C || a[0] == (char)0xD5) 
		    && (a[1] == 0x49 || a[1] == (char)0xB9))
			a++; // 'I
		else if (*a != *b && greek_char_code(*a) != greek_char_code(*b))
			return false;
	}
}	

void dump_greek(FILE* fout, const char *greek)
{
	char tb[1000];
	strcpy(tb, greek_to_text(greek));
	char sb[1000];
	strcpy(sb, text_to_greek(tb));
	if (!equal_greek(greek, sb))
	{	for (const char *s = greek; *s; s++) 
			printf("%02X ", (byte)*s);
		printf("|%s|", tb);
		for (const char *s = sb; *s; s++) 
			printf(" %02X", (byte)*s);
		printf("\n");
		fprintf(fout, "{");
		for (const char *s = greek; *s; s++) 
			fprintf(fout, "#%02X", (byte)*s);
		fprintf(fout, "}]");
	}
	else
		fprintf(fout, "{%s}]", tb);
}

void dump_par(FILE* fout, CTextOnFramesAccessor &text_acc, char* par_style)
{
	bool in_text = false;
	int cur_mode = 0;
	char greekbuf[100];
	char *gs = 0;
				
	for (; text_acc.More(); text_acc.Next())
	{
		if (text_acc.is_newline || text_acc.in_new_par_style)
			break;

		if (text_acc.ch < 5)
		{	// skip
		}
		else
		{
			int new_mode = 0;
			if (text_acc.mode & 1)
				new_mode |= IN_BOLD;
			if (text_acc.mode & 2)
				new_mode |= IN_ITALIC;
			if (text_acc.mode & 288)
				new_mode |= IN_SUP;
			if (text_acc.font_family == 'G')
				new_mode |= IN_GREEK;
			if (text_acc.font_family == 'H')
				new_mode |= IN_HELV;
			
			if (   (cur_mode & IN_GREEK)
				&& (   (cur_mode & (IN_ITALIC|IN_BOLD|IN_SUP)) != (new_mode & (IN_ITALIC|IN_BOLD|IN_SUP))
			        || (cur_mode & IN_GREEK) > (new_mode & IN_GREEK)))
			{	*gs = '\0';
				dump_greek(fout, greekbuf);
				cur_mode &= ~IN_GREEK;
			}
			if (   (cur_mode & IN_HELV)
				&& (   (cur_mode & (IN_ITALIC|IN_BOLD|IN_SUP)) != (new_mode & (IN_ITALIC|IN_BOLD|IN_SUP))
			        || (cur_mode & IN_HELV) > (new_mode & IN_HELV)))
			{	if (in_text) { fprintf(fout, "}"); in_text = false; }
				fprintf(fout, "]");
				cur_mode &= ~IN_HELV;
			}
			if (   (cur_mode & IN_SUP)
			    && (   (cur_mode & (IN_ITALIC|IN_BOLD)) != (new_mode & (IN_ITALIC|IN_BOLD))
			    	|| (cur_mode & IN_SUP) > (new_mode & IN_SUP)))
			{	if (in_text) { fprintf(fout, "}"); in_text = false; }
				fprintf(fout, "]");
				cur_mode &= ~IN_SUP;
			}
			if (   (cur_mode & IN_BOLD)
			    && (   (cur_mode & IN_ITALIC) != (new_mode & IN_ITALIC)
			    	|| (cur_mode & IN_BOLD) > (new_mode & IN_BOLD)))
			{	if (in_text) { fprintf(fout, "}"); in_text = false; }
				fprintf(fout, "]");
				cur_mode &= ~IN_BOLD;
			}
			if ((cur_mode & IN_ITALIC) > (new_mode & IN_ITALIC))
			{	if (in_text) { fprintf(fout, "}"); in_text = false; }
				fprintf(fout, "]");
				cur_mode &= ~IN_ITALIC;
			}
				
			if (text_acc.font_family == '1')
			{	if (in_text) { fprintf(fout, "}"); in_text = false; }
				fprintf(fout, "S1(%d)", text_acc.ch);
			}
			else if (text_acc.font_family == '2')
			{	if (in_text) { fprintf(fout, "}"); in_text = false; }
				fprintf(fout, "S2(%d)", text_acc.ch);
			}
			else if (text_acc.font_family == 'S')
			{	if (in_text) { fprintf(fout, "}"); in_text = false; }
				fprintf(fout, "S(%d)", text_acc.ch);
			}
			else if (text_acc.is_tab && (cur_mode & IN_GREEK) == 0)
			{	if (in_text) { fprintf(fout, "}"); in_text = false; }
				fprintf(fout, "t()");
			}
			else if (text_acc.ch == ' ' && (cur_mode & IN_GREEK) == 0)
			{
				if (!in_text) { fprintf(fout, "{"); in_text = true; }
				fprintf(fout, " ");
			}
			else
			{	
				if ((new_mode & IN_ITALIC) > (cur_mode & IN_ITALIC))
				{	if (in_text) { fprintf(fout, "}"); in_text = false; }
					fprintf(fout, "I[");
					cur_mode |= IN_ITALIC;
				}
				if ((new_mode & IN_BOLD) > (cur_mode & IN_BOLD))
				{	if (in_text) { fprintf(fout, "}"); in_text = false; }
					fprintf(fout, "B[");
					cur_mode |= IN_BOLD;
				}
				if ((new_mode & IN_SUP) > (cur_mode & IN_SUP))
				{	if (in_text) { fprintf(fout, "}"); in_text = false; }
					fprintf(fout, "P[");
					cur_mode |= IN_SUP;
				}
				if ((new_mode & IN_GREEK) > (cur_mode & IN_GREEK))
				{	if (in_text) { fprintf(fout, "}"); in_text = false; }
					fprintf(fout, "G[");
					cur_mode |= IN_GREEK;
					gs = greekbuf;
				}
				if ((new_mode & IN_HELV) > (cur_mode & IN_HELV))
				{	if (in_text) { fprintf(fout, "}"); in_text = false; }
					fprintf(fout, "H[");
					cur_mode |= IN_HELV;
				}
				
				
				if (cur_mode & IN_GREEK)
					*gs++ = text_acc.ch;
				else if (   text_acc.ch < ' ' || text_acc.ch > 127 
						 || text_acc.ch == '}' || text_acc.ch == '#')
				{	if (!in_text) { fprintf(fout, "{"); in_text = true; }
					fprintf(fout, "#%02X", text_acc.ch);
				}
				else
				{	if (!in_text) { fprintf(fout, "{"); in_text = true; }
					char ch = text_acc.ch;
					if (text_acc.mode & 3072)
						ch = _toupper(ch);
					fprintf(fout, "%c", ch);
				}
			}
		}
	}
	if (in_text) fprintf(fout, "}");
	if (cur_mode & IN_SUP) fprintf(fout, "]");
	if (cur_mode & IN_BOLD) fprintf(fout, "]");
	if (cur_mode & IN_ITALIC) fprintf(fout, "]");
	if (cur_mode & IN_GREEK) 
	{	*gs = '\0';
		dump_greek(fout, greekbuf);
	}
	if (cur_mode & IN_HELV) fprintf(fout, "]");
}

void dump_frame_par(FILE *fout, CTextOnFramesAccessor &text_acc)
{
	text_acc.in_new_par_style = false; // a hack!!
	
	char *par_style = "par";
	if (strcmp(text_acc.par_style, "Number") == 0)
		par_style = "number";
	else if (strcmp(text_acc.par_style, "Grieks") == 0)
		par_style = "grieks";
	else if (strcmp(text_acc.par_style, "Trans") == 0)
		par_style = "trans";
	else if (strcmp(text_acc.par_style, "Vertaling") == 0)
		par_style = "vertaling";
	else if (strcmp(text_acc.par_style, "alinea 1") == 0)
		par_style = "a1";
	else if (strcmp(text_acc.par_style, "alinea1") == 0)
		par_style = "a1";
	else if (strcmp(text_acc.par_style, "alinea2") == 0)
		par_style = "a2";
	else if (strcmp(text_acc.par_style, "alinea3") == 0)
		par_style = "a3";
	else if (strcmp(text_acc.par_style, "kopwoord") == 0)
		par_style = "kopwoord";

	bool do_newline = false;
	fprintf(fout, "%s[", par_style);
	depth += 4;

	for(;;)
	{
		if (do_newline)
		{	fprintf(fout, "]");
			depth -= 4;
			newline(fout);
			fprintf(fout, "%s[", par_style);
			depth += 4;
			do_newline = false;
		}
		dump_par(fout, text_acc, par_style);

		if (!text_acc.More() || text_acc.in_new_par_style)
			break;

		if (text_acc.is_newline)
		{	do_newline = true;
			text_acc.Next();
		}
				
		if (!text_acc.More() || text_acc.in_new_par_style)
			break;
	}
	fprintf(fout, "]");
	depth -= 4;
	
	if (text_acc.More())
		newline(fout);	
}

void dump_book(CFrameAccessor &frame_acc)
{
	char fn[200];
	sprintf(fn, "out/%s.txt", frame_acc.book_name);
	FILE *fout = fopen(fn, "wt");
	if (fout)
	{
		fprintf(fout, "[");
		depth++;

		for (; frame_acc.More(); frame_acc.Next())		
		{
			newline(fout);
			switch(frame_acc.geom_type())
			{	case gt_col1 : fprintf(fout, "col1"); break;
				case gt_col2 : fprintf(fout, "col2"); break;
				case gt_col3 : fprintf(fout, "col3"); break;
				case gt_col4 : fprintf(fout, "col4"); break;
				case gt_col1_vn : fprintf(fout, "vn1"); break;
				case gt_col2_vn : fprintf(fout, "vn2"); break;
				case gt_col3_vn : fprintf(fout, "vn3"); break;
				case gt_col4_vn : fprintf(fout, "vn4"); break;
				case gt_peri_title: fprintf(fout, "peri"); break;
				case gt_normal : fprintf(fout, "norm"); break;
				default : fprintf(fout, "other"); break;
			}
			fprintf(fout, "[");
			depth += 3;

			// Create a text accesor to parse the text with formatting.
			CTextOnFramesAccessor text_acc(frame_acc);
			text_acc.Init();
			while (text_acc.More())
				dump_frame_par(fout, text_acc);

			fprintf(fout, "]");
			depth -= 3;
		}
		fprintf(fout, "]");
		depth--;
		fclose(fout);
	}
	
}

int main(int argc, char *argv[])
{
	CFrameAccessor frame_acc;

	frame_acc.Open("DEEL02");
	dump_book(frame_acc);	
	frame_acc.Open("DEEL03");
	dump_book(frame_acc);	
	frame_acc.Open("DEEL04");
	dump_book(frame_acc);	
	frame_acc.Open("DEEL05");
	dump_book(frame_acc);	
	frame_acc.Open("DEEL06");
	dump_book(frame_acc);	
	frame_acc.Open("DEEL07A");
	dump_book(frame_acc);	
	frame_acc.Open("DEEL07B");
	dump_book(frame_acc);	
	frame_acc.Open("DEEL08");
	dump_book(frame_acc);	
	frame_acc.Open("DEEL09");
	dump_book(frame_acc);	
	frame_acc.Open("DEEL10");
	dump_book(frame_acc);	
	frame_acc.Open("DEEL11");
	dump_book(frame_acc);	
	frame_acc.Open("DEEL12");
	dump_book(frame_acc);	
	frame_acc.Open("DEEL13");
	dump_book(frame_acc);	
	frame_acc.Open("DEEL14");
	dump_book(frame_acc);	
	frame_acc.Open("DEEL15");
	dump_book(frame_acc);	
	frame_acc.Open("DEEL16");
	dump_book(frame_acc);	

	return 0;
}

#endif
