// This file describes all the database that is used to store the
// contenst of the Studiebijbel in a logical structure. The classes
// are given in a top-down order (as far as is possible)


// Generic class for a set of constant strings

class ConstStringSet
{
public:
	bool hasString(const char *string)
	{	FOREACH_IN_LIST(ConstStringSetElement, elem, contents)
			if (elem->string == string)
				return true;
		return false;
	}
	
	bool hasString(CMMFConstString2 string)
	{	FOREACH_IN_LIST(ConstStringSetElement, elem, contents)
			if (elem->string == string)
				return true;
		return false;
	}
	
	CMMFConstString2 add(const char *string)
	{	FOREACH_IN_LIST2_REF(ConstStringSetElement, r_elem, contents)
			if ((*r_elem)->string == string)
				return (*r_elem)->string;
				
		(*r_elem) = new ConstStringSetElement(string);
		return (*r_elem)->string;
	}

	CMMFConstString2 add(CMMFConstString2 string)
	{	FOREACH_IN_LIST2_REF(ConstStringSetElement, r_elem, contents)
			if ((*r_elem)->string == string)
				return (*r_elem)->string;
				
		(*r_elem) = new ConstStringSetElement(string);
		return (*r_elem)->string;
	}

	void extendWith(ConstStringSet set)
	{
		FOREACH_IN_LIST(ConstStringSetElement, elem, set.contents)
			add(elem->string);
	}
	
private:
  	CMMFPointer2<ConstStringSetElement> contents;	
};

class ConstStringSetElement : public CMMFObject2
{	// Used by class ConstStringSet for implementing the elements in the set

public:

  	CMMFConstString2 string;
  	CMMFPointer2<ConstStringSetElement> next;

  	ConstStringSetElement(const char *new_string)
  	{	string = new_string;
  	}
  	ConstStringSetElement(CMMFConstString2 new_string)
  	{	string = new_string;
  	}
};



class CDatabase2 : public CMMFObject2
// This is the class representing the whole database
{
public:

	CMMFPointer2<CAllGreekWords> all_greek_words;
};



class CAllGreekWords : public CMMFObject2
//	All greek words were given a number. But the initial numbering
//	was not correct. So, some number are no longer used, and 
//	additional numbers were made by adding Roman numerals.
//	Furthermore, numbers for names were given an additional 0 at
//	the start. We have decided to map all these numbers at the
//	end the numbers for numbers for non-name words by adding a
//	offset.
{
public:

  	CAllGreekWords()
  	{	lword i;
  		allWords.Allocate(5517);
  		for (i = 0; i <= 5517; i++)
  			allWords[i] = NULL;
  	}
  	
  	void addWord(lword nr, CGreekWord *new_greekWord)
  	// This method is only given the number of the word. In case it
  	// is called more than once for a certain number, it is assumed
  	// a Roman numeral was present.
  	{	if (nr > 5516)
  			return;
  		FOREACH_IN_LIST2_REF(CGreekWord, r_greekWord, allWords[nr]);
  		*r_greekWord = new_greekWord;
  	}
  	
  	CGreekWord *getWord(lword nr, word romnr)
  	{	if (nr > 5516)
  			return NULL;
  		CGreekWord *greekWord = allWords[nr];
  		while (romnr > 0 && greekWord != NULL)
  		{	greekWord = greekWord->next;
  			romnr--;
  		}
  		return greekWord;
  	}
  	
  	CMMFArray2<CMMFPointer2<CGreekWord> > allWords;
};


class CGreekWord : public CMMFObject2
// This class represents a single Greek word as found in the
// word study books.
{
public:

	//	General information of the word: Greek, Transliteration, and in Textus Receptus

  	CGreekWord(char *new_greek, char *new_transl, bool new_inTR)
  	{	greek = new_greek;
  		transl = new_transl;
  		inTR = new_inTR;
  		nr_word_forms = 0;
  		nr_verse_refs = 0;
  		next = NULL;
  	}

  	CMMFConstString2 greek;
  	CMMFConstString2 transl;
  	bool inTR;  	

	//	The different word forms of the word as they are used in the bible text.

  	void addWordForms(byte new_nr_word_forms, CGreekWordForm *new_word_forms[])
  	{	nr_word_forms = new_nr_word_forms;
  		word_forms.Allocate(nr_word_forms);
  		for (lword i = 0; i < nr_word_forms; i++)
  			word_forms[i] = new_word_forms[i];
  	}
  	
  	CGreekWordForm *getWordFormByNumber(byte form_number)
  	{	for (lword i = 0; i < nr_word_forms; i++)
  			if (word_forms[i]->form_number == form_number)
  				return word_forms[i];
  		return NULL;
  	}
  	
  	byte nr_word_forms;
  	CMMFArray2<CMMFPointer2<CGreekWordForm> > word_forms;

	//	Reference of the word to uses in the bible text

  	void addRefs(word new_nr_verse_refs, CGreekVerseRef *new_verse_refs[])
  	{	nr_verse_refs = new_nr_verse_refs;
  		verse_refs.Allocate(nr_verse_refs);
  		for (lword i = 0; i < nr_verse_refs; i++)
  			verse_refs[i] = new_verse_refs[i];
  	}
  	
  	word nr_verse_refs;
  	CMMFArray2<CMMFPointer2<CGreekVerseRef> > verse_refs;
  	

  	CMMFPointer2<CGreekWord> next;
};

class CGreekWordForm : public CMMFObject2
// This class represents a single Greek word form of a Greek word.
// Some word forms are defined as being equal to another word form
{
public:
	
	//	General information

  	CGreekWordForm(byte new_form_number, char *new_greek, char *new_transl, char *new_vert, bool new_inTR)
  	{	form_number = new_form_number;
  		greek = new_greek;
  		transl = new_transl;
  		vert = new_vert;
  		inTR = new_inTR;
  		cnt_used_in_ref = 0;
  		next = NULL;
  	}

  	CGreekWordForm(byte new_form_number, CGreekWordForm *new_as)
  	{	form_number = new_form_number;
  		greek = new_as->greek;
  		transl = new_as->transl;
  		vert = new_as->vert;
  		inTR = new_as->inTR;
  		as = new_as;
  		cnt_used_in_ref = 0;
  		next = NULL;
  	}

  	void append_vert(char *extra_vert)
  	//	This methods extends a given "vertaling" 
	{	char buf[200]; 
  		strcpy(buf, vert); // (now can free vert)
  		if (extra_vert[0] != ' ' && buf[strlen(buf)-1] != ' ');
  			strcat(buf, " ");
  		strcat(buf, extra_vert);
  		vert = buf;
  	}

  	byte form_number;
  	CMMFConstString2 greek;
  	CMMFConstString2 transl;
  	CMMFConstString2 vert;
  	bool inTR;
  	CMMFPointer2<CGreekWordForm> as;

	// number of times this word form is used in a reference
  	word cnt_used_in_ref; 

	// Alternative spelling of the word forms. Still needs to be detailed further
  	ConstStringSet alt_greek;   
  	ConstStringSet alt_transl;
  	ConstStringSet alt_vert;


	CMMFPointer2<CGreekWordForm> next;
};

class CGreekVerseRefFormNumber
// This class represents one word form number with a reference,
// including the spelling form used in the bible text.
{	
public:

	byte nr;
	CMMFConstString2 greekWordForm;
};


class CGreekVerseRef : public CMMFObject2
// This class represents one reference of the appearance of the word in the
// text of the bible. It is possible that the word has different word forms
// for different text versions.
// Remark: still need to add member for the "text" explaining the appearance
// of the word (form) in the given verse
{
public:

	//	General information of the reference

	CGreekVerseRef(byte new_book_nr, byte new_chap_nr, byte new_verse_nr)
	{
		book_nr = new_book_nr;
		chap_nr = new_chap_nr;
		verse_nr = new_verse_nr;
		form_numbers[0].nr = form_numbers[1].nr = form_numbers[2].nr = 0;
		tc_code = ' ';
	}

	byte book_nr;
	byte chap_nr;
	byte verse_nr;

	//	The text critical TC explains the appearance of the word in different
	//  text versions.

	char tc_code;
	
	//	The word forms used for the appearance
	
	void addFormNumber(byte form_number, char &error_code)
	{
		error_code = ' ';
		for (lword i = 0; i < 3; i++)
			if (form_numbers[i].nr == 0)
			{	form_numbers[i].nr = form_number;
				return;
			}
			else if (form_numbers[i].nr == form_number)
			{	error_code = 'R';
				return;
			}
			
		error_code = 'F';
	}

	CGreekVerseRefFormNumber form_numbers[3];
};

			




