const byte gt_empty = 0;
const byte gt_header = 1;
const byte gt_col1 = 2;
const byte gt_col2 = 3;
const byte gt_col3 = 4;
const byte gt_col4 = 5;
const byte gt_col1_vn = 6;
const byte gt_col2_vn = 7;
const byte gt_col3_vn = 8;
const byte gt_col4_vn = 9;
const byte gt_peri_title = 10;
const byte gt_normal = 11;

const char *GeomTypeName(byte gt)
{	switch (gt)
	{	case gt_empty: return "empty";
		case gt_header: return "header";
		case gt_col1: return "col1";
		case gt_col2: return "col2";
		case gt_col3: return "col3";
		case gt_col4: return "col4";
		case gt_col1_vn: return "col1-verse-nr";
		case gt_col2_vn: return "col2-verse-nr";
		case gt_col3_vn: return "col3-verse-nr";
		case gt_col4_vn: return "col4-verse-nr";
		case gt_peri_title: return "peri-title";
		case gt_normal: return "normal";
	}
	return "???";
}
 
bool InRange(lword v, lword f, lword t)
{	return v >= f && v <= t;
}

bool InXRange(CBox *box, lword x1f, lword x1t, lword x2f, lword x2t)
{	return InRange(box->x1, x1f, x1t) && InRange(box->x2, x2f, x2t);
}

void SetGeomType(CFrame *frame)
{	CBox *box = &frame->box;
	CPage *page = frame->page;
	CQXDoc *doc = page->doc;
	
	byte geom_type = gt_normal;
	bool okay = TRUE;
			
	char *par_style = frame->FirstParStyle();
	if (par_style == NULL)
		par_style = "<UNDEF>";

	if (!frame->HasText())
		geom_type = gt_empty;
	else if (frame->box.y2 < 6300000)
		geom_type = gt_header;
	else if (frame->page->side_by_side)
	{	
		if (InXRange(box, 72000000, 74000000,101000000,103500000))
		{	okay =    !strcmp(par_style, "Pericopetitel")
				   || !strcmp(par_style, "Normal");
			geom_type = gt_peri_title;
		}
		else if (InXRange(box, 38000000, 39500000, 38500000, 41000000)
				 && frame->TextIsNumber())
		{
			geom_type = gt_col1_vn;
		}
		else if (   InXRange(box, 61500000, 64000000, 63000000, 65500000)
				 && frame->TextIsNumber())
		{	geom_type = gt_col2_vn;
		}
		else if (InXRange(box, 61000000, 64000000, 65000000, 68500000))
		{	okay =    !strcmp(par_style, "Kolom_2_")
				  || !strcmp(par_style, "Kolom _2_");
			geom_type = gt_col2;
		}
		else if (   InXRange(box, 72000000, 74500000, 74000000, 76000000)
				 && frame->TextIsNumber())
		{	geom_type = gt_col3_vn;
		}
		else if (InXRange(box, 71000000, 75000000, 82000000, 86500000))
		{	okay =    !strcmp(par_style, "Kolom_3_")
				   || !strcmp(par_style, "Kolom _3_");
			geom_type = gt_col3;
		}
		else if (   InXRange(box, 84500000, 87000000, 86000000, 88500000)
				 && frame->TextIsNumber())
		{	geom_type = gt_col4_vn;
		}
		else if (InXRange(box, 83000000, 87500000, 88000000,104000000))
		{	okay =    !strcmp(par_style, "Kolom_4_")
				   || !strcmp(par_style, "Kolom _4_");
			geom_type = gt_col4;
		}
		else if (InXRange(box,        0, 62000000,        0, 62900000))
		{	okay = !strcmp(par_style, "Number");
			geom_type = gt_col1;
		}

		if (!okay)
		{	CTextAccessor text_acc(frame->text_fragment);
			text_acc.Init();
			printf("geom%02d %10ld %10ld %20s %d %s%s %d %ld\n", 
				   geom_type, 
				   frame->box.y1, frame->box.y2, par_style,
				   text_acc.ch,
				   okay ? "" : "ERROR ",
				   doc->name.A(), page->page_nr, frame->frame_nr);
		}
	}
	else 
	{
		if (frame->box.y1 < 6000000)
			geom_type = gt_header;
		
		if (    par_style != NULL
			&& (   !strncmp(par_style, "Kolom", 5)
				|| !strcmp(par_style, "Number")
				|| !strcmp(par_style, "Grieks")
				|| !strcmp(par_style, "Trans")
				|| !strcmp(par_style, "Vertaling")))
			printf("NotSideBySide %10ld %10ld %20s %s.%d\n", 
				   frame->box.x1, frame->box.x2, par_style,
				   doc->name.A(), page->page_nr);
	}

	if (!okay)
	{
	}
	
	frame->geom_type = geom_type;
}

int FrameReadingOrder(CFrame *f1, CFrame *f2)
{	return CBox::ReadingOrder(f1->box, f2->box);
}

int FrameReadingOrderPageByPage(CFrame *f1, CFrame *f2)
{	
	bool f1_on_the_left = f1->box.x1 < 70000000;
	bool f2_on_the_left = f2->box.x1 < 70000000;
	if (f1_on_the_left && !f2_on_the_left)
		return -1;
	if (!f1_on_the_left && f2_on_the_left)
		return 1;
	return CBox::ReadingOrder(f1->box, f2->box);
}

void CheckSuccesiveFrames(CFrame *prev_frame, CFrame *frame, word page_nr)
{	int comp = CBox::ReadingOrder(prev_frame->box, frame->box);
	if (comp < -2)
		; // -- in correct order
	else if (comp == 0 && CBox::Equal(prev_frame->box, frame->box))
		/*printf("\n  Warning: frames %ld and %ld are equal",
			   prev_frame->frame_nr, frame->frame_nr)*/;
	else
	{
		if (comp == -2)
			/*printf("\n  Error: frames %ld and %ld not in strict reading order",
			   	   prev_frame->frame_nr, frame->frame_nr)*/;
		else if (   !prev_frame->text_fragment.IsNull()
				 && !frame->text_fragment.IsNull()
				 && !prev_frame->text_fragment->IsEmpty()
				 && !frame->text_fragment->IsEmpty())
		{ 	printf("\n  Error: On page %d frames %ld and %ld in wrong order: '%s'",
			   	   page_nr, prev_frame->frame_nr, frame->frame_nr,
			   	   CBox::ReadingOrderDescription(prev_frame->box, frame->box));
			CBox inter_box = CBox::Intersection(prev_frame->box, frame->box);
			if (inter_box.Dx() != 0 && inter_box.Dy() != 0)
				printf(" Overlap: [%6.3f %6.3f %f]",
					   inter_box.Area()/prev_frame->box.Area(),
					   inter_box.Area()/frame->box.Area(),
					   inter_box.Area());
			else
				printf(" No overlap");
			printf(" [%ld, %ld - %ld, %ld]",
				   prev_frame->box.x1, prev_frame->box.y1, 
				   prev_frame->box.x2, prev_frame->box.y2);
			printf(" [%ld, %ld - %ld, %ld]",
				   frame->box.x1, frame->box.y1, 
				   frame->box.x2, frame->box.y2);
			printf("\n     ");
			dump_text_fragment(prev_frame->text_fragment, stdout);		   
			printf("\n     ");
			dump_text_fragment(frame->text_fragment, stdout); 
		}
	}
}

void CheckSubFrames(CFrame *parent_frame, word page_nr)
{
	CFrame *prev_frame = NULL;
	if (!parent_frame->children.IsNull())
	{	FOREACH_IN_LIST_N(CFrame, frame, parent_frame->children, next_child)
		{	if (prev_frame)
				CheckSuccesiveFrames(prev_frame, frame, page_nr);
			CheckSubFrames(frame, page_nr);
			prev_frame = frame;
		}
	}
}

bool IsSideBySide(CPage *page)
{
	FOREACH_IN_LIST(CFrame, frame, page->all_frames)
	{	char *par_style = frame->FirstParStyle();
		if (    par_style != NULL
			&& (   !strcmp(par_style, "Number")
				|| !strncmp(par_style, "Kolom", 5)))
			return TRUE;
	}
	
	return FALSE;		
}

void PrintFrameId(CFrame *frame)
{	printf("frame %ld on page %d of %s",
		   frame->frame_nr,
		   frame->page->page_nr,
		   frame->page->doc->name.A());
}

void ReorganizeInsideFrames(CMMFPointer<CFrame> *r_first_frame,
							int (*order)(CFrame *f1, CFrame *f2))
{
	CMMFPointer<CFrame> *r_frame = r_first_frame;
	while (!(*r_frame).IsNull())
	{
		CFrame *frame;
		for (frame = (*r_first_frame); frame != NULL; frame = frame->next_child)
			if (CBox::Inside((*r_frame)->box, frame->box) > 0)
				break;
		
		if (frame != NULL)
		{	
			printf("\nInfo: Put child frame inside other child:\n - ");
			PrintFrameId((*r_frame));
			printf("\n - ");
			PrintFrameId(frame);
			printf("\n");
			fflush(stdout);
			
			// remove the current frame
			CFrame *cur_frame = (*r_frame);
			*r_frame = (*r_frame)->next_child;
			
			// insert it as a child of the frame found
			cur_frame->parent = NULL;
			frame->InsertChild(cur_frame, order);
		}
		else
			r_frame = &(*r_frame)->next_child;
	}
	
	// Recursively call ReorganizeInsideFrame
	CFrame *frame;
	for (frame = (*r_first_frame); frame != NULL; frame = frame->next_child)
		ReorganizeInsideFrames(&frame->children, order);
}

void AnalyzeFrameGeometry(CPage *page, word page_nr)
{
	page->side_by_side = IsSideBySide(page);

	int (*order)(CFrame *, CFrame *);
	if (page->side_by_side)
		order = FrameReadingOrder;
	else
		order = FrameReadingOrderPageByPage;
	
	// Determine geometric type of each frame
	FOREACH_IN_LIST(CFrame, frame, page->all_frames)
		SetGeomType(frame);
			
	// First put all sub frames under their correct parent
	FOREACH_IN_LIST(CFrame, frame, page->all_frames)
		if (frame->nr_sub_frames > 0)
			for (lword i = 0; i < frame->nr_sub_frames; i++)
			{	CFrame *sub_frame = page->GetFrameWithNr(frame->sub_frame_nrs[i]);
				if (sub_frame != NULL)
					if (!sub_frame->parent.IsNull())
						if (sub_frame->parent->frame_nr == frame->frame_nr)
							printf("\n  Warning: sub_frane %ld already inserted into frame %ld",
								   sub_frame->frame_nr, frame->frame_nr);
						else
							printf("\n  Error: cannot insert sub_frane %ld into frame %ld because it is already inserted in %ld",
								   sub_frame->frame_nr, frame->frame_nr, sub_frame->parent->frame_nr);
					else
					{	//printf("\n  Insert sub_frame %ld into frame %ld", sub_frame->frame_nr, frame->frame_nr);
						frame->InsertChild(sub_frame, order);
					}
			}
			
   // See, if any of the remaining frames fit inside one of the other frames			
	FOREACH_IN_LIST(CFrame, frame, page->all_frames)
		if (frame->parent.IsNull())
		{	CFrame *candidate_parent_frame = NULL;
			FOREACH_IN_LIST(CFrame, other_frame, page->all_frames)
				if (   other_frame != frame
					&& CBox::Inside(frame->box, other_frame->box) > 0
					&& (   candidate_parent_frame == NULL
						|| CBox::Inside(other_frame->box, candidate_parent_frame->box) > 0))
					candidate_parent_frame = other_frame;
			if (candidate_parent_frame)
			{	//printf("\n  Insert frame %ld into frame %ld because it is inside", 
				//       frame->frame_nr, candidate_parent_frame->frame_nr);
				candidate_parent_frame->InsertChild(frame, order);
			}
		}

	// Also does this for all child frames
	FOREACH_IN_LIST(CFrame, frame, page->all_frames)
		if (frame->parent.IsNull())
			ReorganizeInsideFrames(&frame->children, order);	
 
 	// Sort all the top level frames, leaving out the frames that have a parent
	CFrame *frame = page->all_frames, *next_frame;
	page->all_frames = NULL;
	for (; frame; frame = next_frame)
	{	next_frame = frame->next;
		frame->next = NULL;	
		if (frame->parent.IsNull())
			page->InsertFrame(frame, order);
	}
}

void AnalyzeFrameGeometry(CQXDoc *doc)
{
	word page_nr = 1;
	FOREACH_IN_LIST(CPage, page, doc->all_pages)
	{
		if (page_nr > doc->nr_template_pages)
		{	//printf("\nPage %d", page_nr);
			AnalyzeFrameGeometry(page, page_nr);
		}

		page_nr++;	
	}
}

CMMFPointer<CFrame> *r_cur_frame_reading_order;

bool debug_ro = FALSE;

void SetReadingOrder(CFrame *frame)
{
	FOREACH_IN_LIST_N(CFrame, sub_frame, frame->children, next_child)
		SetReadingOrder(sub_frame);
		
	if (frame->geom_type != gt_empty && frame->geom_type != gt_header)
	{	if (   frame->page->page_nr == 18
	        && (frame->frame_nr == 61 || frame->frame_nr == 1)
	        && !strcmp(frame->page->doc->name.A(),
				 		    "/cygdrive/g/studiebijbel/q/DEEL07B/PAG.269"))
		{	printf("Warning: removed double verse number frame %ld on page 18 of DEEL07B/PAG.269\n",
				   frame->frame_nr);
		}
		else
		{	(*r_cur_frame_reading_order) = frame;
			r_cur_frame_reading_order = &frame->next_reading_order;
		}
	}
}

void MoveFrameToCur(CMMFPointer<CFrame> *&r_frame,
					CMMFPointer<CFrame> *&r_cur_frame)
{
	if (debug_ro)
		printf("     Move frame (gt = %d) to cur (gt =%d) ",
			   (*r_frame)->geom_type, (*r_cur_frame)->geom_type);
	if (*r_frame == *r_cur_frame)
	{	// -- in place move
		r_frame = &(*r_frame)->next_reading_order;
	}
	else
	{	// Take it out
		CFrame *frame_to_move = (*r_frame);
		(*r_frame) = frame_to_move->next_reading_order;
		
		// Put it in
		frame_to_move->next_reading_order = (*r_cur_frame);
		*r_cur_frame = frame_to_move;
	}

 	// Move r_cur_frame as well
	r_cur_frame = &(*r_cur_frame)->next_reading_order;

	if (debug_ro)
		printf("=> after: frame (gt = %d) to cur (gt =%d)\n",
			   (*r_frame).IsNull() ? -1 : (*r_frame)->geom_type,
			   (*r_cur_frame).IsNull() ? -1 : (*r_cur_frame)->geom_type);

}

void MoveColumnFrames(bool find, bool expect, byte gt_col_vn, byte gt_col, lword y, word page_nr, lword vers_nr,
					  CMMFPointer<CFrame> *&r_cur_frame)
{
	if (debug_ro)
		printf(" MoveColumnFrame %d %d\n", gt_col_vn, gt_col);
	CMMFPointer<CFrame> *r_ahead_frame = r_cur_frame;
	byte geom_type;

	geom_type = (*r_ahead_frame)->geom_type;
	if (find)
	{	for(; !(*r_ahead_frame).IsNull(); 
			r_ahead_frame = &(*r_ahead_frame)->next_reading_order)
		{	geom_type = (*r_ahead_frame)->geom_type;
			if (debug_ro)
				printf("    find %ld on %d gt:%d\n", 
					   (*r_ahead_frame)->frame_nr, 
					   (*r_ahead_frame)->page->page_nr, 
					   geom_type);
			if (   geom_type == gt_col_vn
				|| geom_type == gt_col
				|| geom_type == gt_normal
				|| geom_type == gt_peri_title)
				break;
		}
	}
	
	if ((*r_ahead_frame).IsNull())
	{	if (debug_ro)
			printf("    -> not found NULL\n");
		return;
	}
	else if (   geom_type == gt_col_vn
	         && (   (*r_ahead_frame)->NumberValueText() == vers_nr
	             || (   (*r_ahead_frame)->box.y1 <= y
	                 && (*r_ahead_frame)->page->page_nr == page_nr)))
	{	if ((*r_ahead_frame)->NumberValueText() != vers_nr)
		{	printf("Error: verse number is %ld (expect %ld) in ",
				   (*r_ahead_frame)->NumberValueText(), vers_nr);
			PrintFrameId((*r_ahead_frame));
			printf("\n");
		}

		MoveFrameToCur(r_ahead_frame, r_cur_frame);
	}
	else if (geom_type == gt_col && (*r_ahead_frame)->box.y1 <= y)
	{	CMMFPointer<CFrame> *r_next_ahead_frame = &(*r_ahead_frame)->next_reading_order;
		if (   !(*r_next_ahead_frame).IsNull()
			&& (*r_next_ahead_frame)->geom_type == gt_col_vn
	        && (   (*r_next_ahead_frame)->NumberValueText() == vers_nr
	            || (*r_next_ahead_frame)->box.y1 <= y))
	    {	printf("Warning: verse number frame after frame ");
			PrintFrameId((*r_ahead_frame));
			printf("\n");
			
			bool ahead_is_cur = (*r_ahead_frame) == (*r_cur_frame);
			MoveFrameToCur(r_next_ahead_frame, r_cur_frame);
			
			if (ahead_is_cur)
			{   if (debug_ro)
				{	printf("   ahead = cur\n      before ");
					PrintFrameId((*r_ahead_frame));
				}
				r_ahead_frame = r_cur_frame;
				if (debug_ro)
				{	printf("\n      after  ");
					PrintFrameId((*r_ahead_frame));
					printf("\n");
				}
			}
	    }
	    else
		{	printf("Error: no verse number inside ");
			PrintFrameId((*r_ahead_frame));
			printf("\n");
		}
	}
	else if (expect)
	{   printf("Error: expecting frame in column %d, found ", 
			   gt_col_vn - gt_col1_vn + 1);
		PrintFrameId((*r_ahead_frame));
		printf("\n");
	}
	else
	{	if (debug_ro)
			printf("   -> not found %d nr=%ld (expect %ld) y = %ld (expect <= %ld)\n", 
		    	   geom_type, (*r_ahead_frame)->NumberValueText(), vers_nr,
		    	   (*r_ahead_frame)->box.y1, y);
		return;
	}
	bool found_col1_vn = false;
	lword col1_y;
	int nr;
	
	for(nr = 1; !(*r_ahead_frame).IsNull(); nr++)
	{	geom_type = (*r_ahead_frame)->geom_type;
	    if (debug_ro)
			printf("    see frame %ld on %d gt:%d\n", 
				   (*r_ahead_frame)->frame_nr, 
				   (*r_ahead_frame)->page->page_nr, geom_type);
		if (   geom_type == gt_col_vn
			|| geom_type == gt_normal
			|| geom_type == gt_peri_title)
			break;

		if (geom_type == gt_col 
		    && found_col1_vn && nr > 1
		    && (*r_ahead_frame)->box.y2 >= col1_y)
		{	printf("\nInfo: consider frame to be part of next verse: ");
			PrintFrameId((*r_ahead_frame));
			printf("\n");
			break;
		}
		if (!found_col1_vn && geom_type == gt_col1_vn)
		{	found_col1_vn = TRUE;
			col1_y = (*r_ahead_frame)->box.y1;
		}
		
		if (geom_type == gt_col)
			MoveFrameToCur(r_ahead_frame, r_cur_frame);
		else
			r_ahead_frame = &(*r_ahead_frame)->next_reading_order;
	}
}

void SetReadingOrder(CBook *book)
{
	r_cur_frame_reading_order = &book->first_frame_reading_order;
	
	// First chain everything according to the normal order
	FOREACH_IN_LIST(CQXDoc, doc, book->all_QX_docs)
	{
		FOREACH_IN_LIST(CPage, page, doc->all_pages)
		{
			if (page->page_nr > doc->nr_template_pages)
			{
				FOREACH_IN_LIST(CFrame, frame, page->all_frames)
				{
					if (frame->parent.IsNull())
						SetReadingOrder(frame);
				}
			}
		}
	}
	
	// Correct for verses split over more than one page
	CMMFPointer<CFrame> *r_cur_frame = &book->first_frame_reading_order;
	while (!(*r_cur_frame).IsNull())
	{
		byte geom_type = (*r_cur_frame)->geom_type;
		if (debug_ro)
		{	printf("Correct at ");
			PrintFrameId((*r_cur_frame));
			printf(" geom type %d\n", geom_type);
		}
		
		if (   geom_type == gt_col1_vn || geom_type == gt_col1 
		    || geom_type == gt_col3_vn)
		{	lword y = (*r_cur_frame)->box.y1 + 500000;
			word page_nr = (*r_cur_frame)->page->page_nr;
			lword vers_nr = 0;
			bool seek_col1_v1 = FALSE;
			if (geom_type == gt_col1)
			{	if (debug_ro)
					printf("  Found col 1 with no verse number\n");
			}
			else if (geom_type == gt_col1_vn)
			{	vers_nr = (*r_cur_frame)->NumberValueText();
				if (debug_ro)
					printf("  Found col 1 with nr=%ld\n", vers_nr);
			}
			else
			{	vers_nr = (*r_cur_frame)->NumberValueText();
				printf("Warning: Found col %d with nr=%ld\n",
				       geom_type - gt_col1_vn + 1, vers_nr);
				seek_col1_v1 = TRUE;
			}
			
			// search for the next gt_col3 or gt_peri_title
			for (CMMFPointer<CFrame> *r_ahead_frame = r_cur_frame; 
				 !(r_ahead_frame)->IsNull(); 
				 r_ahead_frame = &(*r_ahead_frame)->next_reading_order)
			{	geom_type = (*r_ahead_frame)->geom_type;
				if (debug_ro)
					printf("     seek peri %08lX %d\n", (lword)(*r_ahead_frame).A(), geom_type);
				if (geom_type == gt_peri_title)
				{	if (debug_ro)
						printf("  => pericope title\n");
					MoveFrameToCur(r_ahead_frame, r_cur_frame);
					break;
				}
				if (   geom_type == gt_normal 
					|| geom_type == gt_col3_vn || geom_type == gt_col3 
					|| geom_type == gt_col4_vn || geom_type == gt_col4)
					break;
			}
   
			// --> move pericope title if you find it.
			MoveColumnFrames(TRUE, TRUE,  gt_col1_vn, gt_col1, y, page_nr, vers_nr, r_cur_frame);
			MoveColumnFrames(TRUE, FALSE, gt_col2_vn, gt_col2, y, page_nr, vers_nr, r_cur_frame);
			MoveColumnFrames(TRUE, TRUE,  gt_col3_vn, gt_col3, y, page_nr, vers_nr, r_cur_frame);
			MoveColumnFrames(TRUE, TRUE,  gt_col4_vn, gt_col4, y, page_nr, vers_nr, r_cur_frame);
		}
		else if (   geom_type == gt_col2 || geom_type == gt_col2_vn 
		         || geom_type == gt_col3 
		         || geom_type == gt_col4 || geom_type == gt_col4_vn)
		{	printf("Error: wrong type of frame (gt = %d)\n", geom_type);
			PrintFrameId((*r_cur_frame));
			printf("\n");
			r_cur_frame = &(*r_cur_frame)->next_reading_order;
		}
		else
			r_cur_frame = &(*r_cur_frame)->next_reading_order;
	}
}
	
