#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include "stddef.c"
#define WIN32_LEAN_AND_MEAN
#define NOMINMAX
#include <windows.h>
#include "MMFile.cpp"
#include "CQXDoc.cpp"
#include "DumpQXDoc.cpp"
#include "FrameGeom.cpp"
#include "greek.cpp"

bool isnormal(char ch)
{
	return
	   "         h      "
	   "               h"
	   "hhhhhhhhhhhhhhhh"
	   "hhhhhhhhhhhhhhhh"
	   "hhhhhhhhhhhhhhhh"
	   "hhhhhhhhhhhhhhhh"
	   "hhhhhhhhhhhhhhhh"
	   "hhhhhhhhhhhhhhhh"
	   "h  h  hhhhh  hhh"
	   "hhhhhh hhhh hhhh"
	   "h    h   h      "
	   " h              "
	   "       hhhh     "
	   "h hhhh          "
	   "       hhhh h h "
	   "  h  h  h       "
	[(byte)ch] == 'h';
}

char type_of_word(const char *str)
{
	int l = 0; 
	int h = 0;
	int g = 0;
	for (; *str != '\0'; str++)
	{	l++;
		if (isgreek(*str))
			g++;
		if (isnormal(*str))
			h++;
	}
	if (h < g)
		return 'g';
	if (g < h)
		return 'h';
	return '?';
}

char errmsg[10000];

int depth;
void newline(FILE *fout)
{
	if (errmsg[0] != '\0')
	{	fprintf(fout, "%s", errmsg);
		errmsg[0] = '\0';
	}
	fprintf(fout, "\n");
	int d = depth;
	for (; d >= 4; d -= 4)
		fprintf(fout, "\t");
	for (; d > 0; d--)
		fprintf(fout, " ");	
}

long normal_used[256];
long helv_used[256];
long greek_used[256];

bool in_wordstudy = false;


#define IN_ITALIC  (1<<0)
#define IN_BOLD    (1<<1)
#define IN_SUP     (1<<2)
#define IN_GREEK   (1<<3)
#define IN_HELV    (1<<4)

bool equal_greek(const char *a, const char *b)
{
	for (;; a++, b++)
	{
		if (*a == '\0')
			return *b == '\0';
		if (*b == '\0')
			return false;
		if (   greek_char_code(a[0]) == LOW_I_PRE 
			&& greek_char_code(a[1]) == LOW_I_PRE
		    && greek_char_code(b[0]) == LOW_I_PRE 
			&& greek_char_code(b[1]) != LOW_I_PRE)
		    a++;
		else if (   *b == (char)0xD8 
		    	 && (a[0] == 0x5C || a[0] == (char)0xD5) 
		         && (a[1] == 0x49 || a[1] == (char)0xB9))
			a++; // 'I
		else if (*a != *b && greek_char_code(*a) != greek_char_code(*b))
			return false;
	}
}	

void dump_helv(FILE* fout, const char *helv);

void dump_greek(FILE* fout, const char *greek)
{
	if (greek[0] == '\0')
		return;


	if (type_of_word(greek) == 'h')
	{
		//sprintf(errmsg+strlen(errmsg), "\n%c word '%s' does not look like Greek word", 
		//		'%', greek);
	}

	fprintf(fout, "G[");
	char tb[1000];
	strcpy(tb, greek_to_text(greek));
	char sb[1000];
	strcpy(sb, text_to_greek(tb));
	if (!equal_greek(greek, sb))
	{	for (const char *s = greek; *s; s++) 
			printf("%02X ", (byte)*s);
		printf("|%s|", tb);
		for (const char *s = sb; *s; s++) 
			printf(" %02X", (byte)*s);
		printf("\n");
		fprintf(fout, "{");
		for (const char *s = greek; *s; s++) 
			fprintf(fout, "#%02X", (byte)*s);
		fprintf(fout, "}]");
		//sprintf(errmsg+strlen(errmsg), "\n%c text representation '%s' of Greek word is ambigious",
		//		'%', tb);
	}
	else
		fprintf(fout, "{%s}]", tb);
}

bool in_text;

void dump_normal_char(FILE* fout, const byte ch)
{
	if (!in_text) { fprintf(fout, "{"); in_text = true; }
	switch(ch)
	{	case 0x10: fprintf(fout, " "); break;
		case '#':
		case '}':  fprintf(fout, "#%02X", ch); break;
		//case 0x5C: fprintf(fout, ""); break;
		case 0x5F: fprintf(fout, "%c", in_wordstudy ? '^' : ''); break;
		case 0x7C: fprintf(fout, ""); break;
		case 0x80: fprintf(fout, ""); break;
		case 0x83: fprintf(fout, ""); break;
		case 0x85: fprintf(fout, ""); break;
		case 0x86: fprintf(fout, ""); break;
		case 0x87: fprintf(fout, ""); break;
		case 0x88: fprintf(fout, ""); break;
		case 0x89: fprintf(fout, ""); break;
		case 0x8A: fprintf(fout, ""); break;
		case 0x8D: fprintf(fout, ""); break;
		case 0x8E: fprintf(fout, ""); break;
		case 0x8F: fprintf(fout, ""); break;
		case 0x90: fprintf(fout, ""); break;
		case 0x91: fprintf(fout, ""); break;
		case 0x92: fprintf(fout, ""); break;
		case 0x93: fprintf(fout, ""); break;
		case 0x94: fprintf(fout, ""); break;
		case 0x95: fprintf(fout, ""); break;
		case 0x97: fprintf(fout, ""); break;
		case 0x98: fprintf(fout, ""); break;
		case 0x99: fprintf(fout, ""); break;
		case 0x9A: fprintf(fout, ""); break;
		case 0x9C: fprintf(fout, ""); break;
		case 0x9D: fprintf(fout, ""); break;
		case 0x9E: fprintf(fout, ""); break;
		case 0x9F: fprintf(fout, ""); break;
		case 0xA0: fprintf(fout, ""); break;
		case 0xA5: fprintf(fout, ""); break;
		case 0xA7: fprintf(fout, ""); break;
		case 0xA9: fprintf(fout, ""); break;
		case 0xC7: fprintf(fout, ""); break;
		case 0xC8: fprintf(fout, ""); break;
		case 0xC9: fprintf(fout, "..."); break;
		case 0xCA: fprintf(fout, " "); break;
		case 0xB1: fprintf(fout, ""); break;
		case 0xD0: fprintf(fout, "^"); break;
		case 0xD2: fprintf(fout, ""); break;
		case 0xD3: fprintf(fout, ""); break;
		case 0xD4: fprintf(fout, ""); break;
		case 0xD5: fprintf(fout, ""); break;
		//case 0xE7: fprintf(fout, ""); break;
		case 0xE8: fprintf(fout, ""); break;
		case 0xE9: fprintf(fout, ""); break;
		case 0xEA: fprintf(fout, ""); break;
		case 0xEC: fprintf(fout, ""); break;
		case 0xEE: fprintf(fout, ""); break;
		case 0xF2: fprintf(fout, ""); break;
		case 0xF5: fprintf(fout, "i"); break;
		case 0xF8: fprintf(fout, "^"); break;
		default:
			if (ch < ' ' || ch >= 123)
				fprintf(fout, "#%02X", ch);
			else
				fprintf(fout, "%c", ch);
			break;
	}
}

void dump_helv(FILE* fout, const char *helv)
{
	if (helv[0] == '\0')
		return;
		
	if (type_of_word(helv) == 'g')
	{
		//sprintf(errmsg+strlen(errmsg), "\n%c word '%s' reads like Greek word '%s'", 
		//		'%', helv, greek_to_text(helv));
		//dump_greek(fout, helv);
		//return;
	}
	
	fprintf(fout, "H[");
	for (int i = 0; helv[i] != '\0'; i++)
		dump_normal_char(fout, helv[i]);
	if (in_text)
		fprintf(fout, "}]");
	in_text = false;
}

void dump_par(FILE* fout, CTextOnFramesAccessor &text_acc, char* par_style)
{
	in_text = false;
	int cur_mode = 0;
	char buf[1000];
	char *s = 0;
				
	for (; text_acc.More(); text_acc.Next())
	{
		if (text_acc.is_newline || text_acc.in_new_par_style)
			break;

		if (text_acc.ch < 5)
		{	// skip
		}
		else
		{
			byte ch = text_acc.ch;
			if (ch == 0x10)
				ch = ' ';
			
			if (' ' <= ch || text_acc.is_tab)
			{
				int new_mode = 0;
				if (text_acc.mode & 1)
					new_mode |= IN_BOLD;
				if (text_acc.mode & 2)
					new_mode |= IN_ITALIC;
				if (text_acc.mode & 288)
					new_mode |= IN_SUP;
				if (text_acc.font_family == 'G' && ch != 0x09)
					new_mode |= IN_GREEK;
				if (text_acc.font_family == 'H')
					new_mode |= IN_HELV;
				
				if (   (cur_mode & IN_GREEK)
					&& (   (cur_mode & (IN_ITALIC|IN_BOLD|IN_SUP)) != (new_mode & (IN_ITALIC|IN_BOLD|IN_SUP))
				        || (cur_mode & IN_GREEK) > (new_mode & IN_GREEK)))
				{	*s = '\0';
					dump_greek(fout, buf);
					cur_mode &= ~IN_GREEK;
				}
				if (   (cur_mode & IN_HELV)
					&& (   (cur_mode & (IN_ITALIC|IN_BOLD|IN_SUP)) != (new_mode & (IN_ITALIC|IN_BOLD|IN_SUP))
				        || (cur_mode & IN_HELV) > (new_mode & IN_HELV)))
				{	*s = '\0';
					dump_helv(fout, buf); 
					cur_mode &= ~IN_HELV;
				}
				if (   (cur_mode & IN_SUP)
				    && (   (cur_mode & (IN_ITALIC|IN_BOLD)) != (new_mode & (IN_ITALIC|IN_BOLD))
				    	|| (cur_mode & IN_SUP) > (new_mode & IN_SUP)))
				{	if (in_text) { fprintf(fout, "}"); in_text = false; }
					fprintf(fout, "]");
					cur_mode &= ~IN_SUP;
				}
				if (   (cur_mode & IN_BOLD)
				    && (   (cur_mode & IN_ITALIC) != (new_mode & IN_ITALIC)
				    	|| (cur_mode & IN_BOLD) > (new_mode & IN_BOLD)))
				{	if (in_text) { fprintf(fout, "}"); in_text = false; }
					fprintf(fout, "]");
					cur_mode &= ~IN_BOLD;
				}
				if ((cur_mode & IN_ITALIC) > (new_mode & IN_ITALIC))
				{	if (in_text) { fprintf(fout, "}"); in_text = false; }
					fprintf(fout, "]");
					cur_mode &= ~IN_ITALIC;
				}
					
				if (text_acc.font_family == '1')
				{	if (in_text) { fprintf(fout, "}"); in_text = false; }
					fprintf(fout, "S1(%d)", text_acc.ch);
				}
				else if (text_acc.font_family == '2')
				{	if (in_text) { fprintf(fout, "}"); in_text = false; }
					fprintf(fout, "S2(%d)", text_acc.ch);
				}
				else if (text_acc.font_family == 'S')
				{	if (in_text) { fprintf(fout, "}"); in_text = false; }
					fprintf(fout, "S(%d)", text_acc.ch);
				}
				else if (text_acc.is_tab)
				{	if (in_text) { fprintf(fout, "}"); in_text = false; }
					fprintf(fout, "t()");
				}
				else if (ch == ' ' && (cur_mode & (IN_GREEK|IN_HELV)) == 0)
				{
					if (!in_text) { fprintf(fout, "{"); in_text = true; }
					fprintf(fout, " ");
				}
				else
				{	
					if ((new_mode & IN_ITALIC) > (cur_mode & IN_ITALIC))
					{	if (in_text) { fprintf(fout, "}"); in_text = false; }
						fprintf(fout, "I[");
						cur_mode |= IN_ITALIC;
					}
					if ((new_mode & IN_BOLD) > (cur_mode & IN_BOLD))
					{	if (in_text) { fprintf(fout, "}"); in_text = false; }
						fprintf(fout, "B[");
						cur_mode |= IN_BOLD;
					}
					if ((new_mode & IN_SUP) > (cur_mode & IN_SUP))
					{	if (in_text) { fprintf(fout, "}"); in_text = false; }
						fprintf(fout, "P[");
						cur_mode |= IN_SUP;
					}
					if ((new_mode & IN_GREEK) > (cur_mode & IN_GREEK))
					{	if (in_text) { fprintf(fout, "}"); in_text = false; }
						cur_mode |= IN_GREEK;
						s = buf;
					}
					if ((new_mode & IN_HELV) > (cur_mode & IN_HELV))
					{	if (in_text) { fprintf(fout, "}"); in_text = false; }
						cur_mode |= IN_HELV;
						s = buf;
					}
					
					if (cur_mode & IN_GREEK)
						*s++ = ch;
					else
					{
						if (text_acc.mode & 3072)
							ch = _toupper(ch);
						if (cur_mode & IN_HELV)
							*s++ = ch;
						else
							dump_normal_char(fout, ch);
					}
				}
			}
		}
	}
	if (in_text) fprintf(fout, "}");
	if (cur_mode & IN_SUP) fprintf(fout, "]");
	if (cur_mode & IN_BOLD) fprintf(fout, "]");
	if (cur_mode & IN_ITALIC) fprintf(fout, "]");
	if (cur_mode & IN_GREEK) 
	{	*s = '\0';
		dump_greek(fout, buf);
	}
	if (cur_mode & IN_HELV) 
	{	*s = '\0';
		dump_helv(fout, buf);
	}
}

void dump_frame_par(FILE *fout, CTextOnFramesAccessor &text_acc)
{
	text_acc.in_new_par_style = false; // a hack!!
	
	char *par_style = "par";
	if (strcmp(text_acc.par_style, "Number") == 0)
		par_style = "number";
	else if (strcmp(text_acc.par_style, "Grieks") == 0)
		par_style = "grieks";
	else if (strcmp(text_acc.par_style, "Trans") == 0)
		par_style = "trans";
	else if (strcmp(text_acc.par_style, "Vertaling") == 0)
		par_style = "vertaling";
	else if (strcmp(text_acc.par_style, "alinea 1") == 0)
		par_style = "a1";
	else if (strcmp(text_acc.par_style, "alinea1") == 0)
		par_style = "a1";
	else if (strcmp(text_acc.par_style, "alinea2") == 0)
		par_style = "a2";
	else if (strcmp(text_acc.par_style, "alinea3") == 0)
		par_style = "a3";
	else if (strcmp(text_acc.par_style, "kopwoord") == 0)
		par_style = "kopwoord";

	bool do_newline = false;
	fprintf(fout, "%s[", par_style);
	depth += 4;

	for(;;)
	{
		if (do_newline)
		{	fprintf(fout, "]");
			depth -= 4;
			newline(fout);
			fprintf(fout, "%s[", par_style);
			depth += 4;
			do_newline = false;
		}
		dump_par(fout, text_acc, par_style);

		if (!text_acc.More() || text_acc.in_new_par_style)
			break;

		if (text_acc.is_newline)
		{	do_newline = true;
			text_acc.Next();
		}
				
		if (!text_acc.More() || text_acc.in_new_par_style)
			break;
	}
	fprintf(fout, "]");
	depth -= 4;
	
	if (text_acc.More())
		newline(fout);	
}

void dump_book(CFrameAccessor &frame_acc)
{
	char fn[200];
	sprintf(fn, "sbr/%s.sbr", frame_acc.book_name);
	FILE *fout = fopen(fn, "wt");
	if (fout == 0)
		return;
		
	fprintf(fout, "[");
	depth++;

	for (; frame_acc.More(); frame_acc.Next())		
	{
		newline(fout);
		switch(frame_acc.geom_type())
		{	case gt_col1 : fprintf(fout, "col1"); break;
			case gt_col2 : fprintf(fout, "col2"); break;
			case gt_col3 : fprintf(fout, "col3"); break;
			case gt_col4 : fprintf(fout, "col4"); break;
			case gt_col1_vn : fprintf(fout, "vn1"); break;
			case gt_col2_vn : fprintf(fout, "vn2"); break;
			case gt_col3_vn : fprintf(fout, "vn3"); break;
			case gt_col4_vn : fprintf(fout, "vn4"); break;
			case gt_peri_title: fprintf(fout, "peri"); break;
			case gt_normal : fprintf(fout, "norm"); break;
			default : fprintf(fout, "other"); break;
		}
		fprintf(fout, "[");
		depth += 3;

		// Create a text accesor to parse the text with formatting.
		CTextOnFramesAccessor text_acc(frame_acc);
		text_acc.Init();
		while (text_acc.More())
			dump_frame_par(fout, text_acc);

		fprintf(fout, "]");
		depth -= 3;
	}
	fprintf(fout, "]\n");
	depth--;
	fclose(fout);
}

int main(int argc, char *argv[])
{
	errmsg[0] = '\0';

	// Open a new database file
	if (!mmfile.OpenRead("tmp/studybible.dat"))
		return 0;

	for (int i = 0; i < 256; i++)
	{	normal_used[i] = 0;
		helv_used[i] = 0;
		greek_used[i] = 0;
	}

	// Retrieve root object
	CDatabase *database = (CDatabase*)mmfile.FirstObject();
	CFrameAccessor frame_acc(database->all_books);

	init_greek();
	for_dump = true;

	frame_acc.Open("DEEL02");
	dump_book(frame_acc);	
	frame_acc.Open("DEEL03");
	dump_book(frame_acc);
	frame_acc.Open("DEEL04");
	dump_book(frame_acc);	
	frame_acc.Open("DEEL05");
	dump_book(frame_acc);	
	frame_acc.Open("DEEL06");
	dump_book(frame_acc);	
	frame_acc.Open("DEEL07A");
	dump_book(frame_acc);	
	frame_acc.Open("DEEL07B");
	dump_book(frame_acc);	
	frame_acc.Open("DEEL08");
	dump_book(frame_acc);	
	frame_acc.Open("DEEL09");
	dump_book(frame_acc);	
	frame_acc.Open("DEEL10");
	dump_book(frame_acc);	

	mmfile.Close();
	if (!mmfile.OpenRead("tmp/studybible.dat"))
		return 0;
	
	in_wordstudy = true;
	
	frame_acc.Open("DEEL11");
	dump_book(frame_acc);	
	frame_acc.Open("DEEL12");
	dump_book(frame_acc);	
	frame_acc.Open("DEEL13");
	dump_book(frame_acc);	
	frame_acc.Open("DEEL14");
	dump_book(frame_acc);	
	frame_acc.Open("DEEL15");
	dump_book(frame_acc);	
	frame_acc.Open("DEEL16");
	dump_book(frame_acc);	

	return 0;
	printf("\n");
	for (int i = 0; i < 256; i++)
	{
		printf("%02X %3d %c ", i, i, i < ' ' ? ' ' : i);
		printf(" %5ld", normal_used[i]);
		printf(" %5ld", helv_used[i]);
		printf(" %5ld\n", greek_used[i]);
	}
	return 0;
}
