#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include "stddef.c"
#include "greek.cpp"
#include <windows.h>
#define REMOVE_DOUBLE_WORD_FORMS
#ifdef READ_FROM_DATABASE
#define WIN32_LEAN_AND_MEAN
#define NOMINMAX
#include "MMFile.cpp"
#include "CQXDoc.cpp"
#include "FrameGeom.cpp"
#include "DumpQXDoc.cpp"
#else
#include "CDFreader.cpp"
#endif
#include "MMFile2.cpp"
#define FOREACH_IN_LIST2_REF_N(T,E,L,N) CMMFPointer2<T>*E; for(E=&L;!(*E).IsNull();E=&(*E)->N)
#define FOREACH_IN_LIST2_REF(T,E,L) FOREACH_IN_LIST2_REF_N(T,E,L,next)
#include "CDatabase2.cpp"
#include "CDFOutput.cpp"

/* Memonics for error messages:
	FS - Frame structure
	C1 - Column 1
	C2 - Column 2
	WN - Word number
	GW - Greek word
	TR - Tramsliteration
	TC - Text Critical system
	VD - verse description
	VR - verse range
*/

/* Hierachy of document parse routines

   (Routines between bracket are auxillary routines.)

   - parse_comm
     - parse_peri_title
       - parse_verse_range
     - parse_verse
       - (parse_verse_column)
         - parse_column1
           - (parse_column1_par)
           - parse_word_nr
             - parse_wordnr
           - parse_greek_word
           - parse_transliteration
         - parse_column2
           - parse_variant
         - parse_column4

   - parse_word_studies
     - parse_one_word_study
       - parse_wordnr
       - parse_word_type
       - parse_verse_ref

*/

//FILE *f2 = NULL;

char debugbuf[10000];
#define DEBUGSTART debugbuf[0] = '\0';
#define DEBUG(X) sprintf(debugbuf+strlen(debugbuf),X)
#define DEBUG1(X,A) sprintf(debugbuf+strlen(debugbuf),X,A)
#define DEBUG2(X,A,B) sprintf(debugbuf+strlen(debugbuf),X,A,B)
#define DEBUG3(X,A,B,C) sprintf(debugbuf+strlen(debugbuf),X,A,B,C)
#define DEBUG4(X,A,B,C,D) sprintf(debugbuf+strlen(debugbuf),X,A,B,C,D)
#define DEBUGDUMP printf("%s",debugbuf)

#define NR_NT_BOOKS   27
char *book_abrvs[NR_NT_BOOKS] = {
  "Matt.", "Marc.", "Luc.", "Joh.", "Hand.", "Rom.", "1Cor.", "2Cor.",
  "Gal.", "Ef.", "Fil.", "Col.", "1Thess.", "2Thess.", "1Tim.", "2Tim.",
  "Tit.", "Filem.", "Hebr.", "Jac.", "1Petr.", "2Petr.", "1Joh.", "2Joh.",
  "3Joh.", "Jud.", "Openb." 
};

int find_book_abrv(const char *abrv)
{
	int i;
	for (i = 0; i < NR_NT_BOOKS; i++)
		if (strcmp(abrv, book_abrvs[i]) == 0)
			return i;
	return -1;
}



char context[200];

void PrintContext(CFrameAccessor &frame_acc)
{	frame_acc.PrintContext(context);
}

bool debug_parse_verse = false;
bool debug_parse_peri = false;
bool debug_parse_col1 = false;
bool show_col1_warning = false;
bool debug_parse_greek = false;
bool show_parse_greek_warnings = false;
bool debug_parse_text_crit = false;
bool debug_parse_col2 = false;
bool show_parse_col2_warnings = true;
bool debug_parse_ws_dump = false;
bool debug_parse_ws_ref_dump = false;
bool debug_parse_wordstudy = false;
bool show_parse_wordstudy_warnings = false;
bool show_match_greek_warnings = false;
bool show_transliteration_warnings = false;
bool opt_skip_commentary = false;

/*
bool debug_parse_verse = true;
bool debug_parse_peri = true;
bool debug_parse_col1 = true;
bool show_col1_warning = true;
bool debug_parse_greek = true;
bool show_parse_greek_warnings = true;
bool debug_parse_col2 = true;
bool show_parse_col2_warnings = true;
*/

bool GeomTypeIsText(byte geom_type)
{	return    geom_type == gt_peri_title
		   || geom_type == gt_col1_vn || geom_type == gt_col1
		   || geom_type == gt_col2_vn || geom_type == gt_col2
		   || geom_type == gt_col3_vn || geom_type == gt_col3
		   || geom_type == gt_col4_vn || geom_type == gt_col4;
}

bool isnormal(char ch)
{
	return
	 // 0123456789abcdef
	   "         #      " // 0
	   "                " // 1
	   "#!\"  %&'()*+,-./" // 2
	   "0123456789:;<=>?" // 3
	   " ABCDEFGHIJKLMNO" // 4
	   "PQRSTUVWXYZ[ ]^_" // 5
	   " abcdefghijklmno" // 6
	   "pqrstuvwxyz |   " // 7
	   "                " // 8
	   "              " // 9
	   "              " // a
	   "              " // b
	   "             " // c
	   "           " // d
	   "   " // e
	   "        " // f
	[(byte)ch] != ' ';
}



class CVerseNr
{
  public:
  	CVerseNr() { ch_nr = 0; vr_nr = 0; part = 0; };
	
	word ch_nr;
	word vr_nr;
	char part;
	
	void print(FILE *fout)
	{	if (ch_nr > 0) 
			fprintf(fout, "%d:", ch_nr);
		fprintf(fout, "%d", vr_nr);
		if (part > 0)
			fprintf(fout, "%c", part);
	}
	void print(char *&s)
	{	if (ch_nr > 0)
		{	sprintf(s, "%d:", ch_nr);
			s += strlen(s);
		}
		sprintf(s, "%d", vr_nr);
		s += strlen(s);
		if (part > 0)
		{	sprintf(s, "%c", part);
			s += strlen(s);
		}
	} 	
	
	bool operator==(CVerseNr &rhs)
	{	return ch_nr == rhs.ch_nr && vr_nr == rhs.vr_nr && part == rhs.part;
	}
	
	bool operator<(CVerseNr &rhs)
	{	return    ch_nr < rhs.ch_nr
			   || (   ch_nr == rhs.ch_nr
				   && (   vr_nr < rhs.vr_nr
						|| (vr_nr == rhs.vr_nr && part < rhs.part)));
	}
	bool operator<=(CVerseNr &rhs)
	{	return *this == rhs || *this < rhs;
	}
	bool FirstVerse()
	{	return vr_nr == 1 && (ch_nr == 0 || ch_nr == 1);
	}
	bool Match(CVerseNr &verse_nr)
	{	return    *this == verse_nr
			   || (   verse_nr.ch_nr != 0 && ch_nr != 0
				   && (   (verse_nr.vr_nr == 1 && verse_nr.ch_nr == ch_nr + 1)
					   || (vr_nr == 1 && ch_nr == verse_nr.ch_nr + 1)));
	}
};
bool Preceed(CVerseNr &l, CVerseNr &r)
{	return    (l.ch_nr == r.ch_nr && l.vr_nr + 1 == r.vr_nr)
		   || (l.ch_nr > 0 && l.ch_nr + 1 == r.ch_nr && r.vr_nr == 1 )
		   || (l.ch_nr == r.ch_nr && l.vr_nr == r.vr_nr
			   && l.part + 1 == r.part);
}
bool PreceedExact(CVerseNr &l, CVerseNr &r)
{	if (l.part != 0 || r.part != 0)
		return    l.ch_nr == r.ch_nr
		       && l.vr_nr == r.vr_nr
		       && l.part + 1 == r.part;
	return Preceed(l, r);
}

bool parse_verse_descr(CTextOnFramesAccessor &text_acc, CVerseNr &verse_nr, CFrameAccessor &frame_acc)
{
	if (!text_acc.ParseNumber(verse_nr.vr_nr))
	{	PrintContext(frame_acc);
		printf("Fout(VD01): geen cijfer gevonden");
		return false;
	}
	
	if (text_acc.More() && text_acc.ch == ':')
	{	verse_nr.ch_nr = verse_nr.vr_nr;
		verse_nr.vr_nr = 0;
		
		text_acc.Next();
		
		if (!text_acc.ParseNumber(verse_nr.vr_nr))
		{	PrintContext(frame_acc);
			printf("Fout(VD02): geen cijfer na %d:",
				   verse_nr.ch_nr);
			return false;
		}
	}
	
	if (text_acc.More() && text_acc.ch >= 'a' && text_acc.ch <= 'g')
	{	verse_nr.part = text_acc.ch;
	
		text_acc.Next();
	}
	
	return true;
}

class CVerseRange
{
  public:
	CVerseNr from;
	CVerseNr to;

	void print(FILE *fout)
	{	from.print(fout);
		printf("-");
		to.print(fout);
	}
	void print(char *&s)
	{	from.print(s);
		*s++ = '-';
		to.print(s);
	}
	const char* str()
	{	char *s = buf;
		print(s);
		return buf;
	}
  private:
    char buf[200];
    
};

bool parse_verse_range(CTextOnFramesAccessor &text_acc, CVerseRange &verse_range, CFrameAccessor &frame_acc)
{
	if (!parse_verse_descr(text_acc, verse_range.from, frame_acc))
	{	PrintContext(frame_acc);
		printf("Fout(VR01): geen versaanduiding gevonden");
		return false;
	}
	
	if (text_acc.More() && text_acc.ch == '-')
	{	text_acc.Next();
	
		if (!parse_verse_descr(text_acc, verse_range.to, frame_acc))
		{	PrintContext(frame_acc);
		    printf("Fout(VR02): Er volgt niets na ");
			verse_range.from.print(stdout);
			printf("-");
			return false;
		}
	}
	else
		verse_range.to.vr_nr = verse_range.from.vr_nr;
	
	if (verse_range.to.ch_nr == 0)
		verse_range.to.ch_nr = verse_range.from.ch_nr;
			
	if (verse_range.to < verse_range.from)
	{	PrintContext(frame_acc);
	    printf("Fout(VR03): onmogelijk versbereik ");
		verse_range.print(stdout);
	}

	
	if (debug_parse_peri)
	{	PrintContext(frame_acc);
		printf("VERSE=");
		verse_range.print(stdout);
		printf(" ");
	}
	
	return true;
}

void parse_wordnr(char *s, word &nr, word &romnr, bool &inTR);

class WordNumber
{	
public:
	WordNumber() { init(true, true); }
	void init(WordNumber *wn) 
	{	word_nr = wn->word_nr;
		word_romnr = wn->word_romnr;
		word_inTR = wn->word_inTR;
		has_forms = wn->has_forms;
		form_nr = wn->form_nr;
		form_inTR = wn->form_inTR;
	}
	void init(bool new_word_in_TR, bool new_form_in_TR) 
	{	word_nr = 0; 
		word_romnr = 0; 
		word_inTR = new_word_in_TR; 
		has_forms = false; 
		form_nr = 1; 
		form_inTR = new_form_in_TR && new_word_in_TR;
	}
	void parse(char *wn, char *wfn)
	{
		bool dummy_inTR;
		parse_wordnr(wn, word_nr, word_romnr, dummy_inTR);
		if (*wfn != '\0')
		{	has_forms = true;
			form_nr = atoi(wfn);
		}
	}
	word word_nr;
	word word_romnr;
	bool word_inTR;
	bool has_forms;
	word form_nr;
	bool form_inTR;

	char *word_str() { return str(false); }
	char *word_form_str() { return str(true); }
	bool is_name() { return word_nr > 4932; }
private:
	char buf[20];
	char *str(bool with_form)
	{
		char *s = buf;
		if (!word_inTR)
			*s++ = '[';
		static char *romals[4] = { "", "II", "III", "IV" };
		if (word_nr <= 4932)
			sprintf(s, "%d%s", word_nr, romals[word_romnr-1]);
		else
			sprintf(s, "0%d%s", word_nr - 4932, romals[word_romnr-1]);
		if (has_forms && with_form)
		{	s += strlen(s);
			*s++ = '.';
			if (!form_inTR && word_inTR)
				*s++ = '[';
			sprintf(s, "%d", form_nr);
		}
		if (!word_inTR || !form_inTR)
			strcat(s, "]");
		
		return buf;
	}
};


class CTextCritState
{	
  private:
	bool multiple;
	bool multiple_known;
	struct
	{	bool in_del;
		bool in_ins;
		char ins_letter;
		bool had_del;
		word had_ins;
		// Colom 2
		bool del_note;
		word ins_note;
	}	states[10], *s;
	word last_nr;
	bool remark_note;
  public:
  
  	bool has_remark;
  		
	CTextCritState()
	{	multiple_known = false;
		for (word i = 0; i < 10; i++)
		{	states[i].in_del = false;
			states[i].in_ins = false;
			states[i].ins_letter = '\0';
			states[i].had_del = false;
			states[i].had_ins = 0;
			states[i].del_note = false;
			states[i].ins_note = 0;
		}
		s = NULL;
		last_nr = 0;
		has_remark = false;
		remark_note = false;
	}
	
	bool inside_insert()
	{	for (word i = 0; i < 10; i++)
			if (states[i].in_ins)
				return true;
		return false;
	}
	
	void has_remark_note(CFrameAccessor &frame_acc)
	{	if (remark_note)
		{	PrintContext(frame_acc);
			printf("Fout(TC01): '*'-teken herhaald");
		}
		if (!has_remark)
		{	PrintContext(frame_acc);
			printf("Fout(TC02): Opmerking zonder dat een '*'-teken gebruikt is");
		}
		else
			remark_note = true;
	}
	void has_delete_note(word nr, CFrameAccessor &frame_acc)
	{	word i = (nr <= 9) ? nr : 0;
		if (states[i].del_note)
		{	PrintContext(frame_acc);
			printf("Fout(TC03): \"%d Tekst\" wordt herhaald", nr);
		}
		if (!states[i].had_del && states[i].had_ins == 0)
		{	PrintContext(frame_acc);
			printf("Fout(TC04): Opmerking voor niet bestaande weglating %d", nr);
		}
		else if (nr <= 9)
			states[i].del_note = true;
	}
	void has_insert_note(word nr, char letter, CFrameAccessor &frame_acc)
	{	word i = (nr <= 9) ? nr : 0;
		word b = ins_bit(letter);
		if ((states[i].ins_note & b) != 0)
		{	PrintContext(frame_acc);
			printf("Fout(TC05): \"%d Variant %c\" wordt herhaald", nr, letter);
		}
		if (   (states[i].had_ins & b) == 0
		    && (b != 1 || !states[i].had_del))
		{	PrintContext(frame_acc);
			printf("Fout(TC06): Opmerking voor niet bestaande toevoeging %d%c", nr, letter);
		}
		else if (nr <= 9)
			states[i].ins_note |= b;
	}
	void check_all_notes(CFrameAccessor &frame_acc)	
	{	if (has_remark && !remark_note)
		{	PrintContext(frame_acc);
			printf("Fout(TC07): Geen opmerking bij '*'-teken");
		}
		for (word i = 0; i <= 9; i++)
		{
			for (word j = 0; j <= 4; j++)
			{  	word b = 1 << j;
				char letter = " abcd"[j];
				
				if ((states[i].had_ins & b) != 0 && (states[i].ins_note & b) == 0 
				    && (b != 1 || !states[i].had_del))
				{	PrintContext(frame_acc);
					printf("Fout(TC08): Opmerking voor toevoeging %d%c ontbreekt", i, letter);
				}
			}
		}
	}
			
	void open_del(word nr, CFrameAccessor &frame_acc)
	{	check_nr(nr, true, frame_acc);
		if (debug_parse_text_crit) printf("open_del(%d)", nr);
		if (s->in_del)
		{	PrintContext(frame_acc);
			printf("Fout(TC09): Herhaling van het begin van een verwijdering voor %d", nr);
		}
		if (s->in_ins)
		{	PrintContext(frame_acc);
			printf("Fout(TC10): Het begin van een verwijdering staat in een toevoeging voor %d", nr);
			s->in_ins = false;
		}
		if (s->had_del && show_parse_greek_warnings)
		{	PrintContext(frame_acc);
			printf("Mededeling(TC11): Herhaalde verwijdering voor %d", nr);
		}
		s->in_del = true;
		s->had_del = true;
	}
	void close_del(word nr, CFrameAccessor &frame_acc)
	{	check_nr(nr, false, frame_acc);
		if (s->in_ins)
		{	PrintContext(frame_acc);
			printf("Fout(TC12): Het eind van een verwijdering staat in een toevoeging voor %d", nr);
			s->in_ins = false;
		}
		if (!s->in_del)
		{	PrintContext(frame_acc);
			printf("Fout(TC13): Het einde van een verwijdering wordt herhaald voor %d", nr);
		}
		s->in_del = false;
	}
	void open_ins(word nr, char letter, CFrameAccessor &frame_acc)
	{	check_nr(nr, s == NULL || !s->in_del, frame_acc);
		if (debug_parse_text_crit) printf("open_ins(%d,%c)", nr, letter);
		if (s->in_ins)
		{	PrintContext(frame_acc);
			printf("Fout(TC14): Het begin van een toevoeging wordt herhaald voor %d", nr);
		}
		s->in_del = false;
		s->in_ins = true;
		if (letter == ' ' && (s->had_ins & 1) != 0 && show_parse_greek_warnings)
		{	PrintContext(frame_acc);
			printf("Mededeling(TC15): Herhaling van het begin van een toevoeging voor %d", nr);
		}
		check_ins_letter(nr, letter, frame_acc);
	}
	void ins_letter(word nr, char letter, CFrameAccessor &frame_acc)
	{	check_nr(nr, false, frame_acc);
		if (debug_parse_text_crit) printf("ins_letter(%d,%c)", nr, letter);
		if (!s->in_ins)
		{	PrintContext(frame_acc);
			printf("Fout(TC16): De letter '%c' buiten een toevoeging gebruikt voor %d", letter, nr);
			return;
		}
		check_ins_letter(nr, letter, frame_acc);
	}		
	void close_ins(word nr, CFrameAccessor &frame_acc)
	{	check_nr(nr, false, frame_acc);
		if (s->in_del)
		{	PrintContext(frame_acc);
			printf("Fout(TC17): Er lijkt hier een begin van een toevoeging te ontbreken voor %d?", nr);
			s->in_del = false;
		}
		else if (!s->in_ins)
		{	PrintContext(frame_acc);
			printf("Fout(TC18): Herhaling van het einde van een toevoeging voor %d", nr);
		}
		s->in_ins = false;
	}
	void end(CFrameAccessor &frame_acc)
	{	
		for (word i = 0; i < 10; i++)
		{	if (states[i].in_del)
			{	PrintContext(frame_acc);
				printf("Fout(TC19): Het einde van een verwijdering ontbreekt voor %d", i);
			}
			if (states[i].in_ins)
			{	PrintContext(frame_acc);
				printf("Fout(TC20): Het einde van een toevoeging ontbreekt voor %d", i);
			}
		}
	}

	struct State
	{
		char mode;
		int nr;
		char letter;

		bool operator==(State &r)
		{	return mode == r.mode && nr == r.nr && letter == r.letter; }
		bool operator!=(State &r)
		{	return !(*this == r);
		}
		
		char *to_string()
		{	char *s = temp_char1000_buf();
			if (mode == 'd')
				sprintf(s, "d%d", nr == 0 ? 1 : nr);
			else if (mode == 'i')
				sprintf(s, "i%d%c", nr == 0 ? 1 : nr, letter);
			else if (mode == ' ')
				sprintf(s, "-");
			else
				sprintf(s, "%c", mode);
			return s;
		}
	};
	
	bool get_current_state(State &st, char *descr)
	{	
		bool error = false;
		st.mode = ' ';
		st.nr = 0;
		st.letter = ' ';
		descr[0] = '\0';
		for (word i = 0; i < 10; i++)
			if (states[i].in_del || states[i].in_ins)
			{	if (st.mode != ' ' || (states[i].in_del && states[i].in_ins))
					error = true;
				if (states[i].in_del)
				{	st.mode = 'd';
					st.nr = i;
					sprintf(descr+strlen(descr), "d%d ", i);
				}
				if (states[i].in_ins)
				{	st.mode = 'i';
					st.nr = i;
					st.letter = states[i].ins_letter;
					sprintf(descr+strlen(descr), "i%d%c ", i, states[i].ins_letter);
				}
			}
		return !error;
	}
  
  private: 
  	void check_nr(word &nr, bool open, CFrameAccessor &frame_acc)
  	{	if (nr > 9)
  		{	PrintContext(frame_acc);
  			printf("Fout(TC21): Illegaal getal %d", nr);
  			nr = 0;
  		}
  		if (multiple_known && multiple && nr == 0 && last_nr > 0)
  			nr = last_nr;
  		if (open)
  		{	if (multiple_known)
	  		{	if (!multiple && nr != 0)
	  			{	PrintContext(frame_acc);
	  				printf("Fout(TC22): Eerdere verwijdering/toevoeging moet een cijfer gebruiken");
	  				multiple = true;
	  			}
	  		}
	  		else
	  		{	multiple_known = true;
	  			multiple = nr > 0;
	  		}
	  		last_nr = nr;
	  	}
	  	else
	  	{	if (multiple_known && !multiple)
	  		{	if (nr > 0)
	  			{	PrintContext(frame_acc);
	  				printf("Fout(TC23): Overbodig gebruik van een cijfer");
	  				nr = 0;
	  			}
	  		}
	  	}
	  	s = &states[nr];
  	}
	void check_ins_letter(word nr, char letter, CFrameAccessor &frame_acc)
	{
		int b = ins_bit(letter);
		//printf("had_ins[%c]=%d(%d)", letter, s->had_ins, b);
		
		if (s->ins_letter != '\0' && (letter == ' ') != (s->ins_letter == ' '))
		{	PrintContext(frame_acc);
			printf("Fout(TC24): Inconsistent gebruik van de letters '%c' en '%c' voor %d", 
			       s->ins_letter, letter, nr);
		}
		else if (letter != ' ' && (s->had_ins & b) != 0)
		{	if (show_parse_greek_warnings)
			{	PrintContext(frame_acc);
				printf("Mededeling(TC25): De letter '%c' wordt herhaald voor %d", letter, nr);
			}
		}
		else if (s->ins_letter != '\0' && s->ins_letter != ' ' && letter != s->ins_letter + 1)
		{	PrintContext(frame_acc);
			printf("Fout(TC26): De letters '%c' and '%c' zijn niet oplopend voor %d", 
				   s->ins_letter, letter, nr);
		}
		s->ins_letter = letter;
		s->had_ins |= b;
		//printf("->%d", s->had_ins);
	}
	word ins_bit(char letter)
	{	return 1 << (('a' <= letter && letter <= 'c') ? letter - 'a' + 1 : 0);
	}
};



#include "output.cpp"
#include "output_rest.cpp"

void parse_peri_title(CFrameAccessor &frame_acc, CVerseRange &verse_range)
{	
	char first_line[1000];
	int i = 0;

	CTextOnFramesAccessor text_acc(frame_acc);

	for (text_acc.Init(); text_acc.More(); text_acc.Next())
		if (text_acc.ch >= ' ')
			break;
			
	for (; text_acc.More(); text_acc.Next())
	{	if (text_acc.mode != 0 && text_acc.ch != ' ')
			break;
		if (text_acc.ch >= ' ')
			first_line[i++] = text_acc.ch;
	}
	first_line[i] = '\0';

	if (!parse_verse_range(text_acc, verse_range, frame_acc))
		;
	else
	{	// Update context
		sprintf(context, "in perikoop titel");
		char *s = context + strlen(context);
		verse_range.print(s);
	}
	
	char rest[1000];
	i = 0;
	for (; text_acc.More(); text_acc.Next())
	{	if (text_acc.ch >= ' ')
			rest[i++] = text_acc.ch;
	}
	rest[i] = '\0';

	if (debug_parse_peri) 
	{
		PrintContext(frame_acc);
		printf(" parse_peri_title");
		printf(" Line1: |%s|", first_line);
		printf(" Verse range: %s", verse_range.str());
		printf(" The rest: |%s|", rest);
	}

	output_pericoop(verse_range, first_line, rest);
	
	frame_acc.Next();
}

#ifdef DO_FRAME_FOLLOW_CHECKING
void check_if_frames_follow(CFrameAccessor &prev_frame, CFrameAccessor &frame_acc)
{	CBox &prev_box = prev_frame->box;
	CBox &cur_box = frame_acc->box;

	if (cur_box.x1 < prev_box.x1)
	{	if (cur_box.x1 >   39000000)
		{	PrintContext(frame_acc);
	    	printf("Fout(FS01): First col1 frame at new line does not start at beginning.");
		}
		else if (cur_box.y1 < 7500000)
		{	// At new page
			printf("\nNew page, y = %10ld, x = %10ld", cur_box.y1, cur_box.x1);
		}
		else
		{	long dy = cur_box.y1 - prev_box.y2;
			if (dy < 190000 || dy > 230000)
			{	PrintContext(frame_acc);
				printf("Fout(FS02): First col1 frame at new line does not follow below, yd = %10ld, x = %10ld",
					   dy, cur_box.x1);
			}
		}
	}
	else
	{
		long dx = cur_box.x1 - prev_box.x2;
		long dy = cur_box.y1 - prev_box.y1;
		if (   dx < -300000 || dx > 900000
		    || dy <  -40000 || dy >  40000)
		{	PrintContext(frame_acc);
	    	printf("Fout(FS03): Col1 frame does not follow xd = %10ld, yd = %10ld", 
				   dx, dy);
		}
		else if (dx > 100000)
		{	if (debug_parse_col1) 
			{	PrintContext(frame_acc);
	    		printf("Waarschuwing(FS04): extra space between following frames");
	    	}
		}
	}
}
#endif // DO_FRAME_FOLLOW_CHECKING


CAllGreekWords *all_greek_words;

void parse_wordnr(char *s, word &nr, word &romnr, bool &inTR)
{	
	inTR = true;
	if (*s == '[')
	{	inTR = false;
		s++;
		if (s[strlen(s)-1] == ']')
			s[strlen(s)-1] = '\0';
	}
	bool isname = *s == '0';
	nr = 0;
	while (isdigit(*s))
	{	nr = 10*nr + *s - '0';
		s++;
	}
	if (isname)
		nr += 4932;
		
	if (*s == '\0')
		romnr = 1;
	else if (!strcmp(s, "II"))
		romnr = 2;
	else if (!strcmp(s, "III"))
		romnr = 3;
	else if (!strcmp(s, "IV"))
		romnr = 4;
}

	

int cur_book_nr = -1;


int nr_col1_word_number;
bool two_alter_greek_words;
WordNumber col1_word_number[3];
int nr_col1_greek_text;
char col1_greek_text[3][200];
int nr_col1_transl_text;
char col1_transl_text[3][200];
char col1_translation[200];
struct 
{	char letter;
	int pos;
} col1_punctuation[5];
int nr_col1_punct;

void parse_word_nr(char *s, CFrameAccessor &frame_acc)
{	char *r = s;

	nr_col1_word_number = 0;
	two_alter_greek_words = false;
	nr_col1_greek_text = 0;
	nr_col1_transl_text = 0;
	col1_translation[0] = '\0';
	nr_col1_punct = 0;

	while (*s != '\0')
	{
		bool word_in_TR = true;
		bool word_form_in_TR = true;
		char wn[100];
		char wfn[100];
		word i;
	
		for (; *s != '\0'; s++)
		{	if (('0' <= *s && *s <= '9') || *s == '[' || *s == 'l')
				break;
			PrintContext(frame_acc);
	    	printf("Waarschuwing(WN01): vreemd karakter '%c' (#%02X) in '%s' overgeslagen", *s ? *s : '?', *s, r);
		}
		
		if (*s == '[')
		{	word_in_TR = false;
			s++;
		}
		
		if (*s == '\0')
		{	PrintContext(frame_acc);
			printf("Fout(WN02): geen woordnummer gevonden in '%s'", r);
	    }
		
		i = 0;
		for (; ('0' <= *s && *s <= '9') || *s == 'l'; s++)
			wn[i++] = *s == 'l' ? '1' : *s;
		if	(*s == '[' && s[1] == 'I')
		{	PrintContext(frame_acc);
			printf("Waarschuwing(WN16): '[' moet vooraan staan in '%s'", r);
			word_in_TR = false;
			s++;
		}
		for (; *s == 'I' || *s == 'V'; s++)
			wn[i++] = *s;
		wn[i] = '\0';

		wfn[0] = '\0';
		if (*s == '.' || *s == ',' || *s == '[')
		{	if (*s == '[')
			{	PrintContext(frame_acc);
	    		printf("Waarschuwing(WN03): Verwacht een punt voor '[' in '%s'", r);
	    	}
			if (*s == '.')
				s++;
			if (*s == ',')
			{	PrintContext(frame_acc);
	    		printf("Waarschuwing(WN04): Er wordt een komma gebruikt i.p.v. een punt in '%s'", r);
				s++;
			}
		
			if (*s == '[')
			{	word_form_in_TR = false;
				if (!word_in_TR)
				{	PrintContext(frame_acc);
	    			printf("Waarschuwing(WN05): Vreemde combinatie van karakters in '%s'", r);
	    		}
				s++;
			}
			if (*s == '\0')
			{	PrintContext(frame_acc);
	    		printf("Waarschuwing(WN06): Geen cijfer na de punt in '%s'", r);
	    	}
			else if (   !('0' <= *s && *s <= '9')
			    && *s != 'I')
			{	PrintContext(frame_acc);
	    		printf("Waarschuwing(WN06): Vreemd karakter '%c'(#%02X)) na de punt", *s ? *s : '?', *s);
	    	}
			
			i = 0;
			for (; ('0' <= *s && *s <= '9') || *s == '/' || *s == 'I'; s++)
				wfn[i++] = *s;
			wfn[i] = '\0';
		}
		
		if ((!word_in_TR || !word_form_in_TR) != (*s == ']'))
		{	PrintContext(frame_acc);
	    	printf("Waarschuwing(WN07): ongebalanceerde combinatie van '[' en ']' in '%s'", r);
	    }
		if (*s == ']')
			s++;
		
		if (debug_parse_col1)
			printf("\nWord number %s.%s %s", 
		    	   wn, wfn, !word_in_TR ? "not in TR" : !word_form_in_TR ? "form not in TR" : "");

		if (nr_col1_word_number >= 3)
		{	PrintContext(frame_acc);
			printf("Fout(WN08): Te veel versnummers in eerste kolom");
		}
		else
		{	col1_word_number[nr_col1_word_number].init(word_in_TR, word_form_in_TR);
			col1_word_number[nr_col1_word_number].parse(wn, wfn);
			nr_col1_word_number++;
		}
				       
		if (*s == ' ' || *s == -54 || *s == '+')
		{	// -- some plus symbol
			if (*s == ' ' || *s == -54)
				s++;
			if (*s == '+' || *s == '|')
			{	if (*s == '|')
					two_alter_greek_words = true;
				s++;
			}
			else
			{	PrintContext(frame_acc);
	    		printf("Waarschuwing(WN09): Vreemd karakter '%c' (#%02X) in '%s' overgeslagen", *s ? *s : '?', *s, r);
				return;
			}
			if (*s == ' ' || *s == -54)
				s++;
			if (*s == '\0')
			{	PrintContext(frame_acc);
	    		printf("Fout(WN10): Niets na '+' in '%s'", r);
	    	}
		}
	}
	
	if (nr_col1_word_number == 0)
	{	PrintContext(frame_acc);
	    printf("Fout(WN11): Geen getal gevonden in '%s'", r); 
	}
	if (two_alter_greek_words)
		nr_col1_word_number = 1;
}

void printText(const char *s, const char *f, word from, word to, bool add_fnt = false)
{
	printf("|");
	for (word i = 0; *s && i < to; s++, f++, i++)
		if (from <= i)
		{	printf((*f == 'G' && *s < 0) ? "@%02X" : 
			        *s < 0               ? "#%02X" : 
			                               "%c", 
			       (byte)*s);
			if (add_fnt)
				printf("%c", *f);
		}
	printf("|");
}
void printText(const char *s, const char *f, bool add_fnt = false)
{
	printText(s,f,0,100,add_fnt);
}

bool is_greek_char(char ch, char fnt)
{	return    fnt == 'G' 
		   && (   (ch < 0 && ch >= -125 && ch != (char)0xAF)
			   || ('A' <= ch && ch <= 'Z')
			   || ('a' <= ch && ch <= 'z')
			   || ch == '\\'
			   || ch == '^'
			   || ch == '\''
			   || (byte)ch == 128
			   || ch == 'q');
}

CTextCritState::State col1_greek_tc_state[3];
CTextCritState::State col1_punct_tc_state[5];

void report_transl_greek_warn(const char *warncode,
							  const char *info, const char *greek, 
							  const char *transl, CFrameAccessor &frame_acc,
							  word warnings)
{
	if (warnings != 0 && show_transliteration_warnings)
	{	PrintContext(frame_acc);
		printf("Waarschuwing(%s): Transliteratie %s past niet bij het Grieks %s (%s) wat betreft:",
			   warncode,
			   transl_to_text(transl), 
			   greek_to_text(greek),
			   transl_to_text(transliterate(greek)));
		print_transl_diffs(warnings);
		printf(".");
	}
}

bool check_transl_with_greek(const char *errcode, const char *warncode, 
							 const char *corrcode,
							 const char *info, const char *greek, 
							 char *transl, CFrameAccessor &frame_acc)
{
	word warnings;
	char corr_transl[200];
	bool correct = compare_transl_with_greek(transl, greek, warnings, corr_transl);

	if (!correct)
	{	PrintContext(frame_acc);
		printf("Fout(%s): Transliteratie %s past niet bij Grieks %s (%s).",
			   errcode,
			   transl_to_text(transl), 
			   greek_to_text(greek),
			   transl_to_text(transliterate(greek)));
	}
	else if (warnings != 0 && (warnings & ~TR_CORR) == 0)
	{	PrintContext(frame_acc);
		printf("Correctie(%s): Transliteratie %s vervangen door %s dat past bij %s",
			   corrcode,
			   transl_to_text(transl),
			   transl_to_text(corr_transl),
			   greek_to_text(greek));
		strcpy(transl, corr_transl);
	}
	else
		report_transl_greek_warn(warncode, info, greek, transl, frame_acc, warnings);
	return correct;
}


void search_for_alternative_word_form(CGreekWord *greek_word, CGreekWordForm *greek_word_form, 
									  char *greek, 
									  lword &best_diff, 
									  char *best_word_form,
									  CGreekWordForm *&alt_greek_word_form)
{	for (lword i = 0; i < greek_word->nr_word_forms; i++)
	{	CGreekWordForm *o_greek_word_form = greek_word->word_forms[i];
		if (o_greek_word_form != greek_word_form)
		{
			if (compare_greek_word(greek, o_greek_word_form->greek, best_diff,
								   best_word_form))
			{	alt_greek_word_form = o_greek_word_form;
				if (best_diff == 0)
					return;
			}
		}
	}
}

void search_for_alternative_word(CGreekWord *greek_word, 
								 char *greek, 
								 lword &best_diff, 
								 char *best_word_form,
								 word &best_nr, word &best_romnr,
								 CGreekWord *&alt_greek_word,
								 CGreekWordForm *&alt_greek_word_form)
{	alt_greek_word = NULL;
	alt_greek_word_form = NULL;
	
	for (int nr = 0; nr <= 5516; nr++)
	{	
		for (int romnr = 1; romnr < 5; romnr++)
		{	CGreekWord *o_greek_word = all_greek_words->getWord(nr, romnr - 1);
			if (o_greek_word == NULL)
				break;
			
			if (o_greek_word != greek_word)
			{	
				CGreekWordForm *o_greek_word_form = NULL;
				lword o_best_diff = best_diff;
				char o_best_word_form[200];
				search_for_alternative_word_form(o_greek_word, NULL, 
											 	 greek, 
												 o_best_diff, o_best_word_form,
												 o_greek_word_form);
				if (o_best_diff < best_diff && DIFF_VARIATION(o_best_diff))
				{	best_diff = o_best_diff;
					best_nr = nr;
					best_romnr = romnr;
					strcpy(best_word_form, o_best_word_form);
					alt_greek_word = o_greek_word;
					alt_greek_word_form = o_greek_word_form;
					if (best_diff == 0)
						return;
				}
			}
		}
	}
}

void report_greek_word_diff(const char *info, char *greek, 
							const char *ws_greek, lword best_diff,
							char *best_word_form, CFrameAccessor &frame_acc, bool report)
{
	if (DIFF_SUBSTANTIAL(best_diff))
	{
		PrintContext(frame_acc);
		printf("Fout(GW12): Grieks %s verschilt met %s uit woordstudie", 
			   greek_to_text(greek), greek_to_text(ws_greek));
		//DEBUGDUMP;
	
		//for (char *ri = greek; *ri; ri++)
		//	printf("%02X", (byte)*ri);
		//printf(" ");
		//for (const char *si = greek_word_form->greek; *si; si++)
		//	printf("%02X", (byte)*si);
		return;
	}

	if (best_diff && (report || show_match_greek_warnings))
	{
		PrintContext(frame_acc);
		printf("Waarschuwing(GW14): Grieks %s verschilt met %s uit woordstudie wat betreft",
			   greek_to_text(greek), greek_to_text(best_word_form));
		print_greek_word_diff(best_diff);
	}
}

// Forward declaration:
void new_verse_recent_greek_words();
void add_recent_greek_word(CGreekWord* greek_word, int number, int romnumber);

void check_word(const char* info,
			    WordNumber* number, 
				char* greek, char* transl, CTextCritState::State &tc,
				CFrameAccessor &frame_acc)
{
	DEBUGSTART
	DEBUG2("\n gr: '%s' tr: '%s'", greek_to_text(greek), transl_to_text(transl));	

	// Find word and word form from given number
	CGreekWord *greek_word = all_greek_words->getWord(number->word_nr, 
													  number->word_romnr - 1);
	CGreekWordForm *greek_word_form = NULL;
	if (greek_word != NULL)
		greek_word_form = greek_word->getWordFormByNumber(number->form_nr);
   	DEBUG1("\n greek_word %s", greek_word ? "found" : "not found");
	if (greek_word_form)
		DEBUG2(", greek_word_form gr: '%s' tr: '%s'",
			   greek_to_text(greek_word_form->greek),
			   transl_to_text(greek_word_form->transl));

	bool err_no_greek_word = greek_word == NULL;
	bool err_no_greek_word_form = greek_word_form == NULL;

	// Compare Greek with given number
	lword best_diff = DIFF_ALL;
	char best_greek[200];
	strcpy(best_greek, "**");
	if (greek_word_form)
	{
		compare_greek_word(greek, greek_word_form->greek, best_diff, best_greek);
		DEBUG1("\n compare_greek_word %lX", best_diff);
	}

	// Compare transliteration with number
	bool transl_number_conflicts = false;
	word transl_number_warning = 0;
	char best_transl[200];
	strcpy(best_transl, "**");
	if (greek_word_form)
	{	if (!compare_transl_with_transls(transl, greek_word_form->transl, 
										 transl_number_warning, best_transl))
			transl_number_conflicts = true;
	}

	// Compare transliteration with Greek
	word transl_greek_warning = 0;
	char corr_transl[200];
	bool transl_greek_matches 
	= compare_transl_with_greek(transl, greek, transl_greek_warning, corr_transl);
														 
	// Remember original
	WordNumber orig_number;
	orig_number.init(number);
	CGreekWordForm *orig_greek_word_form = greek_word_form;
	
	if (greek_word)
	{	number->word_inTR = greek_word->inTR;
		number->has_forms = (greek_word->nr_word_forms > 1);
		if (greek_word_form)
			number->form_inTR = greek_word_form->inTR;
	}
	DEBUG3("\n search alt, if %d || (%d && %d)", greek_word_form == NULL, best_diff > DIFF_ACC, transl_number_conflicts);
	
	// Search for alternative word form, if number not found 
	// or if Greek and transliteration differ from those with number
	bool changed_word = false;
	bool changed_word_form = false;
	if (greek_word_form == NULL || (best_diff > DIFF_ACC && transl_number_conflicts))
	{	// -- looks like number is wrong
		
		if (greek_word)
		{	
			DEBUG("\n search form");
			CGreekWordForm *alt_greek_word_form = NULL;
			search_for_alternative_word_form(greek_word, greek_word_form, greek,
										     best_diff, best_greek,
											 alt_greek_word_form);
			if (alt_greek_word_form)
			{   DEBUG2("-- found gr: '%s' tr: '%s'", 
					   greek_to_text(alt_greek_word_form->greek), 
					   transl_to_text(alt_greek_word_form->transl));	
				changed_word_form = true;
				greek_word_form = alt_greek_word_form;
				number->form_nr = alt_greek_word_form->form_number;
				number->form_inTR = alt_greek_word_form->inTR;
			}
		}
		
		if (DIFF_SUBSTANTIAL(best_diff))
		{	// -- no form of word matched with Greek
		
			DEBUG("\n search word");
			word nr, romnr;
			CGreekWord *alt_greek_word = NULL;
			CGreekWordForm *alt_greek_word_form = NULL;
			search_for_alternative_word(greek_word, greek,
										best_diff, best_greek,
										nr, romnr,
										alt_greek_word, alt_greek_word_form);
			if (alt_greek_word_form)
			{	DEBUG2("-- found gr: '%s' tr: '%s'", 
					   greek_to_text(alt_greek_word_form->greek), 
					   transl_to_text(alt_greek_word_form->transl));	
				changed_word = true;
				greek_word = alt_greek_word;
				greek_word_form = alt_greek_word_form;
				number->word_nr = nr;
				number->word_romnr = romnr;
				number->word_inTR = alt_greek_word->inTR;
				number->has_forms = alt_greek_word->nr_word_forms > 1;
				number->form_nr = alt_greek_word_form->form_number;
				number->form_inTR = alt_greek_word_form->inTR;
			}
		}
	}
	
	// Report any corrections, errors and warnings with number
	if (changed_word || changed_word_form)
	{
		transl_number_conflicts
		= !compare_transl_with_transls(transl, greek_word_form->transl, 
									   transl_number_warning, best_transl);
		PrintContext(frame_acc);
		if (   transl_greek_matches
		    && best_diff <= DIFF_ACC && !transl_number_conflicts)
			printf("Correctie(GW06): Nummer %s vervangen door %s.",
				   orig_number.word_form_str(), number->word_form_str());
		else
			printf("Fout(GW07): Nummer %s beter dan %s.",
				   number->word_form_str(), orig_number.word_form_str());
				
		if (err_no_greek_word)
			printf(" Nummer komt niet voor in woordstudie.");
		else if (err_no_greek_word_form)
			printf(" Geen woordvorm met dit nummer.");
		else if (best_diff == 0 && !transl_number_conflicts)
			printf(" Nummer is verkeerd.");
		else
			printf(" Woord bij gegeven vormnummer %s (%s) '%s', woord in de tekst %s (%s), woord bij alternatief vormnummer %s (%s) '%s'",
				   greek_to_text(orig_greek_word_form->greek),
				   transl_to_text(orig_greek_word_form->transl),
				   orig_greek_word_form->vert.A() ? orig_greek_word_form->vert.A() : "",
				   greek_to_text(greek), transl_to_text(transl),
				   greek_to_text(best_greek),
				   transl_to_text(best_transl),
				   greek_word_form->vert.A() ? greek_word_form->vert.A() : "");
		//DEBUGDUMP;
	}
	else if (orig_greek_word_form == NULL)
	{	PrintContext(frame_acc);
		printf("Fout(GW08): Vormnummer %s komt niet voor in woordstudie.",
			   orig_number.word_form_str());
	}
	else if (   number->word_inTR != orig_number.word_inTR
			 || number->has_forms != orig_number.has_forms
			 || number->form_inTR != orig_number.form_inTR)
	{	PrintContext(frame_acc);
		printf("Correctie(GW09): Nummer %s is vervangen door %s",
			   orig_number.word_form_str(), number->word_form_str());
		if (number->word_inTR != orig_number.word_inTR)
			printf(" omdat woord %s in TR voorkomt", number->word_inTR ? "wel" : "niet"); 
		if (number->form_inTR != orig_number.form_inTR)
			printf(" omdat woordvorm %s in TR voorkomt", number->form_inTR ? "wel" : " niet"); 
		if (number->has_forms != orig_number.has_forms)
			printf(" omdat woord %s vorm heeft", number->has_forms ? "meer dan een" : "maar een");
	}
	
	// Checking Greek
	if (greek_word_form == NULL)
	{	PrintContext(frame_acc);
		printf("Fout(GW10): Geen passende woordvorm gevonden voor Grieks %s",
			   greek_to_text(greek));

		// Report differences between transliteration and Greek
		check_transl_with_greek("TL01", "TL02", "TL17", info, greek, transl, frame_acc);
	}
	else
	{
		if (DIFF_VARIATION(best_diff))
		{	// Greek word matches with given number, except maybe for some warnings.
			
			char alt_transl[200];
			
			word transl_greeks_warning = 0;
			bool transl_greeks_matches 
			= compare_transl_with_greeks(transl, greek_word_form->greek,
										 transl_greeks_warning, best_greek);
											    
			if (   !transl_greek_matches
				&& best_diff != DIFF_DEL_EPSILON
			    && (transl_greeks_matches && transl_greeks_warning == 0))
			{	PrintContext(frame_acc);
				if (best_diff & DIFF_DEL_LOW_I_PRE)
					printf("Correctie(GW98): Grieks %s vervangen door %s. Weglating subscript iota t.o.v. woordstudie en transliteratie %s.",
						   greek_to_text(greek),
						   greek_to_text(best_greek),
						   transl_to_text(transl));
				else
					printf("Fout(GW97): Grieks %s past niet bij woordstudie %s en transliteratie %s",
						   greek_to_text(greek),
						   greek_to_text(best_greek),
						   transl_to_text(transl));
			}
			else
			{
				report_greek_word_diff(info, greek, greek_word_form->greek, best_diff,
									   best_greek, frame_acc, false);

				if (transl_greek_matches)
				{	// Transliteration matches with Greek

					if (transl_greek_warning == 0)
						; // er zijn ook geen waarschuwingen
					else if (!transl_number_conflicts && transl_number_warning == 0)
					{	
						if ((transl_greek_warning & ~TR_CORR) != 0)
						{
							report_transl_greek_warn("TL13", info, greek, transl, frame_acc,
													 transl_greek_warning);
							printf(" Maar wel met transliteratie %s woordstudie",
								   transl_to_text(greek_word_form->transl));
						}
					}
					else if ((transl_greek_warning & ~TR_CORR) == 0)
					{	
						PrintContext(frame_acc);
						printf("Correctie(TL16): Transliteratie %s vervangen door %s dat past bij %s",
							   transl_to_text(transl),
							   transl_to_text(corr_transl),
							   greek_to_text(greek));
					}
					else if (transl_greeks_matches && transl_greeks_warning == 0)
					{	report_transl_greek_warn("TL14", info, greek, transl, frame_acc,
												 transl_greek_warning);
						printf(" Maar wel met Grieks %s (%s) woordstudie",
							   greek_to_text(greek_word_form->greek),
							   transl_to_text(transliterate(greek_word_form->greek)));
					}
					else
						report_transl_greek_warn("TL15", info, greek, transl, frame_acc,
												 transl_greek_warning);
				}
				else if (transl_greeks_matches)
				{
					PrintContext(frame_acc);
					printf("Waarschuwing(TL09): Transliteratie %s past niet bij Grieks %s, maar wel bij Grieks %s (%s) woordstudie",
						   transl_to_text(transl),
						   greek_to_text(greek),
						   greek_to_text(best_greek),
						   transl_to_text(transliterate(best_greek)));
					if (transl_greeks_warning != 0)
					{	printf(" afgezien van: ");
						print_transl_diffs(transl_greeks_warning);
					}
					printf(".");
				}
				else if (!transl_number_conflicts)
				{
					PrintContext(frame_acc);
					printf("Waarschuwing(TL10): Transliteratie %s past niet bij Grieks %s, maar wel bij transliteratie %s woordstudie",
						   transl_to_text(transl),
						   greek_to_text(greek),
						   transl_to_text(greek_word_form->transl));
					if (transl_number_warning != 0)
					{	printf(" afgezien van: ");
						print_transl_diffs_inv(transl_number_warning);
					}
					printf(".");
				}
				else if (find_transl_matching_greek(greek_word_form->transl, alt_transl, 
													greek))
				{	PrintContext(frame_acc);
					printf("Correctie(TL03): Transliteratie %s vervangen door %s dat past bij %s",
						   transl_to_text(transl), transl_to_text(alt_transl),
						   greek_to_text(greek));
					strcpy(transl, alt_transl);
				}
				else
				{	PrintContext(frame_acc);
					printf("Fout(TL11): Transliteratie %s past niet bij Grieks %s. Vervang door %s?",
						   transl_to_text(transl),
						   greek_to_text(greek),
						   transl_to_text(transliterate(greek)));
				}
			}
		}
		else
		{	
			DEBUG4("\n can correct %d && (%d || %d || %lX)", greek_word_form != NULL, !transl_greek_matches, transl_greek_warning != 0, best_diff);
			char corr_greek[200];
			best_diff = greek_word_diff(greek, best_greek, corr_greek);
			dd = false;
			
			if (   DIFF_CORRECTING(best_diff) 
			    && (   (best_diff&(DIFF_NU_UPSILON|DIFF_IGN_FIRST)) == 0
			        || !transl_greek_matches))
			{	PrintContext(frame_acc);
				printf("Correctie(GW99): Grieks %s vervangen door %s (naar %s uit woordstudie)",
					   greek_to_text(greek),
					   greek_to_text(corr_greek),
					   greek_to_text(best_greek));
				strcpy(greek, corr_greek);
				compare_greek_word(greek, greek_word_form->greek, best_diff,
								   best_greek);
			}
			else if (   !transl_greek_matches //|| transl_greek_warning != 0 
		        	 || (best_diff & DIFF_ASP != 0))
			{
				DEBUG3("\n try correct %d %d %d", best_diff > DIFF_ACC, transl_number_conflicts, transl_number_warning);
				
				if (   (best_diff > DIFF_ACC && !transl_number_conflicts
				    || (   best_diff > 0 
				        && !transl_number_conflicts && transl_number_warning == 0)))
				{	// Looks like Greek number does not match
					
					char alt_greek[200];
					
					if (find_greek_matching_transl(greek_word_form->greek, alt_greek, transl))
					{	PrintContext(frame_acc);
						printf("Fout(GW11): Grieks %s past niet bij transliteratie %s. Vervangen door %s?",
							   greek_to_text(greek),
							   transl_to_text(transl), 
							   greek_to_text(alt_greek));
						strcpy(greek, alt_greek);
						compare_greek_word(greek, greek_word_form->greek, best_diff, 
										   best_greek);
					}
				}
				
				if (best_diff <= DIFF_ACC && (transl_number_conflicts || transl_number_warning))
				{
					char alt_transl[200];
					word alt_transl_greek_warning;
					if (   find_transl_matching_greek(greek_word_form->transl, alt_transl, 
													  greek)
						&& compare_transl_with_greek(alt_transl, greek,
													 alt_transl_greek_warning, 0)
						&& alt_transl_greek_warning == 0)
					{	PrintContext(frame_acc);
						printf("Correctie(TL12): Transliteratie %s vervangen door %s dat past bij %s",
							   transl_to_text(transl), transl_to_text(alt_transl),
							   greek_to_text(greek));
						strcpy(transl, alt_transl);
						transl_number_conflicts 
						= !compare_transl_with_transls(transl, greek_word_form->transl, 
													   transl_number_warning, best_transl);
						transl_greek_matches = true;
						transl_greek_warning = alt_transl_greek_warning;
					}
				}
			}
			
			// Report differences between Greek and Greek in word study
			report_greek_word_diff(info, greek, greek_word_form->greek, best_diff,
								   best_greek, frame_acc, false);
				
			// Report differences between transliteration and Greek
			check_transl_with_greek("TL01", "TL02", "TL17", info, greek, transl, frame_acc);
			
			// Report differences between transliteration and transliteration in word study
			if (!transl_greek_matches)
			{
				if (transl_number_conflicts)
				{	PrintContext(frame_acc);
					printf("Fout(TL04): Transliteratie %s verschilt met %s uit woordstudie.",
						   transl_to_text(transl), 
						   transl_to_text(greek_word_form->transl));
					//DEBUGDUMP;
				}
				else if (transl_number_warning != 0)
				{	PrintContext(frame_acc);
					printf("Waarschuwing(TL05): Transliteratie %s verschilt met %s uit woordstudie wat betreft ",
						   transl_to_text(transl), 
						   transl_to_text(greek_word_form->transl));
					print_transl_diffs(transl_number_warning);
					//DEBUGDUMP;
				}
			}
		}
	}

#ifdef REMOVE_DOUBLE_WORD_FORMS
	if (greek_word_form != 0 && greek_word_form->as != 0)
	{	//PrintContext(frame_acc);
		//printf("Info(WN33): Using form number %d instead of %d",
		//	   greek_word_form->as->form_number, number->form_nr);
		number->form_nr = greek_word_form->as->form_number;
	}
#endif
	
	// For speeding up recognition of greek words in commentary
	if (greek_word)
		add_recent_greek_word(greek_word, number->word_nr, number->word_romnr);

	if (two_alter_greek_words)
	{
		WordNumber* sec_number = &col1_word_number[1];
		CGreekWord *sec_greek_word = all_greek_words->getWord(sec_number->word_nr, 
															  sec_number->word_romnr - 1);
		CGreekWordForm *sec_greek_word_form = 0;
		if (sec_greek_word == NULL)
		{	PrintContext(frame_acc);
			printf("Fout(WN12): Nummer %s niet gevonden in woordstudies.",
				   sec_number->word_str());
		}
		else
		{	sec_greek_word_form = sec_greek_word->getWordFormByNumber(sec_number->form_nr);
			if (sec_greek_word_form == NULL)
			{	PrintContext(frame_acc);
				printf("Fout(WN13): Vormnummer %s niet gevonden in woordstudie.",
					   sec_number->word_form_str());
			}
			else
			{	lword sec_best_diff = DIFF_ALL;
				char sec_best_greek[200];
				compare_greek_word(greek, sec_greek_word_form->greek, 
								   sec_best_diff, sec_best_greek);
				
				report_greek_word_diff("second alternative", greek, 
									   sec_greek_word_form->greek, sec_best_diff,
									   sec_best_greek, frame_acc, true);
				
			}

			// For speeding up recognition of greek words in commentary
			add_recent_greek_word(sec_greek_word, sec_number->word_nr, sec_number->word_romnr);
		}
#ifdef REMOVE_DOUBLE_WORD_FORMS
		if (sec_greek_word_form != 0 && sec_greek_word_form->as != 0)
		{	//PrintContext(frame_acc);
			//printf("Info(WN34): Using form number %d instead of %d",
			//	   sec_greek_word_form->as->form_number, sec_number->form_nr);
			sec_number->form_nr = sec_greek_word_form->as->form_number;
		}
#endif
		output_word(number, sec_number, greek, transl, tc);
	}
	else
		output_word(number, NULL, greek, transl, tc);

	
	// ToDo: Store information
}

void check_word_group(CFrameAccessor &frame_acc)
{
	output_start_word_group();

	if (nr_col1_word_number == 1 && nr_col1_greek_text == 2)
	{	PrintContext(frame_acc);
		printf("Fout(C123): Twee Griekse woorden worden als een geheel beschouwd");
		strcat(col1_greek_text[0], " ");
		strcat(col1_greek_text[0], col1_greek_text[1]);
		nr_col1_greek_text = 1;
		if (nr_col1_transl_text == 2)
		{	printf(" (en ook voor de transliteratie)");
			strcat(col1_transl_text[0], " ");
			strcat(col1_transl_text[0], col1_transl_text[1]);
			nr_col1_transl_text = 1;
		}
		printf(" - Ik geef het op");
	}
	else if (nr_col1_word_number != nr_col1_greek_text)
	{	PrintContext(frame_acc);
		printf("Fout(C101): Het aantal Griekse woord (%d) komt niet overeen met het aantal nummers (%d) - Ik geef het op",
			   nr_col1_greek_text, nr_col1_word_number);
			   
	}
	else
	{
		if (nr_col1_transl_text == 2 && nr_col1_greek_text == 1)
		{	PrintContext(frame_acc);
			printf("Waarschuwing(TR01): Spatie tussen '%s' en '%s' overgeslagen",
				   transl_to_text(col1_transl_text[0]),
				   transl_to_text(col1_transl_text[1]));
			strcat(col1_transl_text[0], col1_transl_text[1]);
			nr_col1_transl_text = 1;
		}
		
		if (nr_col1_word_number != nr_col1_transl_text)
		{	PrintContext(frame_acc);
			printf("Fout(C102): Het aantal transliteraties (%d) komt niet overeen met het aantal nummers (%d) - Ik geef het op",
				   nr_col1_transl_text, nr_col1_word_number);
		}
		else
		{
			char *word_nr_desc[3] = { "voor het eerste woord ", "voor het tweede woord ", "voor het derde woord " };
			int k = 0;
			for (int i = 0; i <= nr_col1_word_number; i++)
			{	
				for (; k < nr_col1_punct && col1_punctuation[k].pos == i; k++)
					output_punctuation(col1_punctuation[k].letter, 
									   col1_punct_tc_state[k]);
			
				if (i < nr_col1_word_number)
					check_word(nr_col1_word_number == 1 ? "" : word_nr_desc[i],
							   &col1_word_number[i],
							   col1_greek_text[i], col1_transl_text[i],
							   col1_greek_tc_state[i],
							   frame_acc);
			}
		}
	}
	output_word_group_translation(col1_translation);
	output_end_word_group();
}

void replace(byte b, char ch, char *transl, int &i, CFrameAccessor &frame_acc)
{	PrintContext(frame_acc);
	printf("Waarschuwing(TL06): '%c' vervangen door '%c' in transliteratie", b, ch);
	if (i < 99) 
		transl[i++] = ch;
}

void parse_greek_word(char *text, char *fnt, CTextCritState &textCrit, CFrameAccessor &frame_acc)
{
	word i = 0;
	if ((byte)text[i] == 213)
	{	if (show_parse_greek_warnings)
		{	PrintContext(frame_acc);
			printf("Waarschuwing(GW01): Karakter #D5 overgeslagen aan het begin van het Griekse woord");
			printText(text, fnt, true);
		}
		i++;
		if (text[i] == ' ')
			i++;
	}

	if (debug_parse_greek)
	{	printf("\n====");
		printText(text, fnt, true);
	}
	
	for(;text[i] != '\0';)
	{
		for(;;)
		{	char letter = ' ';
			word nr = 0;
			bool nr_found = false;

			while (text[i] == ' ')
				i++;
			if (text[i] == '\0')
				return; // --ready
		
			if (text[i] == '*')
			{	textCrit.has_remark = true;
				i++;
				if (text[i] == ' ')
					i++;
			}
			
			if (text[i] == 'a' || text[i] == 'b' || text[i] == 'c')
			{	// subvariant
				letter = text[i]; 
				//PrintContext(frame_acc);
				//printf("== abc ");
				//printText(text, fnt, true);
				i++;
			}
			else if ('1' <= text[i] && text[i] <= '9')
			{	nr = text[i] - '0';
				nr_found = true;
				i++;
			
				//printf("\n==%s %s", text, fnt);
					
				if (text[i] == '1' && fnt[i] == 'T')
				{	PrintContext(frame_acc);
					printf("Waarschuwing(GW02): Dubbele 1 in Griekse woord is overgeslagen");
					i++;
				}
					
				if (text[i] == 'a' || text[i] == 'b')
				{	if (letter != ' ')
					{	PrintContext(frame_acc);
						printf("Fout(TC27): Twee letters, namelijk '%c' en '%c'", letter, text[i]);
					}
					letter = text[i];
					i++;
				}
			}
			else if (fnt[i] == 'G'
					 && (   (byte)text[i] == 129 || (byte)text[i] == 130
					     || (byte)text[i] == 132))
			{	nr = (byte)text[i] - 128;
				if (nr > 3) nr--;

				if (text[i+1] == 'a' || text[i+1] == 'b')
				{	if (letter != ' ')
					{	PrintContext(frame_acc);
						printf("Fout(TC28): Twee letters, namelijk '%c' en '%c'", letter, text[i+1]);
					}
					nr_found = true;
					letter = text[i+1];
					i += 2;
				}
				else if (fnt[i+1] != 'G' || text[i+1] == '[')
				{	nr_found = true;
					i++;
				}
			}
			if (text[i] == ' ')
				i++;
		
			//printf(" # %d %d %d", nr_found, text[i], fnt[i]);
			bool open_del = false;
			bool close_del = false;
			bool open_ins = false;
			bool close_ins = false;
			if (text[i] == 'o' && (fnt[i] == '1' || fnt[i] == '2'))
			{	if (letter != ' ')
				{	PrintContext(frame_acc);
					printf("Fout(TC29): 'o'-teken (begin verwijdering) overgeslagen");
				}
				else
					open_del = true;
				i++;
				if (text[i] == ' ')
					i++;
			}
			if ((text[i] == '\'' || text[i] == '`') && (fnt[i] == '1' || fnt[i] == '2'))
			{	close_del = true;
				i++;
				if (text[i] == ' ')
					i++;
			}
			if (text[i] == '[')
			{	open_ins = true;
				i++;
				if (text[i] == ' ')
					i++;
			}
			if(text[i] == ']')
			{	close_ins = true;
				i++;
				if (text[i] == ' ')
					i++;
			}
			
			if (text[i] == 'a' || text[i] == 'b' || text[i] == 'c')
			{	if (letter != ' ')
				{	PrintContext(frame_acc);
					printf("Fout(TC30): Twee letters, namelijk '%c' en '%c'", letter, text[i]);
				}
				letter = text[i]; 
				i++;
				if (text[i] == ' ')
					i++;
				if (text[i] == 'o' && (fnt[i] == '2' || fnt[i] == '1'))
				{	PrintContext(frame_acc);
					printf("Fout(TC31): 'o'-teken (begin verwijdering) overgeslagen");
					i++;
				}
			}
			
			if (   (fnt[i] == '1' || fnt[i] == '2') 
			    && text[i] != ',' && text[i] != '\0')
			{	bool first = true;
				word j = i;
				while (fnt[j] == '1' || fnt[j] == '2' && text[i] != '\0')
				{	if (   text[i] != '\0' 
						&& (byte)text[i] != 0xD5
						&& (byte)text[i] != 0xD4)
					{	if (first)
						{	PrintContext(frame_acc);
							printf("Waarschuwing(GW03): Speciale karakters in het Griekse woord overgeslagen:");
						}
						first = false;
						printf(" %d", (byte)text[j]);
					}
					j++;
				}
				//printText(text, fnt, i, j);
				i = j;
			}
			
			if (!open_del && !close_del && !open_ins && !close_ins && letter == ' ')
			{	if (nr_found)
				{	PrintContext(frame_acc);
					if (nr == 2 && textCrit.inside_insert())
					{	letter = 'a'-1 + nr;
						printf("Fout(TC32): Losstaan nummer %d gevonden; Gelezen als '%c' karakter", 
							   nr, 'a' + nr - 1);
						nr = 0;
					}						
					else
					{	printf("Fout(TC33): Losstaan nummer %d gevonden; Beschouwd als begin van verwijdering", nr);
						open_del = true;
					}
					//printText(text, fnt, 0, i, true);
					//printf("   ");
					//printText(text, fnt, i, 100, true);
				}
				else
					break;
			}

			if (debug_parse_greek)
			{	printf(" %d", nr);
				if (letter != ' ')
					printf("%c", letter);
				printf("%s%s%s%s",
					   open_del ? "(" : "",
					   close_del ? ")" : "",
					   open_ins ? "[" : "",
					   close_ins ? "]" : "");
			}

			if (letter != ' ' && (open_del || close_del))
			{	PrintContext(frame_acc);
				printf("Fout(TC34): Letter bij begin of einde van verwijdering overgeslagen");
				letter = ' ';
			}
			if (open_del && close_del)
			{	PrintContext(frame_acc);
				printf("Fout(TC35): Combinatie van begin en einde van verwijdering overgeslagen");
				open_del = false;
				close_del = false;
			}
			if (open_ins && close_ins)
			{	PrintContext(frame_acc);
				printf("Fout(TC36): Combinatie van begin en einde van toevoeging overgeslagen");
				open_ins = false;
				close_ins = false;
			}

			if (close_ins)
			{	
				if (open_del)
				{	PrintContext(frame_acc);
					printf("Fout(TC37): Begin van verwijdering overgeslagen voor einde van toevoeging");
				}
				if (close_del)
				{	PrintContext(frame_acc);
					printf("Fout(TC38): Einde van verwijdering overgeslagen voor einde van toevoeging");
				}
				textCrit.close_ins(nr, frame_acc);
			}
			else if (open_ins)
			{
				if (open_del)
				{	PrintContext(frame_acc);
					printf("Fout(TC39): Begin van verwijdering overgeslagen voor begin van toevoeging");
				}
				if (close_del)
				{	PrintContext(frame_acc);
					printf("Fout(TC40): Einde van verwijdering overgeslagen voor begin van toevoeging");
				}
				textCrit.open_ins(nr, letter, frame_acc);
			}
			else if (close_del)
				textCrit.close_del(nr, frame_acc);
			else if (open_del)
				textCrit.open_del(nr, frame_acc);
			else if (letter != ' ')
				textCrit.ins_letter(nr, letter, frame_acc);
		}

		while (text[i] == ' ')
			i++;
		if (text[i] == '\0')
			return; // --ready
	
		if (is_greek_char(text[i], fnt[i]) || (text[i] == (char)0xD5 && fnt[i] != 'G'))
		{	//char text_greek_word[200];
		
			if (nr_col1_greek_text >= 3)
			{	PrintContext(frame_acc);
				printf("Fataal(WN14): Te veel Griekse woorden in - Alleen eerste twee worden bekeken");
				nr_col1_greek_text = 2;
			}

			word j = 0;	
			if (debug_parse_greek) printf(" W");
			while (is_greek_char(text[i], fnt[i]) || (text[i] == (char)0xD5 && fnt[i] != 'G'))
			{	//printf("%02X", (byte)c);
				if (text[i] == (char)0xD5 && fnt[i] != 'G')
				{	PrintContext(frame_acc);
					printf("Waarschuwing(GW04): Karakter S2(213) overgeslagen");
				}
				else
					col1_greek_text[nr_col1_greek_text][j++] = text[i];
				i++;
			}
			col1_greek_text[nr_col1_greek_text][j] = '\0';
			char tc_state[400];
			if (!textCrit.get_current_state(col1_greek_tc_state[nr_col1_greek_text],
											tc_state))
			{	PrintContext(frame_acc);
				printf("Fout(TC41): Complexe tekstkritische toestand: %s", tc_state);
			}
			else if (   nr_col1_greek_text > 0
					 && col1_greek_tc_state[0] != col1_greek_tc_state[nr_col1_greek_text])
			{	PrintContext(frame_acc);
				printf("Fout(TC42): Woord groep met verschillende tekstkritische toestand %s en %s",
					   col1_greek_tc_state[0].to_string(),
					   col1_greek_tc_state[nr_col1_greek_text].to_string());
			}
			nr_col1_greek_text++;
			
			//output_word_found();
		}
		else if (   text[i] == ','
				 || (   fnt[i] == 'G' 
		             && (text[i] == '.' || text[i] == ';' || text[i] == '?' || text[i] == (char)0xAF)))
		{	
			col1_punctuation[nr_col1_punct].letter = text[i];
			col1_punctuation[nr_col1_punct].pos = nr_col1_greek_text;
			i++;
			char tc_state[400];
			if (!textCrit.get_current_state(col1_punct_tc_state[nr_col1_punct],
											tc_state))
			{	PrintContext(frame_acc);
				printf("Fout(TC43): Complexe tekstkritische toestand: %s", tc_state);
			}
			nr_col1_punct++;
		}
		else
		{	PrintContext(frame_acc);
			printf("Fout(GW05): Vreemd karakter '%c' (#%02X) in Griekse woord", (byte)text[i], (byte)text[i]);
			printText(text, fnt, 0, i, true);
			printf("   ");
			printText(text, fnt, i, 100, true);
			i++;
		}
	}
}

void parse_transliteration(char *text, CFrameAccessor &frame_acc)
{
	char *s = text;
	for (;;)
	{
		while (*s == ' ' || *s == 9)
			s++;
		
		if (*s == '\0')
			break;
		
		if (nr_col1_transl_text >= 3)
		{	PrintContext(frame_acc);
			printf("Fataal(WN15): Te veel transliteraties - Alleen eerste twee worden bekeken");
			nr_col1_transl_text = 2;
		}
		char *transl = col1_transl_text[nr_col1_transl_text];
			
		int i = 0;
		for (; *s != '\0' && *s != ' ' && *s != 9; s++)
		{	if (     *s == '' 
			      || *s == '')  replace(*s, 'o', transl, i, frame_acc);
			else if (*s == ''
			      || *s == ''
			      || *s == '')  replace(*s, 'e', transl, i, frame_acc);
			else if (*s == '')  replace(*s, 'i', transl, i, frame_acc);
			else if (*s == '')  replace(*s, 'a', transl, i, frame_acc);
			else if (*s == '')  replace(*s, 'u', transl, i, frame_acc);
			else if (*s == '')  replace(*s, '', transl, i, frame_acc);
			else if (   *s == '[' || *s == ']' 
			    || *s == ';' || *s == ','
			    || *s == ':')
			{	PrintContext(frame_acc);
				printf("Waarschuwing(TL07): Karakter '%c' in transliteratie overgeslagen", *s);
			}
			else if (   (byte)*s < ' ' 
					 || (   (byte)*s >= 127 
					 	 && *s != '' && *s != '' && *s != '' && *s != ''))
			{	transl[i] = '\0';
				PrintContext(frame_acc);
				printf("Waarschuwing(TL08): Vreemd karakter '%c' (#%02X) in transliteratie overgeslagen", 
					   *s, (byte)*s);
			}
			else if (i < 99)
				transl[i++] = *s;
		}
		transl[i] = '\0';
		nr_col1_transl_text++;
	}
}


void parse_column1_par(CFrameAccessor &frame_acc, CTextOnFramesAccessor &text_acc,
					   bool &more, char *text, char *fnt,
					   const char *&par_style, char &font_family)
{
	more = text_acc.More();
	if (!more)
	{	text[0] = '\0';
		par_style = "<unknown>";
		font_family = ' ';
		return;
	}

	par_style = text_acc.par_style;

	word nr_T = 0;
	word nr_G = 0;
	word nr_S = 0;
	word nr_H = 0;
	word nr_N = 0;
	bool has_greek = false;

	word i = 0;
	bool started = false;
	bool in_new_frame = false;
	while (text_acc.More() && !text_acc.is_newline && !in_new_frame)
	{	if (i < 1000 && (text_acc.ch != ' ' || started))
		{	text[i] = text_acc.ch;
			fnt[i] = text_acc.font_family;
			i++;
			started = true;
		}
		if ('0' <= text_acc.ch && text_acc.ch <= '9')
			nr_N++;
		else			
			switch(text_acc.font_family)
			{	case 'T': nr_T++; break;
				case 'G': 
				case '1':
				case '2':nr_G++; break;
				case 'H': nr_H++; break;
				default: nr_S++; break;
			}
		if (text_acc.font_family == 'G' && text_acc.ch > 130)
			has_greek = true;
		
		text_acc.Next();
		in_new_frame = text_acc.in_new_frame;
	}
	text[i] = '\0';	
	while (i > 0 && text[i-1] == ' ')
		text[--i] = '\0';
	if (text_acc.is_newline)
		text_acc.SkipLine();

	//printf("T:%d, N:%d in |%s|", nr_T, nr_N, text);
	word max = nr_N;
	font_family = 'N';
	if (nr_H > max)
	{	font_family = 'H';
		max = nr_H;
	}
	if (nr_S > max)
	{	font_family = 'S';
		max = nr_S;
	}
	if (nr_T > max)
	{	font_family = 'T';
		max = nr_T;
	}
	if (nr_G > max || has_greek)
	{	font_family = 'G';
		max = nr_G;
	}
	if (max == 0)
		font_family = ' ';
}

void parse_column1(CFrameAccessor &frame_acc, CTextCritState &textCrit, int chapter_nr)
{
	output_start_col1();
	
	new_verse_recent_greek_words();
	
	char *sub_context = context + strlen(context);
	word word_form_nr = 1;

	CTextOnFramesAccessor text_acc(frame_acc);
	text_acc.Init();
	
	char text[1001];
	char fnt[1001];
	bool more = true;
	const char *par_style;
	char font_family;
	parse_column1_par(frame_acc, text_acc, more, text, fnt, par_style, font_family);
	
	while (more)
	{
		sprintf(sub_context, ", woordgroep %d", word_form_nr++);
		bool read_par = false;
		
		// Sometimes there are empty number parts
		bool skipped_empty_number_part = false;
		if (text[0] == '\0' && !(font_family == 'G' || streq(par_style, "Grieks")))
		{	parse_column1_par(frame_acc, text_acc, more, text, fnt, par_style, font_family);
			if (font_family == 'N' && streq(par_style, "Number"))
			{	if (show_col1_warning)
				{	PrintContext(frame_acc);
					printf("Info: Gecorrigeerd voor leeg Number gedeelte");
				}
			}
			else
			{	skipped_empty_number_part = true;
				if (!more)
					break;
			}
		}
			
		if (font_family == 'G' && streq(par_style, "Grieks"))
		{	if (!skipped_empty_number_part)
			{	PrintContext(frame_acc);
				printf("Fout(C103): Number gedeelte is leeg -- bevat Grieks");
			}
		}
		else if (   font_family == 'N'
		         || (streq(par_style, "Number") && font_family != 'G'))
		{	if (text[0] == '\0')
			{	PrintContext(frame_acc);
				printf("Fout(C104): Number gedeelte is leeg");
			}			
			else if (   !streq(par_style, "Number")
				     || (font_family != 'N' && font_family != 'T'))
			{	if (show_col1_warning)
				{	PrintContext(frame_acc);
					printf("Waarschuwing(C105): Number gedeelte met '%s' '%c' in ", par_style, font_family);
					printText(text, fnt);
				}
			}
			if (text[0] != '\0')
			{	parse_word_nr(text, frame_acc);
			}			
			parse_column1_par(frame_acc, text_acc, more, text, fnt, par_style, font_family);
			read_par = true;
		}
		else
		{	if (text[0] != '\0' || show_col1_warning)
			{	PrintContext(frame_acc);
				printf("%s: Number gedeelte '%s' '%c' overgeslagen in ", 
					   text[0] != '\0' ? "Fout(C106)" : "Waarschuwing(C107)", 
					   par_style, font_family);
				printText(text, fnt);
			}
		}			

		if (!more)
		{	PrintContext(frame_acc);
			printf("Fout(C108): Grieks gedeelte overgeslagen");
			break;
		} 		
		if (   font_family == 'G'
			|| streq(par_style, "Grieks"))
		{	if (text[0] == '\0')
			{	PrintContext(frame_acc);
				printf("Fout(C109): Grieks gedeelte is leeg");
			}
			else if (   !streq(par_style, "Grieks") 
		    	 	 || (font_family != 'G' && font_family != 'S'))
			{	if (show_col1_warning)
				{	PrintContext(frame_acc);
					printf("Waarschuwing(C110): Grieks gedeelte met '%s' '%c' in ", par_style, font_family);
					printText(text, fnt);
				}
			}
			
			if (text[0] != '\0')
				parse_greek_word(text, fnt, textCrit, frame_acc);

			parse_column1_par(frame_acc, text_acc, more, text, fnt, par_style, font_family);
			read_par = true;
		}
		else
		{	if (text[0] != '\0' || show_col1_warning)
			{	PrintContext(frame_acc);
				printf("%s: Grieks gedeelte '%s' '%c' overgeslagen in ", 
					   text[0] != '\0' ? "Fout(C111)" : "Waarschuwing(C112)", 
					   par_style, font_family);
				printText(text, fnt);
			}
		}
		
		if (!more)
		{	PrintContext(frame_acc);
			printf("Fout(C113): Trans gedeelte ontbreekt");
			break;
		} 		
		if (   font_family != 'G' && font_family != 'N'
		    && (streq(par_style, "Trans") || font_family == 'H' || font_family == 'T'))
		{	if (text[0] == '\0')
			{	PrintContext(frame_acc);
				printf("Fout(C114): Trans gedeelte is leeg");
			}
			else if (!streq(par_style, "Trans") || font_family != 'H')
			{	if (show_col1_warning)
				{	PrintContext(frame_acc);
					printf("Waarschuwing(C115): Trans gedeelte met '%s' '%c' in ", par_style, font_family);
					printText(text, fnt);
				}
			}

			if (text[0] != '\0')
				parse_transliteration(text, frame_acc);

			parse_column1_par(frame_acc, text_acc, more, text, fnt, par_style, font_family);
			read_par = true;
		}
		else
		{	if (text[0] != '\0' || show_col1_warning)
			{	PrintContext(frame_acc);
				printf("%s: Trans gedeelte '%s' '%c' overgeslagen in ", 
					   text[0] != '\0' ? "Fout(C116)" : "Waarschuwing(C117)", 
					   par_style, font_family);
				printText(text, fnt);
			}
		}			
		
		if (!more)
		{	// -- empy vertaling part is not an error, if inside alternative!!!	
			//PrintContext(frame_acc);
			//printf("Error: Vertaling part missing");
		}
		else if (font_family == 'N' && streq(par_style, "Number"))
		{	// -- empy vertaling part is not an error, if inside alternative!!!	
		}		
		else if (   font_family != 'G'
		         && (   streq(par_style, "Vertaling")
				     || font_family == 'T'
				     || text[0] == '\0'
				     || (streq(par_style, "Trans") && font_family == 'H')
				     || (streq(par_style, "Number") && font_family == 'H')))
		{	if (text[0] == '\0')
			{	// -- empy vertaling part is not an error, if inside alternative!!!
				//PrintContext(frame_acc);
				//printf("Error: Vertaling part is empty");
			}
			else if ('0' <= text[0] && text[0] <= '9')
			{	PrintContext(frame_acc);
				printf("Fout(C118): Vertaling gedeelte begint met een cijfer ");
				printText(text, fnt);
			}
			else if (!streq(par_style, "Vertaling") || font_family != 'T')
			{	if (show_col1_warning)
				{	PrintContext(frame_acc);
					printf("Waarschuwing(C119): Vertaling gedeelte met '%s' '%c' in ", par_style, font_family);
					printText(text, fnt);
				}
			}
			
			strcpy(col1_translation, text);

			parse_column1_par(frame_acc, text_acc, more, text, fnt, par_style, font_family);
			read_par = true;
		}
		else
		{	if (text[0] != '\0' || show_col1_warning)
			{	PrintContext(frame_acc);
				printf("%s: Vertaling gedeelte '%s' '%c' overgeslagen in ", 
					   text[0] != '\0' ? "Fout(C120)" : "Waarschuwing(C121)",
					   par_style, font_family);
				printText(text, fnt);
			}
		}			
		
		if (read_par)
			check_word_group(frame_acc);
		else
		{	PrintContext(frame_acc);
			printf("Fout(C122): In de war van '%s' '%c' paragraaf in ", par_style, font_family);
			printText(text, fnt);
				   
			while (text_acc.More())
				text_acc.Next();
			break;
		}
		
	}

	textCrit.end(frame_acc);

	*sub_context = '\0';
}					

void skip_tc(CTextOnFramesAccessor &text_acc, bool &more, bool echo_output = false)
{
	if (!more)
		return;
		
	for(;;)
	{
		if (!text_acc.More())
		{	more = false;
			return;
		}
		if (text_acc.is_newline)
		{
			text_acc.SkipLine();
			if (   (   text_acc.ch == '*' 
			        && text_acc.font_family != '2' 
			        && text_acc.font_family != '1')
			    || text_acc.StartsWith("Tekst")
			    || text_acc.StartsWith("1 Tekst")
			    || text_acc.StartsWith("2Tekst")
			    || text_acc.StartsWith("2 Tekst")
			    || text_acc.StartsWith("3Tekst")
			    || text_acc.StartsWith("3 Tekst")
			    || text_acc.StartsWith("4 Tekst")
			    || text_acc.StartsWith("5 Tekst")
			    || text_acc.StartsWith("6 Tekst")
			    || text_acc.StartsWith("7 Tekst")
			    || text_acc.StartsWith("8 Tekst")
				|| text_acc.StartsWith("9 Tekst")
				|| text_acc.StartsWith("2  Tekst")
				|| text_acc.StartsWith("Variant")
				|| text_acc.StartsWith("2 Variant")
				|| text_acc.StartsWith("3 Variant")
				|| text_acc.StartsWith("4 Variant")
				|| text_acc.StartsWith("5 Variant")
				|| text_acc.StartsWith("6 Variant")
				|| text_acc.StartsWith("7 Variant")
				|| text_acc.StartsWith("8 Variant")
				|| text_acc.StartsWith("9 Variant")
				|| text_acc.StartsWith("2 zie")
				|| text_acc.StartsWith("2 Zie")
				|| text_acc.StartsWith("3 zie")
				|| text_acc.StartsWith("5 Zie")
				|| text_acc.StartsWith(" Variant")
				|| text_acc.StartsWith("variant:")
				|| text_acc.StartsWith("Varnt:")
				|| text_acc.StartsWith("Variannt:"))
			{	more = false;
				return;
			}
		}
		else if (text_acc.ch <= ' ')
		{	if (echo_output)
				output_tc_remark_char(text_acc.ch, text_acc.font_family);
			text_acc.Next();
		}
		else if (text_acc.font_family == '2' && (text_acc.ch == 86 || text_acc.ch == 83))
		{	if (echo_output)
				output_tc_remark_char(text_acc.ch, text_acc.font_family);
			text_acc.Next();
		}
		else
			return;
	}
}

struct {
	const char* name;
	char type;
	const char* correct;
	char mode;
	int nr_B, nr_E, nr_O, nr_U;
} manuscripts[] = {
	{ "008", 'O', "08" },
	{ "010", 'O' },
	{ "012", 'O' },
	{ "017", 'B' },
	{ "018", 'B' },
	{ "01", 'E' },
	{ "020", 'B' },
	{ "025", 'B' },
	{ "028", 'O' },
	{ "02", 'E' },
	{ "032", 'O' },
	{ "038", 'O' },
	{ "308", 'O', "038" },
	{ "03", 'E' },
	{ "040", 'B' },
	{ "045", 'B' },
	{ "046", 'B' },
	{ "049", 'B' },
	{ "46", 'B', "046" },
	{ "49", 'B', "049" },
	{ "04", 'E' },
	{ "051", 'B' },
	{ "05", 'O' },
	{ "06", 'O' },
	{ "08", 'O' },
	{ "092", 'E' },
	{ "10", 'O', "010" },
	{ "30", 'E', "030" },
	{ "32", 'O', "032" },
	{ "38", 'O', "038" },
	{ "3", 'E', "03" },
	{ "o4", 'E', "04" },
	{ "p10", 'E' },
	{ "p11", 'E' },
	{ "p13", 'E' },
	{ "p15", 'E' },
	{ "p16", 'E' },
	{ "p18", 'E' },
	{ "p1", 'E' },
	{ "p20", 'E' },
	{ "p21", 'O' },
	{ "p22", 'E' },
	{ "p23", 'E' },
	{ "p24", 'E' },
	{ "p25", 'O' },
	{ "p26", 'E' },
	{ "p27", 'E' },
	{ "p28", 'E' },
	{ "p30", 'E' },
	{ "p33", 'E' },
	{ "p34", 'E' },
	{ "p35", 'E' },
	{ "p36", 'E' },
	{ "p37", 'E' },
	{ "p38", 'B' },
	{ "p39", 'E' },
	{ "p3", 'E' },
	{ "p40", 'E' },
	{ "p41", 'E' },
	{ "p43", ' ' },
	{ "p45", 'E' },
	{ "p46", 'E' },
	{ "p47", 'E' },
	{ "p48", 'E' },
	{ "p49", 'E' },
	{ "p4", 'E' },
	{ "p50", 'E' },
	{ "p51", 'E' },
	{ "p53", 'E' },
	{ "p54", 'E' },
	{ "p55", 'E' },
	{ "p56", 'E' },
	{ "p59", 'E' },
	{ "p5", 'E' },
	{ "p60", 'E' },
	{ "p61", 'E' },
	{ "p63", 'E' },
	{ "p65", 'E' },
	{ "p66", 'E' },
	{ "p67", 'E' },
	{ "p68", 'E' },
	{ "p69", 'E' },
	{ "p6", 'E' },
	{ "p71", 'E' },
	{ "p72", 'E' },
	{ "p74", 'E' },
	{ "p75", 'E' },
	{ "p76", 'E' },
	{ "p77", 'E' },
	{ "p78", 'E' },
	{ "p79", ' ' },
	{ "p81", 'E' },
	{ "p84", 'E' },
	{ "p85", 'E' },
	{ "p88", 'E' },
	{ "p8", 'E' },
	{ "p94", 'E' },
	{ "p98", ' ' },
	{ ".017", 'B', "017" },
	{ ".045", 'B', "045" },
	{ "p(13)", 'E', "p13", '('},
	{ "O1", 'E', "01" },
	{ "-046", 'B', "046" },
};
int nr_manuscripts = sizeof(manuscripts)/sizeof(manuscripts[0]);

void parse_tc_coding(CFrameAccessor &frame_acc, CTextOnFramesAccessor &text_acc,
					 bool error)
{
	bool more = true;
	skip_tc(text_acc, more);
	
	for(;more;)
	{	
		if (text_acc.ParseLiteral("H-F"))
		{	output_tc_edition("H-F");
		}
		else if (text_acc.ParseLiteral("H-"))
		{
			if (text_acc.is_newline)
				text_acc.Next();
			text_acc.ParseLiteral("F");
			output_tc_edition("H-F");
		}
		else if (text_acc.ParseLiteral("H:F"))
		{	PrintContext(frame_acc);
			printf("Correctie(C231): grondtekst H:F wordt gelezen als H-F.");
			output_tc_edition("H-F");
		}
		else if (text_acc.ParseLiteral("HF"))
		{	PrintContext(frame_acc);
			printf("Correctie(C231): grondtekst HF wordt gelezen als H-F.");
			output_tc_edition("H-F");
		}
		else if (text_acc.ParseLiteral("N-F"))
		{	PrintContext(frame_acc);
			printf("Correctie(C231): grondtekst N-F wordt gelezen als H-F.");
			output_tc_edition("H-F");
		}
		else if (text_acc.ParseLiteral("(H-F)"))
			output_tc_edition("H-F", '(');
		else if (text_acc.ParseLiteral("H25,26"))
		{	PrintContext(frame_acc);
			printf("Correctie(C231): grondtekst H25,26 wordt gelezen als N25,26.");
			output_tc_edition("N25");
			output_tc_edition("N26");
		}
		else if (text_acc.ParseLiteral("N25,25"))
		{	PrintContext(frame_acc);
			printf("Correctie(C231): grondtekst N25,25 wordt gelezen als N25,26.");
			output_tc_edition("N25");
			output_tc_edition("N26");
		}
		else if (text_acc.ParseLiteral("N25,26,"))
		{	output_tc_edition("N25");
			output_tc_edition("N26");
		}
		else if (text_acc.ParseLiteral("N25,26"))
		{	output_tc_edition("N25");
			output_tc_edition("N26");
		}
		else if (text_acc.ParseLiteral("NN25,26"))
		{	PrintContext(frame_acc);
			printf("Correctie(C231): grondtekst NN25,26 wordt gelezen als N25,26.");
			output_tc_edition("N25");
			output_tc_edition("N26");
		}
		else if (text_acc.ParseLiteral("N25, 26"))
		{	output_tc_edition("N25");
			output_tc_edition("N26");
		}
		else if (text_acc.ParseLiteral("N26,26"))
		{	PrintContext(frame_acc);
			printf("Correctie(C231): grondtekst N26,26 wordt gelezen als N25,26.");
			output_tc_edition("N25");
			output_tc_edition("N26");
		}
		else if (text_acc.ParseLiteral("N25a"))
		{	PrintContext(frame_acc);
			printf("Correctie(C231): grondtekst N25a wordt gelezen als N25.");
			output_tc_edition("N25");
			output_tc_edition("N26");
		}
		else if (text_acc.ParseLiteral("N25,"))
		{	output_tc_edition("N25");
		}
		else if (text_acc.ParseLiteral("N25"))
		{	output_tc_edition("N25");
		}
		else if (text_acc.ParseLiteral("N26,"))
		{	output_tc_edition("N26");
		}
		else if (text_acc.ParseLiteral("N26"))
		{	output_tc_edition("N26");
		}
		else if (text_acc.ParseLiteral("N27"))
		{	output_tc_edition("N27");
		}
		else if (text_acc.ParseLiteral("[N25]"))
		{	output_tc_edition("N25", '[');
		}
		else if (text_acc.ParseLiteral("[N26]"))
		{	output_tc_edition("N26", '[');
		}
		else if (text_acc.ParseLiteral("[N27]"))
		{	output_tc_edition("N27", '[');
		}
		else if (text_acc.ParseLiteral("[26]"))
		{	PrintContext(frame_acc);
			printf("Correctie(C231): grondtekst [26] wordt gelezen als [N26].");
			output_tc_edition("N26", '[');
		}
		else if (text_acc.ParseLiteral("[N25,26]"))
		{	output_tc_edition("N25", '[');
			output_tc_edition("N26", '[');
		}
		else if (text_acc.ParseLiteral("[N25,25]"))
		{	PrintContext(frame_acc);
			printf("Correctie(C231): grondtekst [N25,25] wordt gelezen als [N25,26].");
			output_tc_edition("N25", '[');
			output_tc_edition("N26", '[');
		}
		else if (text_acc.ParseLiteral("TR (Stephanus 1550)"))
		{	output_tc_edition("TR1550");
		}
		else if (text_acc.ParseLiteral("TR (Elzevier 1624)"))
		{	output_tc_edition("TR1624");
		}
		else if (text_acc.ch == '(')
		{	
			text_acc.Next();
			output_tc_remark_open();
			for (; text_acc.More(); )
				if (text_acc.ch == ')' && text_acc.font_family != 'G')
				{	text_acc.Next();
					if (isalpha(text_acc.ch) || text_acc.ch == ')')
						output_tc_remark_char(')', ' ');
					else
						break;
				}
				else if (text_acc.ch == '[')
				{	for (; text_acc.More() && text_acc.ch != ']'; text_acc.Next())
						output_tc_remark_char(text_acc.ch, text_acc.font_family);
				}
				else if (text_acc.ch == '(')
				{	for (; text_acc.More() && text_acc.ch != ')'; text_acc.Next())
						output_tc_remark_char(text_acc.ch, text_acc.font_family);
				}
				else
				{	output_tc_remark_char(text_acc.ch, text_acc.font_family);
					text_acc.Next();
				}
			output_tc_remark_close();
		}
		else
			break;
		skip_tc(text_acc, more);
	}
	
	if (   more
		&& (   (   (text_acc.ch == '.' || text_acc.ch == '*')
		        && (text_acc.font_family == '2' || text_acc.font_family == '1'))
		    || text_acc.ParseLiteral(". ")))
	{
		text_acc.Next();
		// found bullet or star
		if (text_acc.ch == 'a')
		{	text_acc.Next();
			PrintContext(frame_acc);
			printf("Warning(C230): extra 'a' karakter verwijderd");
		}
		skip_tc(text_acc, more);
	}
	
	if (more)
	{
		char bracket = ' ';
		
		for (;more;)
		{
			if (text_acc.ch == '[')
			{	if (bracket != ' ')
				{	PrintContext(frame_acc);
					printf("Fout(C221): [ binnen %c", bracket);
				}
				text_acc.Next();
				bracket = '[';
			}
			if (text_acc.ParseLiteral("("))
			{	
				text_acc.Next();
				output_tc_remark_open();
				for (; text_acc.More(); )
					if (text_acc.ch == ')')
					{	text_acc.Next();
						break;
					}
					else if (text_acc.ch == '(')
					{	for (; text_acc.More() && text_acc.ch != ')'; text_acc.Next())
							output_tc_remark_char(text_acc.ch, text_acc.font_family);
					}
					else
					{	output_tc_remark_char(text_acc.ch, text_acc.font_family);
						text_acc.Next();
					}
				output_tc_remark_close();
			}
			else if (text_acc.ParseLiteral("SV"))
				output_tc_trans_edition("SV", bracket);
			else if (text_acc.ParseLiteral("NBG"))
				output_tc_trans_edition("NBG", bracket);
			else if (text_acc.ParseLiteral("BG"))
			{	PrintContext(frame_acc);
				printf("Fout(C230): onbekend bijbelboek BG. Veronderstel NBG");
				output_tc_trans_edition("NBG", bracket);
			}
			else if (text_acc.ParseLiteral("WV"))
				output_tc_trans_edition("WV", bracket);
			else if (text_acc.ParseLiteral("WB")) 
			{	PrintContext(frame_acc);
				printf("Fout(C230): onbekend bijbelboek WB. Veronderstel WV");
				output_tc_trans_edition("WV", bracket);
			}
			else if (text_acc.ParseLiteral("GNB"))
				output_tc_trans_edition("GNB", bracket);
			else if (text_acc.ParseLiteral("GNG"))
			{	PrintContext(frame_acc);
				printf("Fout(C230): onbekend bijbelboek GNG. Veronderstel GNB");
				output_tc_trans_edition("GNB", bracket);
			}
			else if (text_acc.ParseLiteral("NGB"))
			{	PrintContext(frame_acc);
				printf("Fout(C230): onbekend bijbelboek NGB. GNB of NBG?");
			}
			else if (text_acc.ParseLiteral("NG"))
			{	PrintContext(frame_acc);
				printf("Fout(C230): onbekend bijbelboek NG. Veronderstel NBG");
				output_tc_trans_edition("NBG", bracket);
			}
			else if (text_acc.ParseLiteral("GNV"))
			{	PrintContext(frame_acc);
				printf("Fout(C230): onbekend bijbelboek GNV. Veronderstel GNB");
				output_tc_trans_edition("GNB", bracket);
			}
			else if (text_acc.ParseLiteral("HV"))
				output_tc_trans_edition("HV", bracket);
			else if (text_acc.ParseLiteral("NEB"))
				output_tc_trans_edition("NEB", bracket);
			else if (text_acc.ParseLiteral("NB"))
			{	PrintContext(frame_acc);
				printf("Fout(C230): onbekend bijbelboek NB. Veronderstel NEB");
				output_tc_trans_edition("NEB", bracket);
			}
			else if (text_acc.ParseLiteral("NIB"))
			{	PrintContext(frame_acc);
				printf("Fout(C230): onbekend bijbelboek NIB. Veronderstel NEB");
				output_tc_trans_edition("NEB", bracket);
			}
			else if (text_acc.ParseLiteral("NAEB"))
			{	PrintContext(frame_acc);
				printf("Fout(C230): onbekend bijbelboek NAEB. Verondestel NASB");
				output_tc_trans_edition("NASB", bracket);
			}
			else if (text_acc.ParseLiteral("NASB"))
				output_tc_trans_edition("NASB", bracket);
			else if (text_acc.ParseLiteral("NAB"))
			{	PrintContext(frame_acc);
				printf("Fout(C230): onbekend bijbelboek NAB. Veronderstel NASB");
				output_tc_trans_edition("NASB", bracket);
			}
			else if (text_acc.ParseLiteral("NIVV"))
			{	PrintContext(frame_acc);
				printf("Fout(C230): onbekend bijbelboek NIVV. Veronderstel NIV");
				output_tc_trans_edition("NIV", bracket);
			}
			else if (text_acc.ParseLiteral("NIV"))
				output_tc_trans_edition("NIV", bracket);
			else if (text_acc.ParseLiteral("NIR"))
			{	PrintContext(frame_acc);
				printf("Fout(C230): onbekend bijbelboek NIR. Veronderstel NIV");
				output_tc_trans_edition("SV", bracket);
			}
			else if (text_acc.ParseLiteral("NNIV"))
			{	PrintContext(frame_acc);
				printf("Fout(C230): onbekend bijbelboek NNIV. Veronderstel NIV");
				output_tc_trans_edition("NIV", bracket);
			}
			else if (text_acc.ParseLiteral("NV"))
				output_tc_trans_edition("NV", bracket);
			else if (text_acc.ParseLiteral("SIV"))
			{	PrintContext(frame_acc);
				printf("Fout(C230): onbekend bijbelboek SIV. Veronderstel NIV");
				output_tc_trans_edition("NIV", bracket);
			}
			else if (text_acc.ParseLiteral("NEV"))
			{	PrintContext(frame_acc);
				printf("Fout(C230): onbekend bijbelboek NEV. Veronderstel NIV");
				output_tc_trans_edition("NIV", bracket);
			}
			else if (text_acc.ParseLiteral("VIB"))
			{	PrintContext(frame_acc);
				printf("Fout(C230): onbekend bijbelboek VIB.");
			}
			else if (text_acc.ParseLiteral("RLU"))
				output_tc_trans_edition("RLU", bracket);
			else if (text_acc.ParseLiteral("REU"))
				output_tc_trans_edition("REU", bracket);
			else if (text_acc.ParseLiteral("RUE"))
			{	PrintContext(frame_acc);
				printf("Fout(C230): onbekend bijbelboek RUE. Veronderstel REU?");
				output_tc_trans_edition("REU", bracket);
			}
			else if (text_acc.ParseLiteral("BJ"))
				output_tc_trans_edition("BJ", bracket);
			else if (text_acc.ParseLiteral("B "))
			{	PrintContext(frame_acc);
				printf("Fout(C230): onbekend bijbelboek B. Veronderstel BJ");
				output_tc_trans_edition("BJ", bracket);
			}
			else if (text_acc.ParseLiteral("5"))
			{	PrintContext(frame_acc);
				printf("Fout(C230): onbekend bijbelboek 5.");
			}
			else
				break;
			if (text_acc.ch == ']')
			{	if (bracket != '[')
				{	PrintContext(frame_acc);
					printf("Fout(C221): ] zonder [");
				}
				text_acc.Next();
				bracket = ' ';
			}
			skip_tc(text_acc, more);
		}
		
		if (bracket != ' ')
		{	PrintContext(frame_acc);
			printf("Fout(C221): sluithaak bij %c ontbreekt", bracket);
		}
				
	}
	if (   more
		&& (text_acc.ch == '.' || text_acc.ch == '*')
		&& (text_acc.font_family == '2' || text_acc.font_family == '1'))
	{
		// found bullet or star
		text_acc.Next();
		if (text_acc.ch == 'a')
		{	text_acc.Next();
			PrintContext(frame_acc);
			printf("Warning(C231): extra 'a' karakter verwijderd");
		}
		skip_tc(text_acc, more);
	}
	
	
			
	if (more)
	{
		int mtype = ' ';
		char bracket = ' ';
				
		for (;more;)
		{
			bool progress = false;
			
			if (text_acc.ch == '(')
			{	
				progress = true;
				output_tc_remark_open();
				for (; text_acc.More(); )
					if (text_acc.ch == ')')
					{	text_acc.Next();
						break;
					}
					else if (text_acc.ch == '(')
					{	for (; text_acc.More() && text_acc.ch != ')'; text_acc.Next())
							output_tc_remark_char(text_acc.ch, text_acc.font_family);
					}
					else
					{	output_tc_remark_char(text_acc.ch, text_acc.font_family);
						text_acc.Next();
					}
				output_tc_remark_close();
				skip_tc(text_acc, more);
			}
			if (!more)
				break;
		
			if (   (text_acc.ch == '.' || text_acc.ch == '*')
				&& (text_acc.font_family == '2' || text_acc.font_family == '1'))
			{
				// Error!!!
				progress = true;
				text_acc.Next();
				skip_tc(text_acc, more);
			}
			if (!more)
				break;

			if (   text_acc.ParseLiteral("Byz::")
				|| text_acc.ParseLiteral("Byz:")
				|| text_acc.ParseLiteral("Byz.:")
				|| text_acc.ParseLiteral("Byz")
				|| text_acc.ParseLiteral("Bijz:")
				|| text_acc.ParseLiteral("aByz:"))
			{	progress = true;
				mtype = 'B';
				skip_tc(text_acc, more);
			}
			else if (   text_acc.ParseLiteral("Eg:")
					 || text_acc.ParseLiteral("Eg.:")
					 || text_acc.ParseLiteral("Eg"))
			{	progress = true;
				mtype = 'E';
				skip_tc(text_acc, more);
			}
			else if (   text_acc.ParseLiteral("Ov:")
					 || text_acc.ParseLiteral("Ov.:")
					 || text_acc.ParseLiteral("Ov"))
			{	progress = true;
				mtype = 'O';
				skip_tc(text_acc, more);
			}
			if (!more)
				break; // give error nothing after 

			{
				skip_tc(text_acc, more);
				char bracket = ' ';
				for (;more;)
				{
					if (text_acc.ch == '(')
					{	text_acc.Next();
						progress = true;
						bracket = '(';
					}
					if (!more)
						break;

					int mnr;
					for (mnr = 0; mnr < nr_manuscripts; mnr++)
						if (text_acc.ParseLiteral(manuscripts[mnr].name))
							break;
					
					if (mnr == nr_manuscripts)
					{
						if (bracket == '(')
						{
							output_tc_remark_open();
							for (; text_acc.More(); )
								if (text_acc.ch == ')')
								{	text_acc.Next();
									break;
								}
								else if (text_acc.ch == '(')
								{	for (; text_acc.More() && text_acc.ch != ')'; text_acc.Next())
										output_tc_remark_char(text_acc.ch, text_acc.font_family);
								}
								else
								{	output_tc_remark_char(text_acc.ch, text_acc.font_family);
									text_acc.Next();
								}
							output_tc_remark_close();
							skip_tc(text_acc, more);
							bracket = ' ';
						}
						else
							break;
					}
					else
					{
						if (manuscripts[mnr].correct != 0)
						{	PrintContext(frame_acc);
							printf("Corectie(C232): '%s' wordt als '%s' gelezen.",
								manuscripts[mnr].name,
								manuscripts[mnr].correct);
						}
						if (mtype == ' ')
						{	PrintContext(frame_acc);
							printf("Waarschuwing(C233): Type aanduiding ontbreekt");
						} 
						else if (mtype != manuscripts[mnr].type)
						{	//PrintContext(frame_acc);
							//printf("Waarschuwing(C234): Type aanduiding van %s voor %s moet misschien %s zijn",
							//	mtype == 'B' ? "Byz" : mtype == 'E' ? "Eg" : "Ov",
							//	manuscripts[mnr].name,
							//	manuscripts[mnr].type == 'B' ? "Byz" : manuscripts[mnr].type == 'E' ? "Eg" : "Ov");
						}
						progress = true;
			
						if (mtype == 'B')
							manuscripts[mnr].nr_B++;
						else if (mtype == 'E')
							manuscripts[mnr].nr_E++;
						else if (mtype == 'O')
							manuscripts[mnr].nr_O++;
						else
							manuscripts[mnr].nr_U++;
	
						bool in_bracket = bracket == '(' || manuscripts[mnr].mode == '(';				
						const char *correction = 0;
						     if (text_acc.ParseLiteral(" e ")) correction = in_bracket ? "(o)" : "o";
						else if (text_acc.ParseLiteral(" f e")) correction = in_bracket ? "(c)o" : "co";
						else if (text_acc.ParseLiteral(" fe")) correction = in_bracket ? "(c)o" : "co";
						else if (text_acc.ParseLiteral(" f")) correction = in_bracket ? "(o)" : "o";
						else if (text_acc.ParseLiteral("e")) correction = in_bracket ? "(c)" : "c";
						else if (text_acc.ParseLiteral("fe")) correction = in_bracket ? "(c)o" : "co";
						else if (text_acc.ParseLiteral("f")) correction = in_bracket ? "(c)" : "c";
						else if (text_acc.ParseLiteral("(e)")) correction = "(o)";
						else if (text_acc.ParseLiteral("(f)e")) correction = "(c)o";
						else if (text_acc.ParseLiteral("(f)")) correction = "(c)";
						output_tc_greek_source(  manuscripts[mnr].correct
											   ? manuscripts[mnr].correct
											   : manuscripts[mnr].name,
											   bracket, correction);
						if (text_acc.ch == ')')
						{	if (bracket != '(')
							{	PrintContext(frame_acc);
								printf("Fout(C222): ) zonder (");
							}
							text_acc.Next();
							bracket = ' ';
						}
						skip_tc(text_acc, more);
					}
				}
				if (!error && bracket != ' ')
				{	PrintContext(frame_acc);
					printf("Fout(C222): sluithaak bij %c ontbreekt", bracket);
				}
			}
			if (!progress)
				break;
		}
		if (!error && bracket != ' ')
		{	PrintContext(frame_acc);
			printf("Fout(C222): sluithaak bij %c ontbreekt", bracket);
		}
	}

	if (more)
	{
		PrintContext(frame_acc);
		printf("Fout(C220): het volgende kan niet gelezen worden |");

		if (more)
		{
			output_tc_remark_open();
			for (;more;)
			{	
				if (text_acc.ch >= ' ')
					output_tc_remark_char(text_acc.ch, text_acc.font_family);
				if (text_acc.font_family != 'T' || text_acc.ch < ' ')
					printf("#%02X%c", text_acc.ch, text_acc.font_family);
				else
					printf("%c", text_acc.ch);
				text_acc.Next();
				skip_tc(text_acc, more, true);
			}
			output_tc_remark_close();
		}

		printf("|");
	}
}

void parse_variant(CTextOnFramesAccessor &text_acc, int nr, CFrameAccessor &frame_acc,
				   CTextCritState &textCrit)
{
	char letter = ' ';
	if (text_acc.ParseChar(' '))
	{	if ('a' <= text_acc.ch && text_acc.ch <= 'd')
		{	letter = text_acc.ch;
			text_acc.Next();
		}
	}
	textCrit.has_insert_note(nr, letter, frame_acc);
	output_tc_variant_open(letter);
	
	while (text_acc.More() && text_acc.ch == ' ')
		text_acc.Next();
	if (text_acc.More() && text_acc.ch == '(')
	{
		text_acc.Next();
		output_tc_remark_open();
		while (text_acc.More() && text_acc.ch != ')')
		{
			text_acc.Next();
			output_tc_remark_char(text_acc.ch, text_acc.font_family);
		}
		output_tc_remark_close();
		if (text_acc.More() && text_acc.ch == ')')
			text_acc.Next();
		while (text_acc.More() && text_acc.ch == ' ')
			text_acc.Next();
	}
	else if (text_acc.ParseLiteral("met"))
	{	
		output_tc_remark_open();
		output_tc_remark_char('m',' ');
		output_tc_remark_char('e',' ');
		output_tc_remark_char('t',' ');
		
		for (; text_acc.More(); text_acc.Next())
		{	if (text_acc.ch == ':')
				break;
			output_tc_remark_char(text_acc.ch, text_acc.font_family);
		}
		output_tc_remark_close();
	}
	
	bool error = false;
	if (!text_acc.ParseChar(':'))
	{	// skip " (weglating)"
	    PrintContext(frame_acc);
		printf("Waarschuwing(C201): Verwacht een ':' na 'Variant' in |");
		error = true;
	}

	parse_tc_coding(frame_acc, text_acc, error);
	output_tc_variant_close();
}


void parse_column2(CFrameAccessor &frame_acc, CTextCritState &textCrit, int chapter_nr)
{
	output_start_col2();

	//if (debug_parse_col2)
	//{	PrintContext(frame_acc);
	//	dump_frame(frame_acc, frame_acc->page, 0, stdout);	
	//}
	//CFrameAccessor &start_frame = frame_acc;
		
	CTextOnFramesAccessor text_acc(frame_acc);
	text_acc.Init();

	for(;;)
	{
		// Skip spaces and rubbish
		text_acc.SkipSpacesAndControl();

		//printf("\n==|%s|", text_acc.par_text);
		
		if (!text_acc.More())
			break;

		cdf_output.open_compact_record("tc");

		word nr = 0;
		if (text_acc.ParseNumber(nr))
			text_acc.SkipSpaces();

		if (!text_acc.More())
		{	if (nr > 0)
			{	// seems not to occur anymore!
				PrintContext(frame_acc);
				printf("Fout(C202): Nummer (%d) zonder 'Variant'", nr);
			}
			cdf_output.write_string("Fout(C202)");
			cdf_output.close_record();
			break;
		}
		
		if (text_acc.is_newline)
		{	if (nr > 0)
			{	// seems not to occur anymore!
				PrintContext(frame_acc);
				printf("Fout(C203): Nummer (%d) wordt gevolgd door regelovergang", nr);
			}
			text_acc.Next();
		}
		else if (text_acc.ParseLiteral("Tekst"))
		{	bool error = false;

			output_tc_nr(nr == 0 ? 1 : nr);
			output_tc_text_open();
			textCrit.has_delete_note(nr, frame_acc);
		
			while (text_acc.ParseChar(' ')) 
				;
			if (!text_acc.ParseChar(':'))
			{	text_acc.SkipSpaces();
				bool open_bracket = text_acc.ParseChar('(');
				if (text_acc.ParseLiteral("met"))
				{	if (!open_bracket)
					{	PrintContext(frame_acc);
						printf("Waarschuwing(C204): 'met'-zin tussen haakjes");
					}
					
					output_tc_remark_open();
					output_tc_remark_char('m',' ');
					output_tc_remark_char('e',' ');
					output_tc_remark_char('t',' ');
					
					const char *end_string = open_bracket ? "):" : ":";
					for (; text_acc.More(); text_acc.Next())
					{	if (text_acc.ParseLiteral(end_string))
							break;
						if (text_acc.is_newline)
							break;
						output_tc_remark_char(text_acc.ch, text_acc.font_family);
					}
					output_tc_remark_close();
				}
				else if (open_bracket)
				{
					output_tc_remark_open();
					while (text_acc.More() && text_acc.ch != ')')
					{
						output_tc_remark_char(text_acc.ch, text_acc.font_family);	
						text_acc.Next();
					}
					output_tc_remark_close();
					if (text_acc.More() && text_acc.ch == ')')
						text_acc.Next();
					if (!text_acc.ParseChar(':'))
					{	PrintContext(frame_acc);
						printf("Waarschuwing(C205): verwacht een ':' na 'Tekst' in |");
					}
				}
				else
				{	PrintContext(frame_acc);
					printf("Waarschuwing(C205): verwacht een ':' na 'Tekst' in |");
				}
			}
	
			parse_tc_coding(frame_acc, text_acc, error);
			output_tc_text_close();
						
			while (   text_acc.ParseLiteral("Variant")
				   || text_acc.ParseLiteral("variant")
				   || text_acc.ParseLiteral("Varnt")
				   || text_acc.ParseLiteral("Variannt:"))
				parse_variant(text_acc, nr, frame_acc, textCrit);
		}
		else if (   text_acc.ParseLiteral("Variant")
				 || text_acc.ParseLiteral("variant"))
		{
			output_tc_nr(nr == 0 ? 1 : nr);
			do
			{	parse_variant(text_acc, nr, frame_acc, textCrit);
			} while (   text_acc.ParseLiteral("Variant")
					 || text_acc.ParseLiteral("variant"));
		}
		else if (text_acc.ParseChar('*'))
		{
			textCrit.has_remark_note(frame_acc);

			output_tc_nr(nr);
			output_tc_remark_open();
			if (text_acc.More() && text_acc.ch == ' ')
				text_acc.Next();
			for (; text_acc.More() && !text_acc.is_newline; text_acc.Next())
				output_tc_remark_char(text_acc.ch, text_acc.font_family);
			output_tc_remark_close();
		}
		else if (text_acc.ParseLiteral("Zie") || text_acc.ParseLiteral("zie"))
		{	
			output_tc_nr(nr == 0 ? 1 : nr);
			output_tc_remark_open();
			output_tc_remark_char('Z',' ');
			output_tc_remark_char('i',' ');
			output_tc_remark_char('e',' ');
			for (; text_acc.More() && !text_acc.is_newline; text_acc.Next())
				output_tc_remark_char(text_acc.ch, text_acc.font_family);
			output_tc_remark_close();
		}
		else			
		{	PrintContext(frame_acc);
			printf("Fout(C206): Verwacht 'Tekst/Variant:' (na nummer %d) in |", nr);
			for (; text_acc.More(); text_acc.Next())
			{	if (text_acc.is_newline)
					break;

				if (text_acc.font_family != 'T' || text_acc.ch < ' ')
					printf("#%02X%c", text_acc.ch, text_acc.font_family);
				else
					printf("%c", text_acc.ch);
			}
			printf("|");
			//dump_frame(start_frame, start_frame->page, 0, stdout);
		}
		cdf_output.close_record();
	}
	
	for (; text_acc.More(); text_acc.Next())
	{
		if (text_acc.is_newline)
		{	printf("\\n");
		}
		else if (text_acc.ch < 128 && text_acc.ch >= ' ')
		{	if (text_acc.font_family != 'T' || text_acc.ch < ' ')
				printf("#%02X%c", text_acc.ch, text_acc.font_family);
			else
				printf("%c", text_acc.ch);
		}
	}
	
	textCrit.check_all_notes(frame_acc);
}

void parse_column3(CFrameAccessor &frame_acc, CTextCritState &textCrit, int chapter_nr)
{
	output_start_col3();

	for (; frame_acc.More() && frame_acc.geom_type() == gt_col3; frame_acc.Next())
	{
		CTextOnFramesAccessor text_acc(frame_acc);
		text_acc.Init();
	
		while (text_acc.More())
		{
			while (text_acc.More() && (text_acc.ch == ' ' || text_acc.ch == (char)103))
				text_acc.Next();
			
			char edition[10];
			int i = 0;
			for (; text_acc.More() && 'A' <= text_acc.ch && text_acc.ch <= 'Z'; text_acc.Next())
				if (i < 9)
					edition[i++] = text_acc.ch;
			edition[i] = '\0';
			
			if (text_acc.More() && text_acc.ch == ' ')
				text_acc.Next();
			
			char text[1000];
			i = 0;
			for (; text_acc.More() && !text_acc.is_newline && text_acc.ch != '|'; text_acc.Next())
				if (i < 999)
					text[i++] = text_acc.ch;
			while (i > 0 && text[i-1] == ' ')
				i--;
			text[i] = '\0';

			if (text_acc.ch == '|' || text_acc.is_newline)
				text_acc.Next();
			
			output_translation(edition, text);	
		}	
	}
}

bool build_ws = false;

CGreekWord *cur_greek_word = NULL;

#include "parse_comm.cpp"

void parse_column4(CFrameAccessor &frame_acc, CTextCritState &textCrit, int chapter_nr)
// This method is called when a frame in the fourth column has been found
{

	output_start_col4();

	// Create a text accesor to parse the text with formatting.
	CTextOnFramesAccessor text_acc(frame_acc);
	for (text_acc.Init(); text_acc.More(); text_acc.SkipLine())
		parse_commentary(frame_acc, text_acc, false, chapter_nr);	
}
	

void parse_verse_column(CFrameAccessor &frame_acc, byte col_nr, lword &this_verse_nr,
                        bool have_verse_nr, CVerseNr verse_nr, CTextCritState &textCrit,
                        void (*parse_column)(CFrameAccessor &frame_acc, CTextCritState &textCrit, int chapter_nr))
{
	byte gt_col = gt_col1 + col_nr - 1;
	byte gt_col_vn = gt_col1_vn + col_nr - 1;

	if (!frame_acc.More())
	{	PrintContext(frame_acc);
	    printf("Fout(FS05): Verse afgebroken na kolom %d", col_nr);
		return;
	}
	
	// parse col verse number
	if (frame_acc.geom_type() != gt_col_vn)
	{	if (have_verse_nr)
		{	if (frame_acc.geom_type() == gt_col)
			{	PrintContext(frame_acc);
	    		printf("Fout(FS06): Kolom %d kader gevonden zonder %d vers nummer.",
					   col_nr, col_nr);
			}
			else if (col_nr == 1 || col_nr == 3)
			{	PrintContext(frame_acc);
	    		printf("Fout(FS07): Kolom %d kader niet gevonden.", col_nr);
			}
			else if (col_nr == 4)
			{	PrintContext(frame_acc);
	    		printf("Waarschuwing(FS08): Geen Kolom 4 kader gevonden.");
			}
		}
	}
	else
	{	// -- found column verse number frame
		if (debug_parse_verse) printf(" col%d_vn", col_nr);
	
		if (!have_verse_nr)
		{	PrintContext(frame_acc);
	    	printf("Fout(FS09): Zou geen Kolom %d versnummer kader moeten hebben.",
				   col_nr);
			//dump_text_fragment(frame_acc->text_fragment, stdout);
		}
	
		if (!frame_acc.TextIsNumber())
		{	PrintContext(frame_acc);
	    	printf("Fout(FS10): Kolom %d bevat geen versnummer.\n", col_nr);
			//dump_text_fragment(frame_acc->text_fragment, stdout);
		}
		else
		{	if (col_nr == 1)
				this_verse_nr = frame_acc.NumberValueText();
			else
			{	lword coln_verse_nr = frame_acc.NumberValueText();
				if (this_verse_nr == 0)
					this_verse_nr = coln_verse_nr;
				else if (this_verse_nr != coln_verse_nr)
				{	PrintContext(frame_acc);
	    			printf("Fout(FS11): Het versnummer (%ld) in kolom %d verschilt van %ld",
						   coln_verse_nr, col_nr, this_verse_nr);
				}
			}
			if (this_verse_nr != 0)
			{	// Update context
				sprintf(context, "in vers %s", book_abrvs[cur_book_nr]);
				if (this_verse_nr == 1 && verse_nr.vr_nr > 1)
				{	verse_nr.ch_nr++;
					verse_nr.vr_nr = 1;
				}
				else
					verse_nr.vr_nr = this_verse_nr;
				char *s = context + strlen(context);
				verse_nr.print(s);
			}
		}
		
		frame_acc.Next();

		if (!frame_acc.More() || frame_acc.geom_type() != gt_col)
		{	PrintContext(frame_acc);
	    	printf("Waarschuwing(FS12): geen kolom %d kader gevonden na versnummer kader.",
				   col_nr);
		}
	}

	// parse all col frames
	if (   frame_acc.More() && frame_acc.geom_type() == gt_col
	    && parse_column != NULL)
		(*parse_column)(frame_acc, textCrit, verse_nr.ch_nr);

	while (frame_acc.More() && frame_acc.geom_type() == gt_col)
		frame_acc.Next();
}

void parse_verse(CFrameAccessor &frame_acc, CVerseNr &verse_nr)
{
	output_start_verse(verse_nr);
	
	if (debug_parse_greek) printf("\n");
	lword this_verse_nr = 0;

	bool have_verse_nr = verse_nr.part == 0;
	
	CTextCritState textCrit;
	
	parse_verse_column(frame_acc, 1, this_verse_nr, have_verse_nr, verse_nr,
					   textCrit, parse_column1);
	parse_verse_column(frame_acc, 2, this_verse_nr, have_verse_nr, verse_nr,
					   textCrit, parse_column2);
	parse_verse_column(frame_acc, 3, this_verse_nr, have_verse_nr, verse_nr,
					   textCrit, parse_column3);
	parse_verse_column(frame_acc, 4, this_verse_nr, have_verse_nr, verse_nr,
					   textCrit, parse_column4);

	if (!have_verse_nr)
		verse_nr.vr_nr++;
	else if (this_verse_nr == 0)
		;
	else if (verse_nr.vr_nr == this_verse_nr)
		verse_nr.vr_nr++;
	else if (this_verse_nr == 1 && verse_nr.ch_nr != 0)
	{	verse_nr.ch_nr++;
		verse_nr.vr_nr = 2;
	}
	else
	{	PrintContext(frame_acc);
	    printf("Fout(FS13): versnummer %ld niet in volgorde", 
			   this_verse_nr);
		if (this_verse_nr > verse_nr.vr_nr)
			verse_nr.vr_nr = this_verse_nr + 1;
		return;
	}
	verse_nr.part = 0;

	// Update context
	sprintf(context, "na vers ");
	char *s = context + strlen(context);
	verse_nr.print(s);
}

void parse_comm(CFrameAccessor &frame_acc)
{
	fprintf(stderr, "\nVerwerk: %s", frame_acc.book_name);

	sprintf(context, "Aan het begin van het boek ");

	while (frame_acc.More())
	{
		// Skip to first frame of the text
		while (frame_acc.More() && !GeomTypeIsText(frame_acc.geom_type()))
		{	output_rest(frame_acc);
			frame_acc.Next();
		}
		output_rest_bar();
		if (!frame_acc.More())
			// -- no text found
			break; 

		if (cur_book_nr == 22)  // Yes, this is a hack
		{	PrintContext(frame_acc);
			printf("Fout(FS14): Niet aan het begin van 2Joh.!!!");
		}
		else
		{	cur_book_nr++;
			PrintContext(frame_acc);
			printf("Mededeling(FS15): Begin van het boek %s", book_abrvs[cur_book_nr]);
			output_start_book(book_abrvs[cur_book_nr]);
		}
		
		CVerseNr cur_verse_nr;
		bool first_peri_title = true;
				
		while (frame_acc.More() && GeomTypeIsText(frame_acc.geom_type()))
		{	CVerseRange verse_range;
		
			if (frame_acc.geom_type() != gt_peri_title)
			{	PrintContext(frame_acc);
	    		printf("Fout(FS16): Begint niet met een perikoop titel");
			}
			else
			{	parse_peri_title(frame_acc, verse_range);
				
				// PrintContext(frame_acc);
				// printf("Info: Pericope ");
				// verse_range.print(stdout);
			}
			
			if (first_peri_title)
			{	if (!verse_range.from.FirstVerse())
				{	PrintContext(frame_acc);
	    			printf("Fout(FS17): Eerste perikoop begint niet bij het eerste vers");
				}
				first_peri_title = false;
			}
			else if (verse_range.from.FirstVerse())
			{	PrintContext(frame_acc);
				cur_book_nr++;
				PrintContext(frame_acc);
				printf("Mededeling(FS18): Begin van het boek %s (afgeleid)", book_abrvs[cur_book_nr]);
				output_start_book(book_abrvs[cur_book_nr]);
	    	}
			else if (!PreceedExact(cur_verse_nr, verse_range.from))
			{	PrintContext(frame_acc);
	    		printf("Fout(FS19): Begin van perikoop past niet bij voorgaand vers");
			}

			if (frame_acc.geom_type() == gt_peri_title)
			{	PrintContext(frame_acc);
	    		printf("Fout(FS20): Geen versaanduiding in Perikoop gevonden");
			}
			else
			{						
				cur_verse_nr = verse_range.from;
				
				// parse all verses under pericope title
				while (   frame_acc.More() 
					   && GeomTypeIsText(frame_acc.geom_type())
					   && frame_acc.geom_type() != gt_peri_title)
					parse_verse(frame_acc, cur_verse_nr);
				
				if (!Preceed(verse_range.to, cur_verse_nr))
				{	PrintContext(frame_acc);
	    			printf("Fout(FS21): Laatste versnummer van perikoop klopt niet met titel");
				}
				else
					cur_verse_nr = verse_range.to;
			}
		}
	}
	frame_acc.ReportError();
}

//CGreekWord *cur_greek_word = NULL; -- moved above 
word nr_greek_word_forms;
CGreekWordForm *greek_word_forms[200];

void parse_word_type(CFrameAccessor &frame_acc, CTextOnFramesAccessor &text_acc, word &last_form_number)
{	
	word form_number = 0;
	bool inTR = true;
	char *s = context + strlen(context);
	
	for (; text_acc.More(); text_acc.Next())
	{	if (text_acc.is_newline || text_acc.ch == '\t')
			break;

		if (text_acc.font_family != 'T' && show_parse_wordstudy_warnings)
		{	PrintContext(frame_acc);
			printf("Fout(WS01): Karacter '%c' met Non-Times font ('%c') overgeslagen", text_acc.ch, text_acc.font_family);
		}
		if (' ' <= text_acc.ch && text_acc.ch <= 128)
			/*printf("%c", text_acc.ch)*/;
		else if (text_acc.ch == 7 || text_acc.ch == 11)
			/*printf("#%02X", text_acc.ch)*/;
		else
		{	PrintContext(frame_acc);
			printf("Fout(WS02): Vreemd karakter '%c' (#%02X) in vormnummer overgeslagen",
				   text_acc.ch, text_acc.ch);
		}
		if (text_acc.ch == '[')
			inTR = false;
		else if ('0' <= text_acc.ch && text_acc.ch <= '9' && text_acc.font_family == 'T')
			form_number = form_number*10 + text_acc.ch - '0';
	}

	if (debug_parse_wordstudy)
		fprintf(stderr, "\n  form number=%d, inTR=%c", form_number, inTR ? 'T' : 'F');
	sprintf(s, ", vormnummer %d", form_number);

	if (!text_acc.ParseChar('\t'))
	{	PrintContext(frame_acc);
		printf("Fout(WS03): Verwacht een tab na vormnummer");
	}

	char greek[200];
	int i = 0;	
	int ps = 0;
	bool had_ws71 = false;
	for (; text_acc.More(); text_acc.Next())
	{	if (text_acc.is_newline || text_acc.ch == '\t')
			break;

		if (text_acc.ch == '/' || text_acc.ch == ' ')
		{	if (ps == 1)
				ps = 2;
		}
		else if (text_acc.font_family != 'G')
		{	PrintContext(frame_acc);
			greek[i] = '\0';
			printf("Fout(WS04): Non-Greek font ('%c') na Grieks '%s'", 
				   text_acc.font_family, greek_to_text(greek));
		}
		else if (text_acc.ch == '-')
		{	PrintContext(frame_acc);
			greek[i] = '\0';
			printf("Fout(WS70): '-' karakter overgeslagen");
		}
		else if (	text_acc.ch < 0x40 && text_acc.ch != 0x20
				 && text_acc.ch != 0x27)
		{	had_ws71 = true;
			PrintContext(frame_acc);
			greek[i] = '\0';
			printf("Fout(WS71): Vreemd karakter '%c' (#%02X) overgeslagen na Grieks '%s'", 
				   text_acc.ch, text_acc.ch, greek_to_text(greek));
		}
		else
		{	if (ps == 2)
			{	if (!had_ws71)
				{	if (i < 199)
						greek[i++] = '/';
				}
				had_ws71 = false;
			}
			if (i < 199)
				greek[i++] = text_acc.ch;
			ps = 1;
			
		}
	}
	greek[i] = '\0';
	
	if (debug_parse_wordstudy)
		fprintf(stderr, " greek=|%s|", greek);

	if (!text_acc.ParseChar('\t'))
	{	PrintContext(frame_acc);
		printf("Fout(WS05): Verwacht een tab na Grieks '%s'", 
			   greek_to_text(greek)); 
	}

	char transl[100];
	i = 0;	
	
	for (; text_acc.More(); text_acc.Next())
	{	if (text_acc.is_newline || text_acc.ch == '\t')
			break;

		if (text_acc.font_family != 'T')
		{	PrintContext(frame_acc);
			transl[i] = '\0';
			printf("Fout(WS06): non-Times font ('%c') in transliteratie '%s'", 
				   text_acc.font_family, transl_to_text(transl));
		}
		bool include_char = true;
		if (isnormal(text_acc.ch))
			/*printf("%c", text_acc.ch)*/;
		else
		{	PrintContext(frame_acc);
			transl[i] = '\0';
			printf("Fout(WS07): vreemd karakter '%c' (#%02X) in transliteratie '%s' overgeslagen", 
				   text_acc.ch, text_acc.ch, transl_to_text(transl));
			include_char = false;
		}
		if (include_char && i < 99)
			transl[i++] = text_acc.ch;
	}
	transl[i] = 0;
	
	if (debug_parse_wordstudy)
		fprintf(stderr, " transl=|%s|", transl);
	
	check_transl_with_greek("WS66", "WS67", "WS98", NULL, greek, transl, frame_acc);	
	
	if (!text_acc.ParseChar('\t'))
	{	PrintContext(frame_acc);
		printf("Fout(WS08): verwacht een tab na transliteratie");
	}

	char vert[100];
	i = 0;	

	for (; text_acc.More(); text_acc.Next())
	{	if (text_acc.is_newline)
			break;

		if (text_acc.font_family != 'T')
		{	PrintContext(frame_acc);
			vert[i] = '\0';
			printf("Fout(WS09): non-Times font ('%c') in vertaling '%s'", text_acc.font_family, vert);
		}
		if (isnormal(text_acc.ch))
			/*printf("%c", text_acc.ch)*/;
		else
		{	PrintContext(frame_acc);
			vert[i] = '\0';
			printf("Fout(WS10): Vreemd karakter '%c' (#%02X) in vertaling '%s' overgeslagen", 
				   text_acc.ch, text_acc.ch, vert);
		}
		if (i < 99 && text_acc.ch != '\t')
			vert[i++] = text_acc.ch;
	}
	vert[i] = '\0';

	if (debug_parse_wordstudy)
		fprintf(stderr, " vertaling=|%s|", vert);

	// Parsed whole line; process information	
	if (form_number == 0 && greek[0] == '\0' && transl[0] == '\0')
	{
		if (nr_greek_word_forms == 0)
		{	PrintContext(frame_acc);
			printf("Fataal(WS11): Geen eerdere woordvorm om tekst aan toe te voegen");
		}
		else
		{	if (build_ws)
				greek_word_forms[nr_greek_word_forms-1]->append_vert(vert);
	  	}
	}
	else
	{	if (form_number == 0)
		{	PrintContext(frame_acc);
			printf("Fout(WS12): vormnummer ontbreekt");
			form_number = 1;
		}
		if (form_number <= last_form_number)
		{	PrintContext(frame_acc);
			printf("Fout(WS13): vormnumner %d gelijk of kleiner dan vorig vormnummer (%d)", form_number, last_form_number);
		}
		last_form_number = form_number;

		CGreekWordForm *as_word_form = NULL;
		if (strncmp(vert, "zie", 3) == 0)
		{	char *s = vert + 3;
			while (*s == ' ') s++;
			int as_form_number = 0;
			for (; '0' <= *s && *s <= '9'; s++)
				as_form_number = as_form_number*10 + *s - '0';
			if (as_form_number > 0)
			{	if (*s != '\0')
				{	PrintContext(frame_acc);
					printf("Fout(WS14): karakter '%c' na 'zie <nr>' overgeslagen", *s);
				}
				
				for (int wi = 0; wi < nr_greek_word_forms; wi++)
					if (greek_word_forms[wi]->form_number == as_form_number)
					{	as_word_form = greek_word_forms[wi];
						break;
					}
				
				if (as_word_form == NULL)
				{	PrintContext(frame_acc);
					printf("Fout(WS15): Woord met vormnummer %d niet gevonden", as_form_number);
				}
				else
				{	
					bool the_same = true;
					if (strcmp(as_word_form->greek, greek) != 0)
					{	PrintContext(frame_acc);
						printf("Fout(WS16): Grieks '%s' van vormnummer %d verschilt van "
						       "'%s' van vormnummer %d",
							   greek_to_text(greek), form_number,
							   greek_to_text(as_word_form->greek.A()), as_word_form->form_number);
						the_same = false;
					}
					if (strcmp(as_word_form->transl, transl) != 0)
					{	PrintContext(frame_acc);
						printf("Fout(WS17): transliteratie '%s' van vormnummer %d verschilt van '%s' van vormnummer %d",
							   transl, form_number, as_word_form->transl.A(), as_word_form->form_number);
						the_same = false;
					}
					if (the_same && as_word_form->inTR != inTR)
					{	PrintContext(frame_acc);
						printf("Fout(WS18): in TR '%s' van vormnummer %d verschilt van '%s' van vormnummer %d",
							   inTR ? "wel" : "niet", form_number, as_word_form->inTR ? "wel" : "niet", as_word_form->form_number);
						the_same = false;
					}
					if (!the_same)
					{	/* Only copy the translation (vert) */
						strcpy(vert, as_word_form->vert);
						as_word_form = NULL;
					}
				}
			}
		}
		if (build_ws)
		{
			if (as_word_form)
				greek_word_forms[nr_greek_word_forms++] = new CGreekWordForm(form_number, as_word_form);
			else
				greek_word_forms[nr_greek_word_forms++] = new CGreekWordForm(form_number, greek, transl, vert, inTR);
		}
		else
		{
			greek_word_forms[(lword)nr_greek_word_forms] = cur_greek_word->word_forms[(lword)nr_greek_word_forms];
			nr_greek_word_forms++;
		}
	}
	
	*s = '\0';
}

int book_nr;
int chap_nr;
int verse_nr;

char verse_ref_text[1000];
char *p_verse_ref_text;
char verse_ref_tc_code;

word nr_greek_verse_refs;
CGreekVerseRef *greek_verse_refs[10000];
CGreekVerseRef *cur_greek_verse_ref = NULL;

void init_verse_refs()
{
	if (debug_parse_wordstudy) fprintf(stderr, "\ninit_verse_refs");

	cur_greek_verse_ref = 0;
	p_verse_ref_text = verse_ref_text;
	verse_ref_tc_code = '\0';
}
void produce_ws_text_ref_output()
{
	if (debug_parse_wordstudy) fprintf(stderr, "\nproduce_ws_text_ref_output");

	if (cur_greek_verse_ref == 0)
		return;
		
	output_ws_verse_ref_text(verse_ref_text);

	for (int j = 0; j < 3; j++)
	{	int formnr = cur_greek_verse_ref->form_numbers[j].nr;
		if (formnr != 0)
		{	
#ifdef REMOVE_DOUBLE_WORD_FORMS
			CGreekWordForm *wordform = cur_greek_word->getWordFormByNumber(formnr);
			if (wordform != 0 && wordform->as != 0)
				formnr = wordform->as->form_number;
#endif
			output_ws_verse_ref_form_number(formnr);
		}
	}
	output_ws_word_last_part(verse_ref_tc_code);
	init_verse_refs();
}

void parse_verse_ref(CFrameAccessor &frame_acc, CTextOnFramesAccessor &text_acc, bool &missing_newline)
{
	if (debug_parse_wordstudy) fprintf(stderr, "\nparse_verse_ref 1");

	if (debug_parse_ws_ref_dump || debug_parse_ws_dump)
		printf("\n ");
	if (debug_parse_ws_dump)
		printf(" book=|");
	char buffer[1000];
	int i = 0;
	for (; text_acc.More(); text_acc.Next())
	{	if (text_acc.is_newline || text_acc.ch == '\t')
			break;

		if (text_acc.font_family != 'T')
		{	PrintContext(frame_acc);
			printf("Fout(WS19): non-Times font ('%c') in boekafkorting", text_acc.font_family);
		}
		if (' ' <= text_acc.ch && text_acc.ch <= 128)
		{	if (debug_parse_ws_dump)
				printf("%c", text_acc.ch);
		}
		else
		{	PrintContext(frame_acc);
			printf("Fout(WS20): Vreem karakter '%c' (#%02X) in boekafkorting overgeslagen",
				   text_acc.ch, text_acc.ch);
		}
		if (' ' < text_acc.ch && text_acc.ch <= 128 && i < 9)
			buffer[i++] = text_acc.ch;
	}
	buffer[i] = '\0';
	if (!text_acc.ParseChar('\t'))
	{	PrintContext(frame_acc);
		printf("Fout(WS21): Verwacht een tab na boekafkorting");
	}

	if (debug_parse_wordstudy) fprintf(stderr, "\nparse_verse_ref 2");

	// Process book abbreviation
	if (i > 0)
	{	book_nr = find_book_abrv(buffer);
		if (book_nr == -1)
		{	PrintContext(frame_acc);
			printf("Fout(WS22): Onbekende boekafkorting '%s'", buffer);
		}
	}

	if (debug_parse_ws_ref_dump)
		printf("| verse=|");
	i = 0;
	for (; text_acc.More(); text_acc.Next())
	{	if (text_acc.is_newline || text_acc.ch == '\t')
			break;

		if (text_acc.font_family != 'T')
		{	PrintContext(frame_acc);
			printf("Fout(WS23): non-Times font ('%c') in versnummer", text_acc.font_family);
		}
		if (' ' <= text_acc.ch && text_acc.ch <= 128)
		{	if (debug_parse_ws_dump)
				printf("%c", text_acc.ch);
		}
		else if (text_acc.ch == 16)
		{	if (debug_parse_ws_dump)
				printf("_");
		}
		else
		{	PrintContext(frame_acc);
			printf("Fout(WS24): Vreemd karakter '%c' (#%02X) in versnummer overgeslagen",
			       text_acc.ch, text_acc.ch);
		}
		if (text_acc.ch != ' ' && text_acc.ch != 16 && i < 999)
			buffer[i++] = text_acc.ch;
	}
	buffer[i] = '\0';
	if (!text_acc.ParseChar('\t'))
	{	PrintContext(frame_acc);
		printf("Fout(WS25): verwacht een tab na versnummer");
	}
	
	if (debug_parse_wordstudy) fprintf(stderr, "\nparse_verse_ref 3");

	// Process verse number
	if (i > 0)
	{	i = 0;
		chap_nr = 0;
		verse_nr = 0;
		for (; '0' <= buffer[i] && buffer[i] <= '9'; i++)
			chap_nr = chap_nr * 10 + buffer[i] - '0';
		if (buffer[i] != ':')
		{	verse_nr = chap_nr;
			chap_nr = 0;
		}
		else
		{	i++;
			for (; '0' <= buffer[i] && buffer[i] <= '9'; i++)
				verse_nr = verse_nr * 10 + buffer[i] - '0';
		}
		if (buffer[i] != '\0')
		{	PrintContext(frame_acc);
			printf("Fout(WS26): Vreemd karakter in versnummer '%s'", buffer + i);
		}
	}

	if (debug_parse_ws_ref_dump && book_nr >= 0)
	{
		//fprintf(f2, "%s %d:%d\n", book_abrvs[book_nr], chap_nr, verse_nr);
		printf(" %s %d:%d ", book_abrvs[book_nr], chap_nr, verse_nr);
	}

	char person_letter = '\0';	
	if (text_acc.SingleCharTab())
	{
		if (debug_parse_ws_dump)
			printf("| letter=|%c", text_acc.ch);
		person_letter = text_acc.ch;
		text_acc.Next();
		text_acc.Next();
	}
	
	*p_verse_ref_text = '\0';
	char *text_start = p_verse_ref_text;
	
	if (debug_parse_wordstudy) fprintf(stderr, "\nparse_verse_ref 4");

	if (debug_parse_ws_ref_dump)	
		printf("| text=|");
	i = 0;
	bool in_bold = false;
	for (; text_acc.More(); text_acc.Next())
	{	if (text_acc.is_newline || (text_acc.ch == '\t' && i != 0))
			break;

		if (text_acc.font_family != 'T')
		{	PrintContext(frame_acc);
			printf("Fout(WS27): non-Times font ('%c') in tekst", text_acc.font_family);
		}
		if (isnormal(text_acc.ch))
			/*printf("%c", text_acc.ch)*/;
		else
		{	PrintContext(frame_acc);
			printf("Fout(WS28): Vreemd karakter '%c' (#%02X) in tekst",
				   text_acc.ch, text_acc.ch);
		}
		if (text_acc.ch >= ' ')// && !(i == 0 && text_acc.ch == ' '))
		{	
			if (p_verse_ref_text == text_start && text_acc.ch != ' ')
			{
				if (cur_greek_verse_ref != 0)
				{
					if (build_ws)
						init_verse_refs();
					else
						produce_ws_text_ref_output();
				}
				text_start = p_verse_ref_text;
			}
			
			if ((text_acc.mode & 1) != 0 && !in_bold)
			{	*p_verse_ref_text++ = '{';
				in_bold = true;
			}
			else if ((text_acc.mode & 1) == 0 && in_bold)
			{	*p_verse_ref_text++ = '}';
				in_bold = false;
			}
			i++;
			*p_verse_ref_text++ = text_acc.ch;
		}
	}
	if (in_bold)
		*p_verse_ref_text++ = '}';
	*p_verse_ref_text = '\0';
	if (!text_acc.ParseChar('\t'))
	{	PrintContext(frame_acc);
		printf("Fout(WS29): verwacht een tab na tekst");
	}
	
	if (debug_parse_wordstudy) fprintf(stderr, "\nparse_verse_ref 5");
	
	// Process text
	if (debug_parse_ws_ref_dump)
	{	int j = 0;
		for (j = 0; buffer[j] != '\0'; j++)
			printf("%c", buffer[j]);
	}
	
	if (debug_parse_wordstudy) fprintf(stderr, "\nparse_verse_ref 6");
	
	
	//bool lit_vert = strncmp(buffer, "letterlijk", 10) == 0; 
	if (*text_start != ' ')
	{	
		if (build_ws)
		{
			cur_greek_verse_ref = new CGreekVerseRef(book_nr, chap_nr, verse_nr);
			greek_verse_refs[nr_greek_verse_refs++] = cur_greek_verse_ref;
		}
		else
		{
			produce_ws_text_ref_output();
			
			cur_greek_verse_ref = greek_verse_refs[nr_greek_verse_refs] = cur_greek_word->verse_refs[(lword)nr_greek_verse_refs];
			nr_greek_verse_refs++;
			
			output_ws_word_verse_ref(book_abrvs[book_nr], chap_nr, verse_nr, person_letter);
		} 
	}
	

	if (text_acc.More() && !text_acc.is_newline && text_acc.font_family == '1')
	{	PrintContext(frame_acc);
		printf("Fout(WS30): tab voor leeg vormnummer ontbreekt");
	}
	else 
	{	if (debug_parse_ws_ref_dump)
			printf("| form number=|");

		if (text_acc.is_newline)
			/* no form number */;
		else if (text_acc.ch == '\t')
			text_acc.Next();
		else
		{	char form_numberbuf[100];
			int i = 0;
			for (; text_acc.More(); text_acc.Next())
			{	
				if (text_acc.is_newline || text_acc.ch == '\t')
					break;
				if (text_acc.font_family != 'T')
				{	PrintContext(frame_acc);
					printf("Fout(WS31): non-Times font ('%c') in vormnummer", text_acc.font_family);
				}
				if (   text_acc.ch == '['
				    || text_acc.ch == '.'
				    || text_acc.ch == ']'
				    || text_acc.ch == '/'
				    || ('0' <= text_acc.ch && text_acc.ch <= '9'))
				{	if (i < 99)
						form_numberbuf[i++] = text_acc.ch;
				}
				else
				{	PrintContext(frame_acc);
					form_numberbuf[i] = '\0';
					printf("Fout(WS32): Vreemd karakter (#%02X) in vormnummer na '%s'", text_acc.ch, form_numberbuf);
				}
			}
			form_numberbuf[i] = '\0';
			char *s = form_numberbuf;
			while (*s != '\0')
			{
				bool inTR = true;
				bool period = false;
				if (*s == '.')
				{	period = true;
					s++;
				}
				if (*s == '[')
				{	inTR = false;
					if (s[1] == '.')
					{	period = true;
						PrintContext(frame_acc);
						printf("Fout(WS33): Vormnummer bevat '[.' combinatie bij '%s'", s);
						s++;
					}
					s++;
					
				}
				if (!period)
				{	PrintContext(frame_acc);
					printf("Fout(WS34): verwacht een punt in vormnummer bij '%s'", s);
				}
				int form_number = 0;
				while ('0' <= *s && *s <= '9')
				{	form_number = form_number*10 + *s - '0';
					s++;
				}
				if ((*s == ']') == inTR)
				{	PrintContext(frame_acc);
					printf("Fout(WS35): ongebalanceerde [] in vormnummer '%s'", form_numberbuf);
				}
				if (*s == ']')
					s++;

				CGreekWordForm *greek_word_form = cur_greek_word->getWordFormByNumber(form_number);
				if (greek_word_form == NULL)
				{	PrintContext(frame_acc);
					printf("Fout(WS36): geen woordvorm voor vormnummber %d in '%s'", form_number, form_numberbuf);
				}
				else
				{	if (build_ws)
						greek_word_form->cnt_used_in_ref++;
					if (greek_word_form->inTR != inTR)
					{	PrintContext(frame_acc);
						printf("Fout(WS37): inTR status (%d) van vormnummer %d is verkeerd", inTR, form_number);
					}
					char error_code;
					if (build_ws)
						cur_greek_verse_ref->addFormNumber(form_number, error_code);
					if (error_code == 'R')
					{	PrintContext(frame_acc);
						printf("Fout(WS38): vormnummer %d wordt herhaald", form_number);
					}
					else if (error_code == 'F')
					{	PrintContext(frame_acc);
						printf("Fout(WS39): te veel vormnummers -- Overgeslagen");
					}
				}
				if (*s == '.' && s[1] == '/')
				{	PrintContext(frame_acc);
					printf("Fout(WS40): vond './' inplaats van '/.' in vormnummer bij '%s'", s);
					*s = '/';
					s[1] = '.';
				}	
				else if (*s != '\0' && *s != '/')
				{	PrintContext(frame_acc);
					printf("Fout(WS41): ontbrekende '/' in vormnummer '%s'", form_numberbuf);
				}
				if (*s == '/')
					s++;
			}
			if (!text_acc.ParseChar('\t'))
			{	PrintContext(frame_acc);
				printf("Fout(WS42): verwacht een tab na vormnummer");
			}
		}
	}

	char text_crit[10];
	i = 0;
	if (debug_parse_ws_ref_dump)
		printf("|");
	for (; text_acc.More(); text_acc.Next())
	{	if (text_acc.is_newline || text_acc.ch == '\t')
			break;
		if (debug_parse_ws_ref_dump)
			printf("%02X", text_acc.ch);
		if (text_acc.ch == ' ' || text_acc.ch == 0x0B)
			/* skip space */;
		else if (   text_acc.font_family == '1'
		         && (   text_acc.ch == 0x27   // v driehoek
				 	 || text_acc.ch == 0x60   // v driehoek
		         	 || text_acc.ch == 'o'))  // o
		{	if (i < 8)
				text_crit[i++] = text_acc.ch;
		}
		else if (text_acc.font_family == 'T' && text_acc.ch == '?')
		{	/* three ??? -> o */
		    if (i == 0)
				text_crit[i] = 'o';
		}
		else if (   text_acc.ch == 127
				 || (text_acc.font_family == '1' && text_acc.ch == 0xD4 ))
			/* ignore */;
		else if (   text_acc.font_family == 'T' 
				 && ('A' <= text_acc.ch && text_acc.ch <= 'Z'))
		{	PrintContext(frame_acc);
			printf("Fout(WS43): ontbreekt hier een regelovergang???");
			missing_newline = true;
			break;
		} 
		else
		{	PrintContext(frame_acc);
			printf("Fout(WS44): vreemd karakter %c (#%02X) met font '%c' in TC", 
				   text_acc.ch, text_acc.ch, text_acc.font_family );
		}
	}
	text_crit[i] = '\0';
	
	char tc_code = ' ';
	if (i > 0)
	{	char codes[9][4] = {
		  {  'o', '\0',  '\0', 'A' },
		  { 0x27, '\0',  '\0', 'B' },
		  { 0x60, '\0',  '\0', 'B' },
		  {  'o', 0x27,  '\0', 'C' },
		  {  'o', 0x60,  '\0', 'C' },
		  {  'o',  'o',  '\0', 'D' },
		  { 0x27, 0x27,  '\0', 'E' },
		  { 0x60, 0x60,  '\0', 'E' }};
		int c;
		for (c = 0; c < 9; c++)
			if (strcmp(text_crit, codes[c]) == 0)
			{	tc_code = codes[c][3];
				break;
			}
		if (tc_code == ' ')
		{	PrintContext(frame_acc);
			printf("Fout(WS45): vreemde TC code");
			for (char *s = text_crit; *s; s++)
				printf(" #%02X", *s);
			printf(".");
		}
	}
	if (debug_parse_ws_ref_dump)
		printf(" TC='%c'", tc_code);
	if (build_ws)
		cur_greek_verse_ref->tc_code = tc_code;
	if (verse_ref_tc_code == '\0')
		verse_ref_tc_code = tc_code;
	text_acc.ParseChar('\t');
}


word cur_number = 0;
word cur_romnumber = 0;

bool startwith(const char *s, const char *r)
{
	for (; *r != '\0' && *s == *r; s++, r++)
		;
	return *r == '\0';
}

void output_grammar(char c, bool has_ev, bool has_mv, bool has_m, bool has_v, bool has_o, bool has_aor, bool has_pass, bool has_med, char *vert)
{
	char emv = has_ev ? 'e' : has_mv ? 'm' : ' ';
	if (has_m || has_v || has_o)
	{	if (has_m) output_ws_word_form_grammar(c, emv, 'm', has_aor, has_pass, has_med, vert);
		if (has_v) output_ws_word_form_grammar(c, emv, 'v', has_aor, has_pass, has_med, vert);
		if (has_o) output_ws_word_form_grammar(c, emv, 'o', has_aor, has_pass, has_med, vert);
	}
	else
		output_ws_word_form_grammar(c, emv, ' ', has_aor, has_pass, has_med, vert);
}

void analyze_vert(const char *vert)
{
	const char *s = vert;

	while (*s != '\0')
	{	
		for (; *s == ' '; s++);
		char buf[2000], *b = buf; 
		for (; *s != '\0' && *s != '^'; s++)
			if (   *s == '('
				&& (   ('1' <= s[1] && s[1] <= '5')
					|| startwith(s+1, "aor")
					|| startwith(s+1, "med")
					|| startwith(s+1, "pass")
					|| startwith(s+1, "onverb")))
				break;
			else
				*b++ = *s;
		*b = '\0';
		for (; buf < b && *(b-1) == ' '; *(--b) = '\0');

		if (startwith(s, "(onverb.)"))
		{	s += 9;
			output_grammar(' ', false, false, false, false, false, false, false, false, buf);
		}			
		else if (startwith(s, "(onverb)"))
		{	s += 8;
			output_grammar(' ', false, false, false, false, false, false, false, false, buf);
		}			
		else if (*s == '(')
		{
			s++;
			for (; *s != '\0' && *s != ')'; )
			{
				bool has_1 = false;
				bool has_2 = false;
				bool has_3 = false;
				bool has_4 = false;
				bool has_5 = false;
				bool has_ev = false;
				bool has_mv = false;
				bool has_m = false;
				bool has_v = false;
				bool has_o = false;
				bool has_aor = false;
				bool has_med = false;
				bool has_pass = false;
			
				for (;'1' <= *s && *s <= '5';)
				{	switch(*s)
					{	case '1' : has_1 = true; break;
						case '2' : has_2 = true; break;
						case '3' : has_3 = true; break;
						case '4' : has_4 = true; break;
						case '5' : has_5 = true; break;
					}
					s++;
					if (*s == '/')
						s++;
					else
						break;
				}
				for (; *s == ' '; s++);
				
				if (startwith(s, "ev"))
				{	has_ev = true;
					s += 2;
					if (*s == '.')
						s++;
				}
				else if (startwith(s, "e "))
				{	has_ev = true;
					s++;
				}
				if (*s == '/')
					s++;
				if (startwith(s, "mv"))
				{	has_mv = true;
					s += 2;
				}
				for (; *s == ' '; s++);
				if (s[0] == 'm')
				{	if (s[1] == '/')
					{	has_m = true;
						s += 2;
					}
					else if (s[1] == ')' || s[1] == '\0' || s[1] == ' ' || s[1] == ';')
					{	has_m = true;
						s++;
					}
				}
				if (s[0] == 'v')
				{	if (s[1] == '/')
					{	has_v = true;
						s += 2;
					}
					else if (s[1] == ')' || s[1] == '\0' || s[1] == ' ' || s[1] == ';')
					{	has_v = true;
						s++;
					}
				}
				if (s[0] == 'o')
				{	if (s[1] == ')' || s[1] == '\0' || s[1] == ' ' || s[1] == ';')
					{	has_o = true;
						s++;
					}
				}
				if (startwith(s, "m, "))
				{	s += 3;
					has_m = true;
				}
				else if (startwith(s, " m)"))
				{	s += 2;
					has_m = true;
				}
				else if (startwith(s, "/m"))
				{	s += 2;
					has_m = true;
				}
				if (startwith(s, "vrl "))
				{	s += 4;
					has_v = true;
				}
				if (startwith(s, "onz "))
				{	s += 4;
					has_o = true;
				}
				for (; *s == ' '; s++);
				if (startwith(s, "ev"))
				{	has_ev = true;
					s += 2;
				}
				else if (startwith(s, "mv"))
				{	has_mv = true;
					s += 2;
				}
				if (*s == ';')
					s++;
				for (; *s == ' '; s++);
				
				if (*s == ';')
					s++;
				if (startwith(s, "aor"))
				{	has_aor = true;
					for (s += 3; *s == ' '; s++);
				}
				if (startwith(s, "med"))
				{	has_med = true;
					for (s += 3; *s == ' '; s++);
				}
				if (startwith(s, "pass") || startwith(s, "pas "))
				{	has_pass = true;
					for (s += 4; *s == ' '; s++);
				}

				if (has_1 || has_2 || has_3 || has_4 || has_5)
				{
					if (has_1) output_grammar('1', has_ev, has_mv, has_m, has_v, has_o, has_aor, has_pass, has_med, buf);
					if (has_2) output_grammar('2', has_ev, has_mv, has_m, has_v, has_o, has_aor, has_pass, has_med, buf);
					if (has_3) output_grammar('3', has_ev, has_mv, has_m, has_v, has_o, has_aor, has_pass, has_med, buf);
					if (has_4) output_grammar('4', has_ev, has_mv, has_m, has_v, has_o, has_aor, has_pass, has_med, buf);
					if (has_5) output_grammar('5', has_ev, has_mv, has_m, has_v, has_o, has_aor, has_pass, has_med, buf);
				}
				else
					output_grammar(' ', has_ev, has_mv, has_m, has_v, has_o, has_aor, has_pass, has_med, buf);
				
				// more
				if (*s == '-' || *s == '^')
					for (s++; *s == ' '; s++);
				else if (startwith(s, "2 mv)"))
					;
				else
					break;
			}
			if (*s == ')')
				++s;
		}
		else
			output_grammar(' ', false, false, false, false, false, false, false, false, buf);
		for (; *s == ' '; s++);
		if (*s == '-' || *s == '^')
			for (s++; *s == ' '; s++);
		else
			break;
	}
	if (*s != '\0')
		printf("ERROR: stopped at |%s| in |%s|\n", s, vert);
}

	

void parse_one_word_study(CFrameAccessor &frame_acc, CTextOnFramesAccessor &text_acc)
{
	char number[100];
	int i = 0;
	
	// fprintf(stderr, "\nparse_one_word_study\n");
	
	for (; text_acc.More(); text_acc.Next())
	{	if (text_acc.is_newline || text_acc.ch == '\t')
			break;

		if (text_acc.font_family != 'T')
		{	PrintContext(frame_acc);
			printf("Fout(WS46): non-Times font ('%c') in nummer", text_acc.font_family);
		}
		if (' ' <= text_acc.ch && text_acc.ch <= 128)
		{	if (i < 99)
				number[i++] = text_acc.ch;
		}
		else
		{	PrintContext(frame_acc);
			printf("Fout(WS47): Vreemd karakter (#%02X) op %d in nummer", text_acc.ch, i);
		}
	}
	number[i] = '\0';
	if (debug_parse_wordstudy)
		fprintf(stderr, "\n number=|%s|", number);

	sprintf(context, "in woord '%s'", number);
	bool inTR;
	{	word nr;
		word romnr;
		parse_wordnr(number, nr, romnr, inTR);

		if (nr == 1065 && cur_number > nr)
		{	PrintContext(frame_acc);
			printf("Fout(WS48): Verander woordnummer 1065 in 1265.");
			nr = 1265;
		}			
		if (romnr > 1)
		{	if (nr != cur_number || romnr != cur_romnumber+1)
			{	PrintContext(frame_acc);
				printf("Fout(WS49): fout in volgorde van woordnummers %d!=%d of %d!=%d",
				       nr, cur_number, romnr, cur_romnumber+1);
			}
			cur_romnumber = romnr;
		}
		else
		{	cur_romnumber = 1;
			if (nr <= cur_number)
			{	PrintContext(frame_acc);
				printf("Fout(WS50): woordnummer niet oplopend (%d)", cur_number);
			}
		}
		cur_number = nr;
	}						
	
	char *s = context + strlen(context);

	if (!text_acc.ParseChar('\t'))
	{	PrintContext(frame_acc);
		printf("Fout(WS51): verwacht een tab na het woordnummer");
	}

	char greek[100];
	i = 0;
	for (; text_acc.More(); text_acc.Next())
	{	if (text_acc.is_newline || text_acc.ch == '\t')
			break;

		if (text_acc.font_family != 'G')
		{	PrintContext(frame_acc);
			printf("Fout(WS52): non-Greek font (%c) gebruikt voor karakter (#%02X) in Grieks", text_acc.font_family, text_acc.ch);
		}
		if (i < 99)
			greek[i++] = text_acc.ch;
	}
	greek[i] = '\0';
	
	if (debug_parse_wordstudy)
	{	fprintf(stderr, " greek=|");
		for (char *g = greek; *g; g++)
			fprintf(stderr, "%02X", (byte)*g);
		fprintf(stderr, "|");
	}

	if (!text_acc.ParseChar('\t'))
	{	PrintContext(frame_acc);
		printf("Fout(WS53): verwacht een tab na het Griekse woord");
	}
	
	char transl[100];
	i = 0;
	for (; text_acc.More(); text_acc.Next())
	{	if (text_acc.is_newline || text_acc.ch == '\t')
			break;

		if (text_acc.font_family != 'T')
		{	PrintContext(frame_acc);
			printf("Fout(WS54): non-Times font ('%c') in transliteratie", text_acc.font_family);
		}
		bool exclude_char = false;
		if (' ' <= text_acc.ch && text_acc.ch <= 128)
			/*printf("%c", text_acc.ch)*/;
		else if (text_acc.ch == 0xF8)
			/*printf("~")*/;
		else if (   text_acc.ch == (byte)'' 
				 || text_acc.ch == (byte)''
				 || text_acc.ch == (byte)'')
			/*printf("#%02X", text_acc.ch)*/;
		else
		{	PrintContext(frame_acc);
			printf("Fout(WS55): Vreemd karakter '%c' (#%02X) op %d in transliteratie", 
				   text_acc.ch, text_acc.ch, i);
			exclude_char = true;
		}
		if (!exclude_char && i < 99 && (i != 0 || text_acc.ch != ' '))
			transl[i++] = text_acc.ch;
	}
	transl[i] = '\0';
	
	if (debug_parse_wordstudy)
		fprintf(stderr, "transl=|%s|", transl);
	
	check_transl_with_greek("WS68","WS69","WS97", NULL, greek, transl, frame_acc);

	text_acc.SkipLine();
	while (text_acc.More() && text_acc.is_newline)
	{	if (debug_parse_ws_dump)
			printf("\n - (1) skip empty line '%s'", text_acc.par_style);
		text_acc.SkipLine();
	}

	if (build_ws)
	{	
		cur_greek_word = new CGreekWord(greek, transl, inTR);
		all_greek_words->addWord(cur_number, cur_greek_word);
	}
	else
	{
		cur_greek_word = all_greek_words->getWord(cur_number, cur_romnumber-1);
		add_recent_greek_word(cur_greek_word, cur_number, cur_romnumber);
		output_ws_word(cur_number, cur_romnumber, greek, transl, inTR);
	}
	
	// Skip the first paragraph because sometimes it has a strange par style
	if (   text_acc.More()
		&& strcmp(text_acc.par_style, "alinea1") != 0
	    && strcmp(text_acc.par_style, "alinea 1") != 0)
	{	PrintContext(frame_acc);
		printf("Fout(WS56): Eerste paragraaf is van type '%s'.", text_acc.par_style);
	}
	//if (text_acc.More())
	//{	printf("\n - (2) skip line '%s'", text_acc.par_style);
	//	text_acc.SkipLine();
	//}

	output_ws_start_description();

	while (   text_acc.More()
	       && (   strcmp(text_acc.par_style, "alinea1") == 0
	       	   || strcmp(text_acc.par_style, "alinea 1") == 0
	       	   || strcmp(text_acc.par_style, "normal") == 0
	       	   || strcmp(text_acc.par_style, "Normaal") == 0
	       	   || text_acc.ch == ' '
	       	   || text_acc.is_newline))
	{	
		if (debug_parse_ws_dump)
			printf("\n - 1: found '%s'", text_acc.par_style);
		while (text_acc.More() && text_acc.ch == ' ')
			text_acc.Next();
		
		if (   text_acc.StartsWith(".1")
		    || (text_acc.ch == '1' && (cur_number == 4001 || cur_number == 3973)))// && strcmp(text_acc.par_style, "alinea2") == 0)
			break;
			
		if (   !text_acc.is_newline
			&& strcmp(text_acc.par_style, "alinea1") != 0
		    && strcmp(text_acc.par_style, "alinea 1") != 0)
		{	PrintContext(frame_acc);
			printf("Fout(WS57): verwacht alinea1, maar '%s' gevonden.", text_acc.par_style);
		}

		if (!build_ws)
			parse_commentary(frame_acc, text_acc, true, 0);
		text_acc.SkipLine();
	}

	output_ws_end_description();

	if (   text_acc.More() && strcmp(text_acc.par_style, "alinea2") != 0
		&& !text_acc.is_newline)
	{	PrintContext(frame_acc);
		printf("Fout(WS58): verwacht alinea2, maar '%s' gevonden", text_acc.par_style);
	}
	word last_form_number = 0;
	nr_greek_word_forms = 0;

	if (debug_parse_ws_dump)
		printf("\n - 1b: found '%s' more: %d", text_acc.par_style, text_acc.More());
	
	while (   text_acc.More() 
	       && (   strcmp(text_acc.par_style, "alinea2") == 0
	           || text_acc.ch == '.' 
	           || (text_acc.ch == '1' && (cur_number == 4001 || cur_number == 3973))))
	{
		if (debug_parse_ws_dump)
			printf("\n - 2: found '%s'", text_acc.par_style);
		if (text_acc.ch == 11)
		{	text_acc.Next();
			if (text_acc.StartsWith("Openb."))
				break;
		}
		if (!text_acc.is_newline)
		{	parse_word_type(frame_acc, text_acc, last_form_number);
			if (!text_acc.is_newline)
			{	PrintContext(frame_acc);
				printf("Fout(WS59): rommel voor regelovergang |");
				
				while (!text_acc.is_newline)
				{	printf("%c", text_acc.ch);
					text_acc.Next();
				}
				printf("|\n");
			}
		}
		text_acc.SkipLine();
	}

	if (build_ws)
		cur_greek_word->addWordForms(nr_greek_word_forms, greek_word_forms);
	else
	{
		for (lword i = 0; i < nr_greek_word_forms; i++)
		{	CGreekWordForm *word_form = cur_greek_word->word_forms[i];	

#ifdef REMOVE_DOUBLE_WORD_FORMS
			if (word_form->as.IsNull())
#endif
			{
				output_ws_word_form_open(word_form->form_number,
									   word_form->as.IsNull() ? 0 : word_form->as->form_number,
									   word_form->greek,
									   word_form->transl, word_form->vert,
									   word_form->inTR);
#ifdef ANALYZE_VERT
				analyze_vert(word_form->vert);
#endif
				output_ws_word_form_close();
			}
		}
	}
		
	if (text_acc.More() && strcmp(text_acc.par_style, "kopwoord") == 0)
	{	if (show_parse_wordstudy_warnings)
		{	PrintContext(frame_acc);
			printf("Mededeling(WS60): Frequent gebruikt woord. Heeft geen lijst van referenties");
		}
	}
	nr_greek_verse_refs = 0;
	int nr_vr = 1;

	if (debug_parse_ws_dump)
		printf("\n - 2b: found '%s' more: %d", text_acc.par_style, text_acc.More());

	init_verse_refs();

	while (   text_acc.More() 
	       && (   strcmp(text_acc.par_style, "alinea3") == 0
	           || (   strcmp(text_acc.par_style, "alinea2") == 0
	               && text_acc.StartsWith("\13Open"))
	           || strcmp(text_acc.par_style, "alinea1") == 0)
	       && text_acc.ch != 11)
	{
		if (debug_parse_ws_dump)
			printf("\n - 3: found '%s'", text_acc.par_style);
		if (text_acc.More() && strcmp(text_acc.par_style, "alinea1") == 0)
		{	if (show_parse_wordstudy_warnings)
			{	PrintContext(frame_acc);
				printf("Mededeling(WS61): vond alinea1 paragraaf na alinea2 paragraaf.");
			}
			text_acc.SkipLine();
		}
		else
		{
			sprintf(s, ", versreferentie %d", nr_vr++);
			bool missing_newline = false;
			if (!text_acc.is_newline)
			{	parse_verse_ref(frame_acc, text_acc, missing_newline);
				while (text_acc.More() && text_acc.ch == ' ')
					text_acc.Next();
				if (!text_acc.is_newline)
				{	if (   ('A' <= text_acc.ch && text_acc.ch <= 'Z')
						|| text_acc.ch == '1'
						|| text_acc.ch == '2'
						|| (   text_acc.ch == '3' 
						    && strcmp(text_acc.par_style, "kopwoord") == 0)
				        || text_acc.ch == '\t')
					{	PrintContext(frame_acc);
						printf("Fout(WS62): ontbreekt hier een regelovergang???");
						missing_newline = true;
					}
					else
					{	PrintContext(frame_acc);
						printf("Fout(WS63): rommel voor regelovergang (%02X)|", text_acc.ch);
						while (!text_acc.is_newline)
						{	printf(' ' < text_acc.ch && text_acc.ch < 127 ? "%c" : "#%02X", text_acc.ch);
							text_acc.Next();
						}
						printf("|\n");
						return;
					}
				}
			}
			if (!missing_newline)				
				text_acc.SkipLine();
		}
	}
	
	if (build_ws)
	{	//fprintf(stderr, "\naddRefs(%d )", nr_greek_verse_refs);
		//for (int i = 0; i < nr_greek_verse_refs; i++)
		//{	unsigned int v = (unsigned int)greek_verse_refs[i];
		//	fprintf(stderr, "\n%d %p %c%c%c%c", i, v, (char)v, (char)(v >> 8), (char)(v >> 16), (char)(v >> 24));
		//}
		cur_greek_word->addRefs(nr_greek_verse_refs, greek_verse_refs);
	}
	else
	{
		produce_ws_text_ref_output();
	}

	if (   cur_greek_word->nr_word_forms > 1
		&& cur_greek_word->nr_verse_refs > 0)
	{	
		for (lword i = 0; i < cur_greek_word->nr_word_forms; i++)
		{	CGreekWordForm *greek_word_form = cur_greek_word->word_forms[i];
			if (greek_word_form->cnt_used_in_ref == 0)
			{	PrintContext(frame_acc);
				printf("Waarschuwing(WS64): vormnummer %d wordt niet gebruikt bij enige referentie",
					   greek_word_form->form_number);
			}
		}
		
		for (lword i = 0; i < cur_greek_word->nr_verse_refs; i++)
		{	CGreekVerseRef *greek_verse_ref = cur_greek_word->verse_refs[i];
			if (greek_verse_ref->form_numbers[0].nr == 0)
			{	
				PrintContext(frame_acc);
		 		printf("Fout(WS65): %s %d:%d referentie heeft geen vormnummer",
					   book_abrvs[greek_verse_ref->book_nr],
					   greek_verse_ref->chap_nr,
					   greek_verse_ref->verse_nr);
			}
		}
	}
	text_acc.SkipEmptyLines();
}

void parse_index_line(const char *text, const char *mode, CFrameAccessor &frame_acc)
{
	for (;;)
	{
		if (*text != '\t')
		{
			output_index_entry_open();
			for (; *text != '\0' && *text != '\t'; ++text, ++mode)
				output_index_entry_text_char(*text, *mode & 1);
			output_index_entry_text_close();
		}
		for (; *text == '\t'; text++);
		if (*text == '\0')
			return;
		
		output_index_entry_elem_open();
		if (*text == '(')
		{	output_index_entry_elem_indirect();
			text++;
		}
		if (startwith(text, "SV "))
		{	output_index_entry_elem_only_SV();
			text += 3;
		}
		if (*text == '*')
		{	output_index_entry_elem_part_of();
			text++;
		}
		for (; *text == ' '; text++);

		char transl[100];
		char *s = transl;
				
		for (; *text != '\0' && *text != '\t'; text++)
			if (*text != ' ' || text[1] != '\t')
				*s++ = *text;
		*s = '\0';
		
		char numbertext[100];
		s = numbertext;
		
		if (*text == '\t')
			text++;
		for (; *text == ' '; text++);
		for (; *text != '\0' && *text != '\t' && *text != ')'; text++)
			if (*text != ' ' || text[1] != '\t')
				*s++ = *text;
		*s = '\0';
		
		if (*text == ')')
			text++;

		WordNumber number;
		number.parse(numbertext, "");
		CGreekWord *greek_word = all_greek_words->getWord(number.word_nr, 
														  number.word_romnr - 1);
		if (greek_word == 0)
		{	
			int word_nr, word_romnr;
			if (find_greek_word_with_transl(transl, word_nr, word_romnr))
			{
				PrintContext(frame_acc);
				printf("Correctie(IN04): Nummer %s", numbertext);
				number.word_nr = (word)word_nr;
				number.word_romnr = (word)word_romnr;
				strcpy(numbertext, number.word_str());
				printf(" vervangen door %s", numbertext);					
			}
			else
			{
				PrintContext(frame_acc);
				printf("Fout(IN01): Geen woord voor nummer %s", numbertext);
			}
		}
		else if (streq(transl, greek_word->transl))
			; // They are the same!!
		else if (is_transl_with_greek_word(transl, greek_word))
		{	
			word transl_number_warning = 0;
			compare_transl(transl, greek_word->transl, transl_number_warning, 0);
			
			if (transl_number_warning != 0)
			{	PrintContext(frame_acc);
				printf("Correctie(IN02): Transliteratie %s vervangen door %s uit woordstudie. Deze verschilt wat betreft ",
					   transl_to_text(transl), 
					   transl_to_text(greek_word->transl));
				print_transl_diffs(transl_number_warning);
				strcpy(transl, greek_word->transl);
			}
		}
		else if (is_transl_with_greek_word_forms(transl, greek_word))
		{
			PrintContext(frame_acc);
			printf("Correctie(IN03): Transliteratie %s vervangen door %s uit woordstudie",
				   transl_to_text(transl), 
				   transl_to_text(greek_word->transl));
			strcpy(transl, greek_word->transl);
		}
		else
		{
			int word_nr, word_romnr;
			if (find_greek_word_with_transl(transl, word_nr, word_romnr))
			{
				PrintContext(frame_acc);
				printf("Correctie(IN04): Nummer %s", numbertext);
				number.word_nr = (word)word_nr;
				number.word_romnr = (word)word_romnr;
				strcpy(numbertext, number.word_str());
				printf(" vervangen door %s", numbertext);	
			}
			else
			{
				PrintContext(frame_acc);
				printf("Correctie(IN05): Transliteratie %s vervangen door %s uit woordstudie",
					   transl_to_text(transl), 
					   transl_to_text(greek_word->transl));
				strcpy(transl, greek_word->transl);
			}
		}
		output_index_entry_elem_greek(transl);
		output_index_entry_elem_number(numbertext);

		output_index_entry_elem_close();

		if (*text == '\t')
			text++;  // Two lines in one (happens twice)
		else
			break;
	}		
	if (*text != '\0')
	{	PrintContext(frame_acc);
		printf("Waarschuwing(IN06): Tekst fragment '%s' is overgeslagen\n", text);
	}
}

void parse_indexes(CFrameAccessor &frame_acc, CTextOnFramesAccessor &text_acc)
{
	strcpy(context, "indexen");
	output_start_indexes();
	
	int nr_con_empty = 0;

	const char* text;
	
	text_acc.SkipLine();
	text = text_acc.par_text;
	for (;;)
	{
		if (*text == '\0')
		{	//printf("Empty\n");
			if (++nr_con_empty > 20)
				break;
		}
		else
		{	nr_con_empty = 0;
			if (text[1] == '\0')
				; //printf("Start of %c\n", text[0]);
			else if (streq(text, "Index II"))
				break;
			else
				parse_index_line(text, text_acc.par_mode, frame_acc);
		}
		text_acc.SkipLine();
		text = text_acc.par_text;
	}
	
	for (;;)
	{	
		if (*text == '\0')
		{	//printf("Empty\n");
			if (++nr_con_empty > 20)
				break;
		}
		else 
		{	nr_con_empty = 0;
			if (streq("A", text))
				break;
		}
		text_acc.SkipLine();
		text = text_acc.par_text;
	}
	
	for (;;)
	{
		if (   *text == '\0' 
			|| (text[0] == '\t' && text[1] != '\0' && text[2] == '\0'))
		{	//printf("Empty\n");
			if (++nr_con_empty > 20)
				break;
		}
		else
		{	nr_con_empty = 0;
			if (text[1] == '\0')
				; //printf("Start of %c\n", text[0]);
			else
				parse_index_line(text, text_acc.par_mode, frame_acc);
		}
		text_acc.SkipLine();
		text = text_acc.par_text;
	}
	//if (nr_con_empty <= 20)
	//	printf("End with %s\n", text_acc.par_text);
		
	output_end_indexes();
}

void parse_word_studies(CFrameAccessor &frame_acc, bool indexes = false)
{
	fprintf(stderr, "\nVerwerk: %s", frame_acc.book_name);
	sprintf(context, "aan het begin van het boek");
	
	CTextOnFramesAccessor text_acc(frame_acc);
	text_acc.Init();
	
	bool in_ws = false;
	
	while (text_acc.More())
	{
		// printf("\n |%s|", text_acc.par_style ? text_acc.par_style : "<unknown>");
		
		if (strcmp(text_acc.par_style, "kopwoord") == 0 && !text_acc.is_newline)
		{	output_rest_bar();
			parse_one_word_study(frame_acc, text_acc);
			in_ws = true;
		}
		else if (indexes //&& strcmp(text_acc.par_style, "par") == 0
				 && text_acc.par_text[0] == 'a' && text_acc.par_text[1] == '\0')
		{	output_rest_bar();
			parse_indexes(frame_acc, text_acc);
		}
		else
		{	
			if (in_ws && debug_parse_ws_dump)
				printf("\n - %s", text_acc.par_style);
			output_rest(text_acc);
			text_acc.SkipLine();
		}		
		
		//if (par_koptekst && max_i++ == 100)
		//	return;
	}
	frame_acc.ReportError();
}

bool alles = false;
bool wsopnieuw = false;

bool has_arg(char *arg, int argc, char *argv[])
{
	if (alles)
		return true;
	
	for (int i = 1; i < argc; i++)
		if (streq(argv[i], arg))
			return true;
	return false;
}

int main(int argc, char *argv[])
{
	greek_to_output = greek_to_greek;
	//f2 = fopen("oo","w");
	//bool parse_ws = argc == 1;
	for (int i = 1; i < argc; i++)
		if (streq(argv[i], "-wC1"))
			show_col1_warning = true;
		else if (streq(argv[i], "-wTC"))
			show_parse_greek_warnings = true;
		else if (streq(argv[i], "-wC2"))
			show_parse_col2_warnings = true;
		else if (streq(argv[i], "-wGW"))
			show_match_greek_warnings = true;
		else if (streq(argv[i], "-dWS"))
			debug_parse_wordstudy = true;
		else if (streq(argv[i], "-wWS"))
			show_parse_wordstudy_warnings = true;
		else if (streq(argv[i], "-wTL"))
			show_transliteration_warnings = true;
		else if (streq(argv[i], "-oRG"))
			greek_to_output = greek_to_text;
		else if (streq(argv[i], "-oGH"))
			greek_to_output = greek_to_hexadecimal;
		else if (streq(argv[i], "-oSC"))
			opt_skip_commentary = true;
		else if (argv[i][0] == '-')
			fprintf(stderr, "Onbekende optie '%s'; geldig zijn: -wC1 -wTC -wC2 -wGW -wTL\n", argv[1]);
		else if (streq(argv[i], "ALLES"))
			alles = true;
		else if (streq(argv[i], "WSOPNIEUW"))
			wsopnieuw = true;
		else if (streq(argv[i], "2")
			|| streq(argv[i], "3")
			|| streq(argv[i], "4")
			|| streq(argv[i], "5")
			|| streq(argv[i], "6")
			|| streq(argv[i], "7A")
			|| streq(argv[i], "7B")
			|| streq(argv[i], "8")
			|| streq(argv[i], "9")
			|| streq(argv[i], "10")
			|| streq(argv[i], "11")
			|| streq(argv[i], "12")
			|| streq(argv[i], "13")
			|| streq(argv[i], "14")
			|| streq(argv[i], "15")
			|| streq(argv[i], "16"))
			;
		else
			fprintf(stderr, "Onbekend boekdeel: %s\n", argv[i]);

	output_rest_open();

#ifdef READ_FROM_DATABASE	
	// Open a new database file
	if (!mmfile.Open("tmp/studybible.dat", 400000000))
		return 0;

	// Retrieve root object
	CDatabase *database = (CDatabase*)mmfile.FirstObject();
	CFrameAccessor frame_acc(database->all_books);
#else
	CFrameAccessor frame_acc;
#endif

	// Open second database file
	if (!mmfile2.Open("tmp/studybible2.dat", 200000000))
		return 0;

	// Retrieve root object
	CDatabase2 *database2 = (CDatabase2*)mmfile2.FirstObject();
	if (database2 == NULL || wsopnieuw)
	{	
		wsopnieuw = true;
		
		mmfile2.Reset();
		database2 = new CDatabase2;
	}
	if (database2->all_greek_words.IsNull())
	{
		wsopnieuw = true;
		
		database2->all_greek_words = new CAllGreekWords;

		all_greek_words = database2->all_greek_words;

		build_ws = true;

		frame_acc.Open("DEEL11");
		parse_word_studies(frame_acc);
		frame_acc.Open("DEEL12");
		parse_word_studies(frame_acc);	
		frame_acc.Open("DEEL13");
		parse_word_studies(frame_acc);	
		frame_acc.Open("DEEL14");
		parse_word_studies(frame_acc);	
		frame_acc.Open("DEEL15");
		parse_word_studies(frame_acc);	
		frame_acc.Open("DEEL16");
		parse_word_studies(frame_acc, true);	
		
		build_ws = false;
	}
	all_greek_words = database2->all_greek_words;

	output_start_bible_books();

	cur_book_nr = -1;
	if (has_arg("2", argc, argv))
	{	frame_acc.Open("DEEL02");
		parse_comm(frame_acc);
	}
	if (has_arg("3", argc, argv))
	{	frame_acc.Open("DEEL03");
		parse_comm(frame_acc);
	}
	if (has_arg("4", argc, argv))
	{	frame_acc.Open("DEEL04");
		parse_comm(frame_acc);
	}
	if (has_arg("5", argc, argv))
	{	frame_acc.Open("DEEL05");
		parse_comm(frame_acc);
	}
	if (has_arg("6", argc, argv))
	{	frame_acc.Open("DEEL06");
		parse_comm(frame_acc);
	}
	if (has_arg("7A", argc, argv))
	{	frame_acc.Open("DEEL07A");
		parse_comm(frame_acc);
	}
	if (has_arg("7B", argc, argv))
	{	frame_acc.Open("DEEL07B");
		parse_comm(frame_acc);
	}
	if (has_arg("8", argc, argv))
	{	frame_acc.Open("DEEL08");
		parse_comm(frame_acc);
	}
	if (has_arg("9", argc, argv))
	{	frame_acc.Open("DEEL09");
		parse_comm(frame_acc);
	}
	if (has_arg("10", argc, argv))
	{	frame_acc.Open("DEEL10");
		parse_comm(frame_acc);
	}
	output_end_bible_books();	

	if (!wsopnieuw)
	{
		output_start_word_study_books();
		
		if (has_arg("11", argc, argv))
		{	frame_acc.Open("DEEL11");
			parse_word_studies(frame_acc);
		}
		if (has_arg("12", argc, argv))
		{	frame_acc.Open("DEEL12");
			parse_word_studies(frame_acc);
		}
		if (has_arg("13", argc, argv))
		{	frame_acc.Open("DEEL13");
			parse_word_studies(frame_acc);
		}
		if (has_arg("14", argc, argv))
		{	frame_acc.Open("DEEL14");
			parse_word_studies(frame_acc);
		}
		if (has_arg("15", argc, argv))
		{	frame_acc.Open("DEEL15");
			parse_word_studies(frame_acc);
		}
		if (has_arg("16", argc, argv))
		{	frame_acc.Open("DEEL16");
			parse_word_studies(frame_acc, true);
		}
		
		output_end_word_study_books();
	}

#ifdef READ_FROM_DATABASE	
	mmfile.Close();
#endif

	mmfile2.Close();
	fprintf(stderr, "\n\n");
	printf("\n");
	
	for (int i = 0; i < 10; i++)
		printf("\n %2d %5ld", i, cgw[i]);
	
	{
		for (int i = 0; i < nr_manuscripts; i++)
		{
			printf("%8s %4d%c %4d%c %4d%c %4d%c\n",
				manuscripts[i].name,
				manuscripts[i].nr_B,
				manuscripts[i].type == 'B' ? '*' : ' ',
				manuscripts[i].nr_E,
				manuscripts[i].type == 'E' ? '*' : ' ',
				manuscripts[i].nr_O,
				manuscripts[i].type == 'O' ? '*' : ' ',
				manuscripts[i].nr_U,
				manuscripts[i].type == ' ' ? '*' : ' ');
		}
	}
	
	output_rest_close();

	return 0;
}
