class AbstractTextAccessor
{
public:
	AbstractTextAccessor() { init(); }
	virtual ~AbstractTextAccessor() { close(); }
	bool more;	
	char ch;       // invalid if more is false
	lword line_nr;
	lword pos;
	
	inline void init()
	{	more = false;
		ch = '\0';
		line_nr = 1;
		pos = 1;
	}
	
	virtual void next() = 0;
	virtual void close() {}
};

class FileTextAccessor : public AbstractTextAccessor
{
public:
	FileTextAccessor() : f(0) {}
	virtual ~FileTextAccessor() {}
	
	bool open(char *filename)
	{	close();
		f = fopen(filename, "rt");
		if (f == 0)
			return false;
		more = true;
		line_nr = 1;
		pos = 1;
		ch = fgetc(f);
		if (feof(f))
			close();
		return true;
	}
	
	void next()
	{	if (!more)
			return;
		if (ch == '\n')
		{	line_nr++;
			pos = 1;
		}
		else
			pos++;
		ch = fgetc(f);
		if (feof(f))
			close();
	}
		
	void close()
	{	if (f != 0)
		{	fclose(f);
			f = 0;
		}
		init();
	}
	
private:
	FILE *f;
};

class BufferTextAccessor : public AbstractTextAccessor
{
public:
	BufferTextAccessor() : text(0) {}
	virtual ~BufferTextAccessor() {}
	
	void open(char *new_text)
	{	close();
		if (new_text == 0)
			return;
		text = new_text;
		line_nr = 1;
		pos = 1;
		ch = *text++;
		if (ch == '\0')
			close();
	}
	
	void next()
	{	if (!more)
			return;
		if (ch == '\n')
		{	line_nr++;
			pos = 1;
		}
		else
			pos++;
		ch = *text++;
		if (ch == '\0')
			close();
	}
		
	void close()
	{	text = 0;
		more = false;
	}
	
private:
	const char *text;
};

